<?php
require 'afw_hub_credentials.php';
date_default_timezone_set('America/New_York');
error_reporting(E_ALL);
//
class CONSTANTS {
    const GEN_SQL_DEFAULTS = [
        'cmd' => 'SELECT',
        'table' => null,
        'allowed_vars' => [],
        'union' => [],
        'sub_query' => [],
        'cols' => ['*'],
        'vals' => [],
        'val_exprs' => [],
        'where' => [],
        'inner_join' => [],
        'left_join' => [],
        'right_join' => [],
        'having' => [],
        'group_by' => null,
        'order_by' => [],
        'limit' => []
    ];
}
//
$forbidden_str  = '<h1>403 - Forbidden Page</h1>';
$forbidden_str .= '<br> You do not have permission to access this page.';
$forbidden_str .= '<br><br>';
$forbidden_str .= 'If you believe this is in error contact your system administrator.';
//
///////////////////////////////////////////////////////////////////////////////
//////////////////////////////// General Functions ////////////////////////////
//
// links the style sheet and javascript files based on the current page
function link_external_files($page_root='home_page') {
    //
    // style sheets
    echo '<link rel="stylesheet" type="text/css" encoding="utf-8" href="css/operations_tracking.css">';
    //
    // utility JS file inclusions
    echo '<script src="js/scripts_utility.js"></script>';
    echo '<script src="js/scripts_operations_tracking.js"></script>';
    echo '<script src="js/scripts_utility_forms.js"></script>';
    echo '<script src="js/scripts_utility_tables.js"></script>';
    //
    // page specific JS file inclusions
    if (file_exists('js/scripts_'.$page_root.'_forms.js')) {
        echo '<script src="js/scripts_'.$page_root.'_forms.js"></script>';
    }
    if (file_exists('js/scripts_'.$page_root.'.js')) {
        echo '<script src="js/scripts_'.$page_root.'.js"></script>';
    }
}
//
// adds hidden forms used to logout the user
function page_head() {
    echo '<div id="hidden-forms" class="hidden-elm">';
    echo '<form id="invalid_login" name="invalid-login" method="POST" action="login.php">';
    echo '<input type="hidden" name="invalid-login" value="GTFO-HERE">';
    echo '</form>';
    echo '<form id="logout" name="logout" method="POST" action="login.php">';
    echo '<input type="hidden" name="logout" value="BBL">';
    echo '</form>';
    //
    // non-sensitive user data
    echo '<input id="user-username" type="hidden" value="'.$_SESSION['username'].'">';
    echo '</div>';
}
//
// checks the user login and prepares the page
function check_login($page_root) {
    global $forbidden_str, $misc_database;
    $conn = connect_db($misc_database);
    //
    if (isset($_SESSION['username'])) {
        echo '<span id="welcome-line" class="welcome-line">';
        echo '<button id="home_page" type="button" style="float: left;" onclick="location.href=\'home_page.php\'">Return to Home Page</button>';
        echo "Welcome {$_SESSION["dbuser_first_name"]} {$_SESSION["dbuser_last_name"]}!";
        echo '&nbsp;&nbsp;&nbsp;';
        echo '<input id="logout-button" type="button" value="Logout" onclick="document.getElementById(\'logout\').submit();"></span>';
        echo '<br>';
        //
        if (($page_root == 'home_page') and check_perms($_SESSION['permissions'], 'table_maintenance')) {
            //
            $sql = "SELECT count(*) FROM `suggestions` WHERE `suggestion_status` LIKE 'unread'";
            list($num_unread, $err) = fetch_db(['statement' => $sql, 'params' => []], $conn);
            $sql = "SELECT count(*) FROM `suggestions` WHERE `suggestion_status` LIKE 'flagged'";
            list($num_flagged, $err) = fetch_db(['statement' => $sql, 'params' => []], $conn);
            $conn = null;
            //
            $num_unread = $num_unread[0]['count(*)'];
            $num_flagged = $num_flagged[0]['count(*)'];
            $flag_style = 'color: orange;';
            $unread_style = 'color: red;';
            if ($num_flagged == 0) { $flag_style = 'color: green;';}
            if ($num_unread  == 0) { $unread_style = 'color: green;';}
            //
            echo '<span id="admin-messages" class="welcome-line">';
            echo '<label style="text-align:left; min-width:initial; font-weight: bold; margin-right:1.5em;">Suggestions:</label>';
            echo "<a style=\"{$flag_style}\" onclick=\"view_suggestions('flagged');\">{$num_flagged} Flagged</a>&nbsp;&nbsp;&nbsp;&nbsp;";
            echo "<a style=\"{$unread_style}\" onclick=\"view_suggestions('unread');\">{$num_unread} Unread</a>&nbsp;&nbsp;&nbsp;&nbsp;";
            echo '</span>';
        }
        //
        echo '<span id="suggestion-line" class="welcome-line">';
        echo '<a id="leave_suggestion" onclick="enter_suggestion();">Leave a Suggestion!</a>&nbsp;&nbsp;&nbsp;&nbsp;';
        echo '</span>';
    }
    else {
        echo "<script>document.getElementById('invalid_login').submit();</script>";
        session_destroy();
    }
    // checking user perms for the page
    $access = check_perms($_SESSION['permissions'],$page_root);
    if (!$access) {
        echo '<span id="forbidden-page" class="forbidden-page">'.$forbidden_str.'</span>';
        die();
    }
}
//
// checks user access to a page against their permissions
function check_perms($user_perm_str, $page_str) {
    //
    $m = preg_match($_SESSION['user_perm_pattern'],$page_str);
    //
    if ($page_str == 'leave_suggestion') {
        $m = 1;
    }
    //
    if ($m === 1) {
        return true;
    }
    else {
        return false;
    }
}
//
// fetches auth_table data for use in modify page
function fetch_auth_table_data($data_where) {
    //
    $args = _fetch_auth_table_data($data_where);
    //
    // fetching meta_data
    $sql = gen_prepared_sql($args['cols_sql_args']);
    $cols = fetch_db($sql, $args['connection'])[0];
    //
    $sql = gen_prepared_sql($args['meta_sql_args']);
    $meta_data = fetch_db($sql, $args['connection'])[0];
    //
    // fetching table data SQL statement
    for ($i = 0; $i < count($cols); $i++) {
        array_push($args['data_sql_args']['cols'], $cols[$i]['column_name']);
    }
    $sql = gen_prepared_sql($args['data_sql_args']);
    $users_data = fetch_db($sql, $args['connection'])[0];
    //
    $args['connection'] = null;
    return ['data' => $users_data, 'meta_data' => $meta_data];
}
//
// performs basic python style format replacements
function format($msg, $vars) {
    // http://stackoverflow.com/a/17372566
    $vars = (array)$vars;
    $msg = preg_replace_callback('#\{\}#', function($r){
        static $i = 0;
        return '{'.($i++).'}';
    }, $msg);
    //
    return str_replace(
        array_map(function($k) {
            return '{'.$k.'}';
        }, array_keys($vars)),
        array_values($vars),
        $msg
    );
}
//
// recursively "pretty prints" an array
function print_array($key, $value=null, $indent=0) {
    if (is_array($key)) {
        $value = $key;
        $key = null;
    }
    if (!is_array($value)) {
        echo str_repeat('&nbsp;', $indent), $key, ' => ', $value, ',<br>';
        return;
    }
    //
    echo str_repeat('&nbsp;', $indent);
    echo is_null($key) ? '[ <br>' :  $key.' => [ <br>';
    //
    foreach($value as $k => $v) {
        print_array($k, $v, $indent + 4);
    }
    echo str_repeat('&nbsp;', $indent);
    echo $indent === 0 ? '] <br>' : '], <br>';
}
//
///////////////////////////////////////////////////////////////////////////////
////////////////////////////// Table Generation  //////////////////////////////
//
// builds a sql_args array to fetch all required table meta data
function build_html_table_meta_sql_args($table_name, $page, $cols) {
    //
    // fetching general table meta data to construct table with
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            '`column_name`', '`column_type`', '`data_type`', '`db_table_col`', '`total_type`',
            'COALESCE(table_meta_data.display_name, column_meta_data.display_name) AS `display_name`'

        ],
        'table' => 'column_meta_data',
        'sub_query' => [
            '%full_column_name%' => [
                'cmd' => 'SELECT',
                'cols' => [
                    "COALESCE(CONCAT(`in_database`, '.'), '') AS `db`",
                    "COALESCE(CONCAT(`in_table`, '.'), '') AS `tbl`",
                    'CONCAT((SELECT `db`), (SELECT `tbl`), `column_name`) AS `db_table_col`',
                    'internal_id'
                ],
                'table' => 'column_meta_data',
            ]
        ],
        'inner_join' => [
            ['%full_column_name% AS `full_column_name`',
             'column_meta_data.internal_id',
             'full_column_name.internal_id'],
            ['table_meta_data',
             'column_meta_data.internal_id',
             'table_meta_data.column_internal_id']
        ],
        'where' => [
            ['table_meta_data.column_usage', 'IN', ['DISPLAY', 'ALWAYS']],
            ['table_meta_data.html_table', 'LIKE', $table_name],
            ['table_meta_data.use_on_pages', 'REGEXP', "(^|%){$page}(%|$)"]
        ],
        'order_by' => [
            ['table_meta_data.display_order', 'ASC'],
            ['display_name', 'ASC'],
            ['column_meta_data.column_name', 'ASC']
        ]
    ];
    if ($cols) {
        array_push($sql_args['where'], ['`db_table_col`', 'IN', $cols]);
    }
    //
    return $sql_args;
}
//
// determines tables to link to
function determine_html_table_data_links($table_name, $page, $cols) {
    //
    global $meta_database;
    $conn = connect_db($meta_database);
    //
    // performing query to set primary table
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            "COALESCE(CONCAT(column_meta_data.in_database, '.'), '') AS `db`",
            'CONCAT((SELECT `db`), `in_table`) AS `database_table`',
            "CONCAT((SELECT `database_table`), '.', `column_name`) AS `db_table_col`",
        ],
        'table' => 'column_meta_data',
        'inner_join' => [
            ['table_meta_data',
             'column_meta_data.internal_id',
             'table_meta_data.column_internal_id']
        ],
        'where' => [
            ['table_meta_data.html_table', 'LIKE', $table_name],
            ['table_meta_data.linking_column', 'LIKE', 'MASTER'],
        ]
    ];
    $sql = gen_prepared_sql($sql_args);
    $primary_table = fetch_db($sql, $conn)[0];
    if (count($primary_table) !== 1) {
        $msg = 'One MASTER table must be specfied for each html_table and page combination';
        throw new Exception($msg);
    }
    $primary_table = $primary_table[0];
    //
    // performing query to determine any table relations
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            "COALESCE(CONCAT(column_meta_data.in_database, '.'), '') AS `db`",
            'CONCAT((SELECT `db`), `in_table`) AS `database_table`',
            '`column_name`',
            '`linking_column`',
            '`linking_column_name`',
            '`subquery_generated`'
        ],
        'table' => 'column_meta_data',
        'inner_join' => [
            ['table_meta_data',
             'column_meta_data.internal_id',
             'table_meta_data.column_internal_id']
        ],
        'where' => [
            ['table_meta_data.html_table', 'LIKE', $table_name],
            ['table_meta_data.use_on_pages', 'REGEXP', "(^|%){$page}(%|$)"],
            ['table_meta_data.linking_column', 'IS NOT NULL'],
            ['table_meta_data.linking_column', 'NOT LIKE', 'MASTER']
        ]
    ];
    $sql = gen_prepared_sql($sql_args);
    $table_relations = fetch_db($sql, $conn)[0];
    //
    // retrieving all column names used in a query to avoid "SELECT *"s
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            '`database_table`',
            '`db_table_col`',
            '`column_name`',
            '`column_type`',
            'COALESCE(table_meta_data.display_expr, column_meta_data.display_expr) AS `display_expr`',
            '`display_order`',
            '`query_order`'
        ],
        'table' => 'column_meta_data',
        'inner_join' => [
            ['%table_joins% AS `table_joins`',
             'column_meta_data.internal_id',
             'table_joins.internal_id']
        ],
        'left_join' => [
            ['table_meta_data',
             'column_meta_data.internal_id',
             'table_meta_data.column_internal_id']
        ],
        'sub_query' => [
            '%table_joins%' => [
                'cmd' => 'SELECT',
                'cols' => [
                    "COALESCE(CONCAT(`in_database`, '.'), '') AS `db`",
                    "COALESCE(CONCAT(`in_table`, '.'), '') AS `tbl`",
                    'CONCAT((SELECT `db`), `in_table`) AS `database_table`', //can be NULL
                    'CONCAT((SELECT `db`), (SELECT `tbl`), `column_name`) AS `db_table_col`',
                    'internal_id'
                ],
                'table' => 'column_meta_data',
            ]
        ],
        'where' => [
            ['table_meta_data.html_table', 'LIKE', $table_name],
            ['table_meta_data.use_on_pages', 'REGEXP', "(^|%){$page}(%|$)"],
            ['table_meta_data.column_usage', 'IS NOT NULL'],
            ['`db_table_col`', 'IN', $cols]
        ],
        'order_by' => [
            ['query_order', 'ASC'],
            ['display_order', 'ASC'],
            ['column_name', 'ASC']
        ]
    ];
    // adding union statements to catch columns that are set to always be returned
    $unions = [$sql_args];
    $unions[0]['where'] = [
        ['table_meta_data.html_table', 'LIKE', $table_name],
        ['table_meta_data.use_on_pages', 'REGEXP', "(^|%){$page}(%|$)"],
        ['table_meta_data.column_usage', 'IN', ['DATA', 'ALWAYS']],
    ];
    $sql_args['union'] = $unions;
    //
    $sql = gen_prepared_sql($sql_args);
    $column_names = fetch_db($sql, $conn)[0];
    //
    return ['primary_table' => $primary_table,
            'table_relations' => $table_relations,
            'column_names' => $column_names];
}
//
// builds the data sql
function build_html_table_data_sql_args($table_name, $page, $cols, $data_sql_args) {
    //
    // getting table relational data
    $relation_data = determine_html_table_data_links($table_name, $page, $cols);
    $sql_args = build_relational_sql_args($relation_data, $data_sql_args);
    //
    // returning merged sql_args array
    return array_merge($data_sql_args, $sql_args);
}
//
//
///////////////////////////////////////////////////////////////////////////////
//////////////////////// Form Manipulation Functions //////////////////////////
//
// builds a sql_args array to fetch all required form meta data
function build_form_meta_sql_args($form_name, $init_sql_args) {
    //
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            '`column_meta_data`.`in_database`',
            '`column_meta_data`.`in_table`',
            '`column_meta_data`.`column_name`',
            '`column_meta_data`.`display_name`',
            '`column_meta_data`.`column_type`',
            '`column_meta_data`.`data_type`',
            '`form_meta_data`.`submission_expr`',
            '`form_meta_data`.`submission_order`',
            '`form_meta_data`.`display_expr`',
            '`db_table_col`',
        ],
        'table' => 'column_meta_data',
        'sub_query' => [
            '%full_column_name%' => [
                'cmd' => 'SELECT',
                'cols' => [
                    "COALESCE(CONCAT(`in_database`, '.'), '') AS `db`",
                    "COALESCE(CONCAT(`in_table`, '.'), '') AS `tbl`",
                    'CONCAT((SELECT `db`), (SELECT `tbl`), `column_name`) AS `db_table_col`',
                    'internal_id'
                ],
                'table' => 'column_meta_data',
            ]
        ],
        'inner_join' => [
            ['%full_column_name% AS `full_column_name`',
             'column_meta_data.internal_id',
             'full_column_name.internal_id'],
            ['form_meta_data',
             'column_meta_data.internal_id',
             'form_meta_data.column_internal_id']
        ],
        'where' => [
            ['form_meta_data.form_name', 'LIKE', $form_name],
        ],
        'order_by' => [
            ['form_meta_data.submission_order', 'ASC'],
            ['column_meta_data.column_name', 'ASC']
        ]
    ];
    //
    return array_merge_recursive($init_sql_args, $sql_args);
}
//
// determines what tables to link to
function determine_form_data_links($form_name) {
    //
    global $meta_database;
    $conn = connect_db($meta_database);
    //
    // performing query to set primary table
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            "COALESCE(CONCAT(column_meta_data.in_database, '.'), '') AS `db`",
            'CONCAT((SELECT `db`), `in_table`) AS `database_table`',
            "CONCAT((SELECT `database_table`), '.', `column_name`) AS `db_table_col`",
            '`column_meta_data`.`column_name`'
        ],
        'table' => 'column_meta_data',
        'inner_join' => [
            ['form_meta_data',
             'column_meta_data.internal_id',
             'form_meta_data.column_internal_id']
        ],
        'where' => [
            ['form_meta_data.form_name', 'LIKE', $form_name],
            ['form_meta_data.linking_column', 'LIKE', 'MASTER'],
        ]
    ];
    $sql = gen_prepared_sql($sql_args);
    $primary_table = fetch_db($sql, $conn)[0];
    if (count($primary_table) !== 1) {
        $msg = 'One MASTER table must be specfied for each form combination';
        throw new Exception($msg);
    }
    $primary_table = $primary_table[0];
    //
    // performing query to determine any table relations
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            "COALESCE(CONCAT(`column_meta_data`.`in_database`, '.'), '') AS `db`",
            'CONCAT((SELECT `db`), `in_table`) AS `database_table`',
            '`column_meta_data`.`column_name`',
            '`form_meta_data`.`linking_column`',
            '`form_meta_data`.`linking_column_name`',
            '`column_meta_data`.`subquery_generated`'
        ],
        'table' => 'column_meta_data',
        'inner_join' => [
            ['form_meta_data',
             'column_meta_data.internal_id',
             'form_meta_data.column_internal_id']
        ],
        'where' => [
            ['`column_meta_data`.`column_type`', 'NOT REGEXP', 'set_statement'],
            ['`form_meta_data`.`form_name`', 'LIKE', $form_name],
            ['`form_meta_data`.`linking_column`', 'IS NOT NULL'],
            ['`form_meta_data`.`linking_column`', 'NOT LIKE', 'MASTER']
        ]
    ];
    $sql = gen_prepared_sql($sql_args);
    $table_relations = fetch_db($sql, $conn)[0];
    //
    // retrieving all column names used in a query to avoid "SELECT *"s
    $db_tables = [$primary_table['database_table']];
    $db_tables = array_merge($db_tables, array_column($table_relations, 'database_table'));
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [
            '`database_table`',
            '`db_table_col`',
            '`column_meta_data`.`column_name`',
            '`column_meta_data`.`column_type`',
            '`form_meta_data`.`display_expr`',
            '`form_meta_data`.`submission_order`'
        ],
        'table' => 'column_meta_data',
        'inner_join' => [
            ['%table_joins% AS `table_joins`',
             '`column_meta_data`.`internal_id`',
             '`table_joins`.`internal_id`']
        ],
        'left_join' => [
            ['`form_meta_data`',
             '`column_meta_data`.`internal_id`',
             '`form_meta_data`.`column_internal_id`']
        ],
        'sub_query' => [
            '%table_joins%' => [
                'cmd' => 'SELECT',
                'cols' => [
                    "COALESCE(CONCAT(`in_database`, '.'), '') AS `db`",
                    "COALESCE(CONCAT(`in_table`, '.'), '') AS `tbl`",
                    'CONCAT((SELECT `db`), `in_table`) AS `database_table`', //can be NULL
                    'CONCAT((SELECT `db`), (SELECT `tbl`), `column_name`) AS `db_table_col`',
                    'internal_id'
                ],
                'table' => '`column_meta_data`',
            ]
        ],
        'where' => [
            ['`database_table`', 'IN', $db_tables]
        ],
        'order_by' => [
            ['`submission_order`', 'ASC'],
            ['`column_name`', 'ASC'],
        ]
    ];
    // adding union statements to catch purely SQL generated columns
    $unions = [$sql_args];
    $unions[0]['where'] = [
        ['form_meta_data.form_name', 'LIKE', $form_name],
        ['column_meta_data.in_database', 'IS NULL'],
        ['column_meta_data.in_table', 'IS NULL'],
    ];
    $sql_args['union'] = $unions;
    //
    $sql = gen_prepared_sql($sql_args);
    $column_names = fetch_db($sql, $conn)[0];
    //
    return ['primary_table' => $primary_table,
            'table_relations' => $table_relations,
            'column_names' => $column_names];
}
//
// builds the sql_args array to query the database for form data
function build_form_data_sql_args($form_name, $data_sql_args) {
    //
    // getting table relational data
    $relation_data = determine_form_data_links($form_name);
    $sql_args = build_relational_sql_args($relation_data, $data_sql_args);
    //
    // returning merged sql_args array
    return array_merge($data_sql_args, $sql_args);
}
//
// handles setting up sql statements to submit an array of form values
function process_form_data_array($form_name, $form_val_array, $insert_all) {
    global $meta_database;
    //
    // pulling form meta data from database to setup sql
    $sql_args = ['where' => [['form_meta_data.submit_value', 'LIKE', 1]]];
    $sql_args = build_form_meta_sql_args($form_name, $sql_args);
    //
    // fetching meta data about the form
    $sql = gen_prepared_sql($sql_args);
    $conn = connect_db($meta_database);
    $meta_data = fetch_db($sql, $conn)[0];
    $relation_data = determine_form_data_links($form_name);
    //
    // setting up bulk of SQL statements grouped by database
    $main_sql_groups = [];
    $allowed_vars = [];
    foreach ($form_val_array as $form_values) {
        $meta_groups = setup_meta_data_groups($form_values, $meta_data, $allowed_vars);
        $sql_groups = generate_form_sql($meta_groups, $allowed_vars, $insert_all);
        foreach ($sql_groups as $database => $group) {
            if (isset($main_sql_groups[$database])) {
                $group = array_merge($main_sql_groups[$database], $group);

            }
            $main_sql_groups[$database] = $group;
        }
    }
    //
    return $main_sql_groups;
}
//
// sets up the meta data groups for form data submission
function setup_meta_data_groups($form_values, $meta_data, &$allowed_vars) {
    //
    // setting up array to ensure all form values are used
    $unused_values = [];
    foreach ($form_values as $key => $value) {
        $unused_values[$key] = true;
    }
    //
    // setting up meta data groups indexed by [database][table]
    $meta_data_groups = [];
    foreach ($meta_data as $col) {
        $database = $col['in_database'];
        $table = $col['in_table'];
        $name = $col['column_name'];
        //
        // initializing meta data database group if needed
        if (!isset($meta_data_groups[$database])) {
            $meta_data_groups[$database] = [];
        }
        //
        // special logic for handling set statments
        if (preg_match('/set_statement/i', $col['column_type'])) {
            $pat = '/(^@[A-z_]+)=([A-z1-9_()]+$)/';
            if (!preg_match($pat, $col['submission_expr'])) {
                throw new Exception("Invalid SET SQL: {$col['submission_expr']}");
            }
            $table = "{$name}-{$col['submission_order']}";
            preg_match($pat, $col['submission_expr'], $params);
            //
            // adding set statement to args if needed
            if (isset($form_values[$name])) {
                array_push($allowed_vars, $params[1]);
                $meta_data_groups[$database][$table] = [
                    'set_statment' => true,
                    'sql_args' => [
                        'cmd' => 'SET',
                        'variable_name' => $params[1],
                        'variable_value' => $params[2]
                    ]
                ];
            }
            $unused_values[$name] = false;
            continue;
        }
        //
        // initializing meta data table group if needed
        if (!isset($meta_data_groups[$database][$table])) {
            $group = ['meta_data' => [],
                      'form_values' => [],
                      'primary_index' => null];
            $meta_data_groups[$database][$table] = $group;
        }
        $group =& $meta_data_groups[$database][$table];
        //
        // adding form value to group
        if (array_key_exists($name, $form_values)) {
            $unused_values[$name] = false;
            $group['form_values'][$name] = $form_values[$name];
        }
        //
        // adding column meta data to group or setting primary index
        if (preg_match('/primary/i', $col['column_type'])) {
            $group['primary_index'] = $col;
        }
        $group['meta_data'][$name] = $col;
        //
        unset($group);
    }
    //
    // checking for unused values
    foreach ($unused_values as $key => $value) {
        if ($value) {
            throw new Exception($key.' was not used in data submission!');
        }
    }
    //
    return $meta_data_groups;
}
//
// generates table specfic sql statements based on meta data groups
function generate_form_sql($meta_data_groups, $allowed_vars, $insert_all) {
    //
    // setting up a sql statement for each meta data group
    $sql_groups = [];
    foreach ($meta_data_groups as $database => $database_group) {
        //
        // creating database group if it doesn't exist
        if (!isset($sql_groups[$database])) {
            $sql_groups[$database] = [];
        }
        //
        foreach ($database_group as $table => $table_group) {
            if (isset($table_group['set_statment'])) {
                array_push($sql_groups[$database], $table_group['sql_args']);
                continue;
            }
            //
            // pulling out variables for easy access
            $primary = $table_group['primary_index']['column_name'];
            $values = $table_group['form_values'];
            //
            // initializing sql args based on existance of primary index
            $sql_args = [
                'table' => $table,
                'cols' => [],
                'vals' => [],
                'val_exprs' => [],
                'allowed_vars' => $allowed_vars
            ];
            if ($insert_all) {
                $sql_args['cmd'] = 'INSERT';
            }
            else if (!isset($values[$primary]) || $values[$primary] === '') {
                $sql_args['cmd'] = 'INSERT';
                unset($values[$primary]);
            }
            else {
                $sql_args['cmd'] = 'UPDATE';
                $sql_args['where'] = [[$primary, 'LIKE', $values[$primary]]];
                unset($values[$primary]);
            }
            //
            // skipping group if there are no values
            if (count($values) === 0) { continue;}
            //
            // adding columns and values to sql args
            foreach ($values as $key => $value) {
                $col_meta_data = $table_group['meta_data'][$key];
                array_push($sql_args['cols'], $key);
                array_push($sql_args['vals'], $value);
                array_push($sql_args['val_exprs'], $col_meta_data['submission_expr']);
            }
            //
            array_push($sql_groups[$database], $sql_args);
        }
    }
    //
    return $sql_groups;
}
//
// submits the sql array using a transaction
function execute_form_sql($sql_groups) {
    global $mod_username;
    global $mod_password;
    //
    foreach ($sql_groups as $database => $database_group) {
        $conn = connect_db($database, $mod_username, $mod_password);
        $sql_arr = [];
        //
        foreach ($database_group as $sql_args) {
            $sql = gen_prepared_sql($sql_args);
            array_push($sql_arr, $sql);
        }
        //
        list($msg, $err) = exec_transaction($sql_arr, $conn);
        if ($err) {
            return [$msg, $err];
        }
    }
    //
    return ['Success',false];
}
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////// SQL Statement Generation ////////////////////////
//
// builds a set of relational sql args
function build_relational_sql_args($relation_data, $data_sql_args) {
    //
    $table_relations = $relation_data['table_relations'];
    $primary_table = $relation_data['primary_table']['database_table'];
    //
    // incrementally building data sql args
    $sql_args = [
        'cmd' => 'SELECT',
        'cols' => [],
        'table' => $primary_table,
        'inner_join' => [],
        'left_join' => [],
        'right_join' => []
    ];
    // linking tables
    foreach($table_relations as $table) {
        //
        $db_table = $table['database_table'];
        $join_type = strtolower($table['linking_column']).'_join';
        if ($table['subquery_generated']) {
            $db_table = "%{$table['database_table']}% AS `{$table['database_table']}`";
        }
        //
        $join = [
            $db_table,
            $primary_table.'.'.$table['linking_column_name'],
            $table['database_table'].'.'.$table['column_name']
        ];
        //
        array_push($sql_args[$join_type], $join);
    }
    // adding columns
    foreach($relation_data['column_names'] as $column) {
        $col = $column['db_table_col'];
        if ($column['column_type'] === 'js-generated') {
            $column['display_expr'] = 'NULL';
        }
        if (isset($column['display_expr'])) {
            $col = "{$column['display_expr']} AS `{$column['column_name']}`";
        }
        array_push($sql_args['cols'], $col);
    }
    array_push($sql_args['cols'], $relation_data['primary_table']['db_table_col']);
    //
    return $sql_args;
}
//
// validates the supplied SQL args and sets up inital statement
function validate_sql_args($sql_args) {
    //
    $valid = true;
    if (is_null($sql_args['table']) && $sql_args['cmd'] !== 'SET') {
        $msg = "No table provided";
        $valid = false;
    }
    //
    // checking args per command
    if ($sql_args['cmd'] === 'SELECT') {
        $stmt = 'SELECT {cols} FROM {table} ';
    }
    else if ($sql_args['cmd'] === 'SET') {
        if (!$sql_args['variable_name']) {
            $msg = "No variable name provided for {$sql_args['cmd']} command.";
            $valid = false;
        }
        if (!$sql_args['variable_value']) {
            $msg = "No variable value provided for {$sql_args['cmd']} command.";
            $valid = false;
        }
        $stmt = "SET {$sql_args['variable_name']}={$sql_args['variable_value']}";
    }
    else {
        if (!$sql_args['cols']) {
            $msg = "No columns provided for {$sql_args['cmd']} command.";
            $valid = false;
        }
        if (!$sql_args['vals']) {
            $msg =  "No values provided for {$sql_args['cmd']} command.";
            $valid = false;
        }
        if (count($sql_args['cols']) !== count($sql_args['vals'])) {
            $msg =  'Error: cols and vals arrays are different lengths';
            $valid = false;
        }
        //
        if ($sql_args['cmd'] === 'INSERT') {
            if ($sql_args['where']) {
                $msg =  'Error: where array provided for the INSERT command.';
                $valid = false;
            }
            $stmt = 'INSERT INTO {table}({cols}) VALUES ({vals}) ';
        }
        else if ($sql_args['cmd'] === 'UPDATE') {
            if (!$sql_args['where']) {
                $msg =  'No where array provided for the UPDATE command.';
                $valid = false;
            }
            $stmt = 'UPDATE {table} SET {cols} ';
        }
        else {
            $msg =  "Invalid command provided for gen SQL function: {$sql_args['cmd']}";
            $valid = false;
        }
    }
    //
    if (!$valid) {
        //
        throw new Exception($msg);
    }
    //
    return $stmt;
}
//
// generates a SQL statement based on an associative array
// returning a statement to prepare and parameters to execute with
function gen_prepared_sql($sql_args) {
    //
    $sql_args = array_merge(CONSTANTS::GEN_SQL_DEFAULTS, $sql_args);
    $stmt_values = [];
    $stmt = validate_sql_args($sql_args);
    //
    // handling a union statment
    if ($sql_args['union']) {
        // pulling out key values
        $union = $sql_args['union'];
        $main_cols = $sql_args['cols'];
        $main_orderby = $sql_args['order_by'];
        $sql_args['union'] = [];
        //
        // generating SQL statments
        $sql = gen_prepared_sql($sql_args);
        $stmt = format('({})', $sql['statement']);
        $stmt_values = $sql['params'];
        foreach ($union as $sql_args) {
            $sql_args['cols'] = $main_cols;
            $sql = gen_prepared_sql($sql_args);
            $stmt .= format(' UNION ({}) ', $sql['statement']);
            $stmt_values = array_merge($stmt_values, $sql['params']);
        }
        //
        $sql_args = CONSTANTS::GEN_SQL_DEFAULTS;
        $sql_args['order_by'] = $main_orderby;
    }
    //
    // handling sub queries
    if ($sql_args['sub_query']) {
        $sub_queries = $sql_args['sub_query'];
        $sql_args['sub_query'] = null;
        foreach ($sub_queries as $key => $sub_args) {
            $sub_queries[$key] = gen_prepared_sql($sub_args);
        }
        //
        $sql_args['sub_query'] = $sub_queries;
    }
    //
    // adding values to array values and replacing with placeholders
    foreach ($sql_args['vals'] as $i => $value) {
        if (!in_array($value, $sql_args['allowed_vars'])) {
            array_push($stmt_values, $value);
            $sql_args['vals'][$i] = '?';
        }
        if (isset($sql_args['val_exprs'][$i])) {
            $sql_args['vals'][$i] = $sql_args['val_exprs'][$i];
        }
    }
    //
    // special logic for the update command
    if ($sql_args['cmd'] === 'UPDATE') {
         $sql_args['cols'] = array_map(function($c, $v) { return $c.'='.$v;},
                                       $sql_args['cols'], $sql_args['vals']);
    }
    //
    // handle values here
    $sql_args['cols'] = implode(',', $sql_args['cols']);
    $sql_args['vals'] = implode(',', $sql_args['vals']);
    //
    // creating initial SQL statement
    $stmt = format($stmt, ['table'=>$sql_args['table'],
                           'cols'=>$sql_args['cols'],
                           'vals'=>$sql_args['vals']]);
    //
    // adding inner join logic
    foreach ($sql_args['inner_join'] as $value) {
        $stmt .= format('INNER JOIN {0} ON {1}={2} ', $value);
    }
    //
    // adding left join logic
    foreach ($sql_args['left_join'] as $value) {
        $stmt .= format('LEFT JOIN {0} ON {1}={2} ', $value);
    }
    //
    // adding right join logic
    foreach ($sql_args['right_join'] as $value) {
        $stmt .= format('RIGHT JOIN {0} ON {1}={2} ', $value);
    }
    //
    // handling any sub-queries
    preg_match_all('/%.+?%/', $stmt, $sub_query_keys);
    $sub_query_keys = $sub_query_keys[0];
    foreach ($sub_query_keys as $key) {
        $value = $sql_args['sub_query'][$key];
        $stmt = preg_replace("/{$key}/", "({$value['statement']})", $stmt);
        $stmt_values = array_merge($stmt_values, $value['params']);
    }
    //
    // adding where clause
    $prefix = 'WHERE';
    foreach ($sql_args['where'] as $value) {
        if (!array_key_exists(2, $value)) {
            $stmt .= format("{$prefix} {0} {1} ", $value);
        }
        else if ($value[1] === 'BETWEEN') {
            $stmt .= "{$prefix} {$value[0]} {$value[1]} ? AND ? ";
            array_push($stmt_values, $value[2][0], $value[2][1]);
        }
        else if ($value[1] === 'IN') {
            $value[2] = $value[2] ? $value[2] : [];
            foreach ($value[2] as &$v) { array_push($stmt_values, $v); $v = '?';}
            $value[2] = implode(', ', $value[2]);
            $stmt .= format("{$prefix} {0} {1} ({2})", $value);
        }
        else {
            $stmt .= format("{$prefix} {0} {1} ? ", $value);
            array_push($stmt_values, $value[2]);
        }
        $prefix = isset($value[3]) ? $value[3] : 'AND';
    }
    //
    // adding in group by clause
    if ($sql_args['group_by']) {
        $stmt .= "GROUP BY {$sql_args['group_by']} ";
    }
    //
    // adding having clause
    if ($sql_args['having']) {
        $stmt .= "HAVING {$sql_args['having']} ";
    }
    //
    // adding order by clauses
    $prefix = 'ORDER BY';
    foreach ($sql_args['order_by'] as $value) {
        $stmt .= format("{$prefix} {} {}", $value);
        $prefix = ',';
    }
    $stmt .= ' ';
    //
    // adding in limit clause + args.group_by
    if ($sql_args['limit']) {
        $stmt .= 'LIMIT ?, ? ';
        array_push($stmt_values, $sql_args['limit'][0], $sql_args['limit'][1]);
    }
    return ['statement' => $stmt, 'params' => $stmt_values];
}
//
///////////////////////////////////////////////////////////////////////////////
////////////////// General Database Interaction Functions /////////////////////
//
// logs sql executions and their parameters
function sql_logger($statment, $params) {
    $date = date('Y-m-d H:i:s', time());
    if ($params) {
        $message = "[{$date}] Query: $statment, Params:" . print_r($params, true) . PHP_EOL;
    }
    else {
        $message = "[{$date}] Query: $statment". PHP_EOL;
    }
    return file_put_contents(dirname(__FILE__).'/log/queries.log', $message, FILE_APPEND);
}

//
// Connects to the database and fetches data using column name indexing
function fetch_db($sql, $conn) {
    //
    sql_logger($sql['statement'], $sql['params']);
    try {
        //
        // executes a sql SELECT statement.
        $stmt = $conn->prepare($sql['statement']);
        $stmt->execute($sql['params']);
        $results = $stmt->setFetchMode(PDO::FETCH_ASSOC);
        $results = $stmt->fetchAll();
        //
        return [$results, false];
    }
    catch(PDOException $e) {
        // Returns the PDO error encountered and displays it
        $err_str = $e->getMessage();
        return [$err_str, true];
    }
}
//
// Connects to the database and executes sql statement
function exec_db($sql, $conn) {
    sql_logger($sql['statement'], $sql['params']);
    try {
        //
        // executes a sql statement that isn't returning results.
        $stmt = $conn->prepare($sql['statement']);
        $stmt->execute($sql['params']);
        //
        return ['success', false];
    }
    catch(PDOException $e) {
        // Returns the PDO error encountered and displays it
        $err_str = $e->getMessage();
        return [$err_str, true];
    }
}
//
// Connects to the database and executes sql statement
function exec_transaction($sql_arr, $conn) {
    //
    try {
        //
        // executing a number of SQL statements.
        $conn->beginTransaction();
        foreach ($sql_arr as $sql) {
            list($msg, $err) = exec_db($sql, $conn);
            if ($err) {
                throw new Exception($msg);
            }
        }
        $conn->commit();
        //
        return ['Success',false];
    }
    catch(Exception $e) {
        // Returns the PDO error encountered and displays it
        $conn->rollBack();
        $err_str = $e->getMessage();
        return [$err_str, true];
    }
}
?>
