<?php
session_start();
require 'afw_hub.php';
//
// setting up class to handle Async calls
class Async {
    //
    // logs data to aync log file
    static private function logger($data){
        $date = date('Y-m-d H:i:s', time());
        $message = "[{$date}] POST data: " . print_r($data, true) . PHP_EOL;
        return file_put_contents(dirname(__FILE__).'/log/async-requests.log', $message, FILE_APPEND);
    }
    //
    // encodes an object and prints a JSON string
    static private function output_json($object) {
        $json = json_encode($object);
        if ($json === '') {
            echo 'JSON Encoding Error: ', json_last_error(), ' - ', json_last_error_msg();
        }
        else {
            print_r($json);
        }
    }
    //
    // fetches data from one or more sql statements, returns a JSON encoded array
    static private function fetch_db($params) {
        global $main_database;
        $database = $main_database;
        $results = [];
        //
        $sql_args_arr = $params['sql_args_arr'];
        if (is_string($sql_args_arr)) {
            $sql_args_arr = json_decode($sql_args_arr, true);
        }
        //
        $name_arr = $params['return_names'];
        if (is_string($name_arr)) {
            $name_arr = json_decode($name_arr);
        }
        //
        $database_arr = $params['database_arr'];
        if (is_string($database_arr)) {
            $database_arr = json_decode($database_arr);
        }
        //
        // looping over database query groups
        foreach ($database_arr as $i => $database) {
            $database = isset($GLOBALS[$database]) ? $GLOBALS[$database] : null;
            $conn = connect_db($database);
            $args_names = array_map(null, $sql_args_arr[$i], $name_arr[$i]);
            foreach ($args_names as list($sql_args, $name)) {
                $sql_args['cmd'] = 'SELECT';
                $sql = gen_prepared_sql($sql_args);
                list($db_res, $error) = fetch_db($sql, $conn);
                //
                if ($error) {
                    $name = "SQL_REQ_ERROR_MSG-{$name}";
                }
                $results[$name] = $db_res;
            }
            $conn = null;
        }
        return $results;
    }
    //
    // logic to fetch user data
    static private function fetch_auth_data($params) {
        //
        $data_where = [];
        if (isset($params['data_where'])) {
            $data_where = $params['data_where'];
        }
        if (is_string($data_where)) {
            $data_where = json_decode($data_where);
        }
        $results = fetch_auth_table_data($data_where);
        //
        return $results;
    }
    //
    // logic to update the users table
    static private function update_auth_table($params) {
        //
        $form_values = $params['form_values'];
        if (is_string($form_values)) {
            $form_values = json_decode($form_values, true);
        }
        //
        list($sql_args, $conn) = _update_auth_table($form_values);
        $sql = gen_prepared_sql($sql_args);
        //
        list($msg, $err) = exec_db($sql, $conn);
        //
        $conn = null;
        return ['msg' => $msg, 'error' => $err];
    }
    //
    // handles updates to the misc table
    static private function update_misc_table($params) {
        global $meta_database;
        //
        $table = $params['table_name'];
        $form_values = $params['form_values'];
        if (is_string($form_values)) {
            $form_values = json_decode($form_values, true);
        }
        //
        $sql_args = [
            'cmd' => 'SELECT',
            'cols' => [
                '`column_meta_data`.`in_database`',
                '`column_meta_data`.`in_table`',
                '`column_meta_data`.`column_name`',
                '`column_meta_data`.`column_type`',
                '`column_meta_data`.`data_type`',
                'NULL AS `submission_expr`',
                '0 AS `submission_order`',
            ],
            'table' => 'column_meta_data',
            'where' => [
                ['column_type', 'REGEXP', 'static'],
                ['in_table', 'LIKE', $table]
            ]
        ];
        $sql = gen_prepared_sql($sql_args);
        $conn = connect_db($meta_database);
        $meta_data = fetch_db($sql, $conn)[0];
        //
        $allowed_vars = [];
        $meta_groups = setup_meta_data_groups($form_values, $meta_data, $allowed_vars);
        $sql_groups = generate_form_sql($meta_groups, $allowed_vars, false);
        list($msg, $err) = execute_form_sql($sql_groups);
        //
        $conn = null;
        return ['msg' => $msg, 'error' => $err];
    }
    //
    // handles updates to the form meta data table
    static private function submit_form_meta_data($params) {
        global $meta_database;
        //
        $form_name = $params['form_name'];
        $form_val_array = $params['form_values'];
        if (is_string($form_val_array)) {
            $form_val_array = json_decode($form_val_array, true);
        }
        $conn = connect_db($meta_database);
        //
        // setting up SQL to clear any rows using the form_name
        $sql_args = [
            'cmd' => 'UPDATE',
            'table' => 'form_meta_data',
            'cols' => ['form_name', 'column_internal_id'],
            'vals' => [null, null],
            'where' => [['form_name', 'LIKE', $form_name]]
        ];
        //
        // submitting form data and adding clearing sql to front of array
        $sql_groups = process_form_data_array('form_meta_data_entry', $form_val_array, false);
        $sql_groups[$meta_database] = array_merge([$sql_args],
                                                  $sql_groups[$meta_database]);
        //
        list($msg, $err) = execute_form_sql($sql_groups);
        return ['msg' => $msg, 'error' => $err];
    }
    //
    // handles updates to the table meta data table
    static private function submit_table_meta_data($params) {
        global $meta_database;
        //
        $html_table_name = $params['html_table_name'];
        $form_val_array = $params['form_values'];
        if (is_string($form_val_array)) {
            $form_val_array = json_decode($form_val_array, true);
        }
        //
        // setting up SQL to clear any rows using the form_name
        $sql_args = [
            'cmd' => 'UPDATE',
            'table' => 'table_meta_data',
            'cols' => ['html_table', 'column_internal_id'],
            'vals' => [null, null],
            'where' => [['html_table', 'LIKE', $html_table_name]]
        ];
        //
        // submitting table meta data and adding clear sql to front of array
        $sql_groups = process_form_data_array('table_meta_data_entry', $form_val_array, false);
        $sql_groups[$meta_database] = array_merge([$sql_args],
                                                  $sql_groups[$meta_database]);
        //
        list($msg, $err) = execute_form_sql($sql_groups);
        return ['msg' => $msg, 'error' => $err];
    }
    //
    // handles general updates to database tables by forms
    static private function submit_form_data($params) {
        //
        // getting POST values
        $form_name = $params['form_name'];
        $form_values = $params['form_values'];
        $insert_all = isset($params['insert_all']);
        if (is_string($form_values)) {
            $form_values = json_decode($form_values, true);
        }
        //
        // generating and submitting form data sql
        $sql_groups = process_form_data_array($form_name, $form_values, $insert_all);
        list($msg, $err) = execute_form_sql($sql_groups);
        return ['msg' => $msg, 'error' => $err];
    }
    //
    // fetches table meta data and row data to build an HTML table
    static private function fetch_html_table_data($params) {
        global $meta_database;
        //
        // processing POST data and building initial meta sql args
        $database = isset($params['database']) ? $GLOBALS[$params['database']] : null;
        $page = $params['page'];
        $table_name = $params['html_table'];
        //
        $cols = [];
        if (isset($params['display_cols'])) {
            $cols = $params['display_cols'];
        }
        if (is_string($cols)) {
            $cols = json_decode($cols);
        }
        //
        $data_sql_args = [];
        if (isset($params['data_sql_args'])) {
            $data_sql_args = $params['data_sql_args'];
        }
        if (is_string($data_sql_args)) {
            $data_sql_args = json_decode($data_sql_args, true);
        }
        //
        // fetching meta data
        $sql_args = build_html_table_meta_sql_args($table_name, $page, $cols);
        $conn = connect_db($meta_database);
        $sql = gen_prepared_sql($sql_args);
        list($meta_data, $error) = fetch_db($sql, $conn);
        if ($error) {throw new Exception($meta_data);};
        if (isset($params['meta_data_only'])) {
            return ['meta_data' => $meta_data];
        }
        //
        // setting $cols equal to meta data cols if not provided
        if ($cols === null || count($cols) === 0) {
            $cols = array_column($meta_data, 'db_table_col');
        }
        //
        // fetching table data
        $sql_args = build_html_table_data_sql_args($table_name, $page, $cols, $data_sql_args);
        $conn = connect_db($database);
        $sql = gen_prepared_sql($sql_args);
        list($table_data, $error) = fetch_db($sql, $conn);
        if ($error) {throw new Exception($table_data);};
        //
        return ['meta_data' => $meta_data, 'table_data' => $table_data];
    }
    //
    // fetches form meta data as well as data required for population
    static private function fetch_form_data($params) {
        global $meta_database;
        //
        // processing POST data and building initial meta sql args
        $database = isset($params['database']) ? $GLOBALS[$params['database']] : null;
        $form_name = $params['form_name'];
        $data_sql_args = [];
        if (isset($params['data_sql_args'])) {
            $data_sql_args = $params['data_sql_args'];
        }
        if (is_string($data_sql_args)) {
            $data_sql_args = json_decode($data_sql_args, true);
        }
        //
        // setting up SQL args
        $meta_sql_args = build_form_meta_sql_args($form_name, []);
        $data_sql_args = build_form_data_sql_args($form_name, $data_sql_args);
        //
        // fetching meta data
        $conn = connect_db($meta_database);
        $sql = gen_prepared_sql($meta_sql_args);
        list($meta_data, $error) = fetch_db($sql, $conn);
        if ($error) {throw new Exception($meta_data);};
        if (isset($params['meta_data_only'])) {
            return ['meta_data' => $meta_data];
        }
        //
        // fetching table data
        $conn = connect_db($database);
        $sql = gen_prepared_sql($data_sql_args);
        list($form_data, $error) = fetch_db($sql, $conn);
        if ($error) {throw new Exception($form_data);};
        //
        return ['meta_data' => $meta_data, 'form_data' => $form_data];
    }
    //
    // calls a function from the POST submission
    static public function call_async_function() {
        Async::logger($_POST);
        $method = $_POST['method'];
        $results = Async::$method($_POST);
        Async::output_json($results);
    }
//
}
Async::call_async_function();
?>
