/* global AFW_HUB */
////////////////////////////////////////////////////////////////////////////////
//////////////       This file holds the functions that are          ///////////
//////////////       associated only with the ceo_panel page         ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
//
/////////////////////// Ceo Panel Primary Functions ////////////////////////////
//
// creates the tables and populates them with data
function ceo_panel_page() {
    document.getElementById('page-title').textContent = 'Company Overview';
    document.getElementById('page-header').textContent = 'Company Overview';
    //
    build_departmental_expenses_table();
}
//
// builds the departmental expenses table
function build_departmental_expenses_table() {
    //
    // building the overview table structure
    var overview_table = document.createElementWithAttr('TABLE', {});
    var depts = [
        'general',
        'transportation',
        'warehouse-receiving',
        'warehouse-shipping'
    ];
    var ranges = [ 'current-pay-period', 'previous-pay-period', 'year-to-date' ];
    // setting up header row
    var row = document.createElementWithAttr('TR', {});
    var th = document.createElementWithAttr('TH', {
        'style': { 'font-size': '1.5em' }
    });
    var i, j;
    th.colSpan = 1 + ranges.length;
    th.textContent = 'Expenses By Department';
    row.appendChild(th);
    overview_table.appendChild(row);
    //
    row = document.createElementWithAttr('TR', {});
    row.id = 'overview-head-row';
    th = document.createElement('TH');
    th.textContent = 'Department';
    row.appendChild(th);
    for (i = 0; i < ranges.length; i++) {
        th = th.cloneNode();
        th.id = ranges[i];
        th.textContent = ranges[i].replace(/-/g, ' ').toTitleCase();
        row.appendChild(th);
    }
    overview_table.appendChild(row);
    //
    // adding data rows
    var td = document.createElement('TD');
    for (i = 0; i < depts.length; i++) {
        row = row.cloneNode();
        row.id = depts[i] + '-totals-row';
        td = td.cloneNode();
        td.id = depts[i];
        td.textContent = depts[i].replace(/-/g, ' ').toTitleCase();
        row.appendChild(td);
        //
        for (j = 0; j < ranges.length; j++) {
            td = td.cloneNode();
            td.id = depts[i] + '-' + ranges[j];
            row.appendChild(td);
        }
        overview_table.appendChild(row);
    }
    //
    // adding overall total row
    row = row.cloneNode();
    row.id = 'overall-totals';
    th = th.cloneNode();
    th.id = '';
    th.textContent = 'Overall Totals:';
    row.appendChild(th);
    for (i = 0; i < ranges.length; i++) {
        th = th.cloneNode();
        th.id = 'overall-' + ranges[i];
        row.appendChild(th);
    }
    overview_table.appendChild(row);
    //
    document.getElementById('content-div').safeAppendChild(overview_table, false);
    populate_departmental_expenses(depts);
}
//
// populates the table with data
function populate_departmental_expenses(depts) {
    var sql_args,
        sql_arr = [],
        key_arr = [],
        callback,
        curr_pp,
        time_ranges,
        range,
        dates,
        i,
        key,
        args;
    //
    // getting pay period date ranges
    curr_pp = find_pay_period(new Date());
    time_ranges = {
        'current-pay-period': curr_pp,
        'previous-pay-period': find_pay_period(curr_pp[0]),
        'year-to-date': [
            new Date(AFW_HUB.get_constant('FIRST_BUSINESS_DAY')).yyyymmdd(),
            new Date().yyyymmdd()
        ]
    };
    //
    // setting up initial sql args
    sql_args = {
        cmd: 'SELECT',
        cols: [ 'SUM(`total`) `dept_total_pay`' ],
        table: 'employee_data',
        where: [ [ 'entry_status', 'LIKE', 'submitted' ], [], [] ]
    };
    //
    // creating each SQL statement
    for (range in time_ranges) {
        dates = time_ranges[range];
        for (i = 0; i < depts.length; i++) {
            sql_args.where[1] = [ 'department', 'LIKE', depts[i].replace('-', '_') ];
            sql_args.where[2] = [ 'date', 'BETWEEN', dates ];
            key = depts[i] + '-' + range;
            key_arr.push(key);
            sql_arr.push(Object.clone(sql_args));
        }
        //
        sql_args.where[1] = [ 'department', 'REGEXP', '.' ];
        key = 'overall-' + range;
        key_arr.push(key);
        sql_arr.push(Object.clone(sql_args));
    }
    //
    // setting up callback function
    callback = function(response) {
        var total, td;
        // updating table with data
        for (total in response) {
            td = document.getElementById(total);
            args = [
                response[total][0].dept_total_pay,
                'monetary',
                td,
                { format_str: '%value%', add_commas: true }
            ];
            AFW_HUB.TableUtil.display_table_data.apply(null, args);
        }
    };
    //
    AFW_HUB.Ajax.fetch_db(sql_arr, key_arr, null, callback);
}
