/*global AFW_HUB*/
////////////////////////////////////////////////////////////////////////////////
//////////  This file holds functions associated with employee       ///////////
//////////  setup, data entry and maintence                          ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
//
/////////////////////////// Page Setup functions ///////////////////////////////
function employee_data_entry_page() {
    var url_data = AFW_HUB.Navigation.parse_url();
    //
    if (url_data.sub_tab === undefined) {
        setup_employee_department_data_entry();
    } else if (url_data.sub_tab === 'view_data') {
        view_employee_data_page();
    } else if (url_data.sub_tab === 'data_maintenance') {
        employee_data_maintenance_page();
    }
}
//
// sets up the main data entry page showing tabs for each department
function setup_employee_department_data_entry() {
    var nav = document.getElementById('nav-container'),
        legend = document.createElement('LEGEND'),
        fieldset = document.createElementWithAttr('FIELDSET', {
            'id': 'table-fieldset',
            'style': { 'width': '75%' }
        }),
        button = document.createElementWithAttr('BUTTON', {
            'type': 'button',
            'class': 'big'
        }),
        dept = document.createElementWithAttr('INPUT', {
            'id': 'department',
            'type': 'hidden'
        }),
        tabs = {
            occupancy_data_entry: setup_occupancy_data_entry,
            drivers_data_entry: setup_drivers_data_entry,
            freight_logistics: setup_freight_logistics,
            receiving_data_entry: setup_receiving_data_entry,
            shipping_data_entry: setup_shipping_data_entry
        }, tab, i;
    //
    // creating nav bar
    for (tab in tabs) {
        button = button.cloneNode();
        //
        button.id = tab;
        button.textContent = tab.replace(/_/g, ' ').toTitleCase();
        button.addEventListener(
            'click',
            AFW_HUB.Utilities.remove_class.bind(null, 'hidden-elm', 'table-div')
        );
        button.addEventListener('click', tabs[tab]);
        nav.appendChild(button);
    }
    nav.appendChild(dept);
    //
    // modifying the rest of the page
    legend.textContent = 'Select Employee';
    fieldset.appendChild(legend);
    //
    AFW_HUB.Utilities.add_class('hidden-elm', 'input-div');
    AFW_HUB.Utilities.add_class('hidden-elm', 'table-div');
    document.getElementById('table-div').appendChild(fieldset);
    document.getElementById('content-header').id = 'form-header';
    document.getElementById('content-div').id = 'form-div';
    //
    // checking perms for each button
    for (i = 0; i < tabs.length; i++) {
        AFW_HUB.Navigation.check_user_perms('employee_data_entry.' + tabs[i]);
    }
    var clicked = AFW_HUB.Navigation.click_single_button();
    if (clicked) {
        AFW_HUB.Utilities.add_class('hidden-elm', 'home_page');
        AFW_HUB.Utilities.add_class('hidden-elm', clicked);
    }
}
//
// creates the main elements of the edit and view employee data pages
function create_employee_data_page() {
    //
    var url_data = AFW_HUB.Navigation.parse_url();
    var input_div = document.getElementById('input-div');
    var fieldset = document.createElementWithAttr('FIELDSET', {
        'id': 'data-selection-params'
    });
    input_div.appendChild(fieldset);
    //
    // bound functions
    var emp_tbl_fun = data_maintenance_emp_table.bind(
        null,
        1,
        'emp_last_name',
        'ASC'
    );
    var data_tbl_fun = employee_data_table.bind(null, 1, 'date', 'DESC', '.');
    var toggle_emp_tbl = toggle_view_element_button.bind(
        null,
        'show-employee-table',
        'employee-table-div',
        'Hide Employee Table',
        'Show Employee Table'
    );
    var toggle_data_tbl = toggle_view_element_button.bind(
        null,
        'get-all-emp-data',
        'table-div',
        'Hide Employee Data Table',
        'Show Employee Data Table'
    );
    var update_data = show_update_button.bind(
        null,
        'update-employee-data-table',
        'employee-data-table',
        'Update Employee Data Table'
    );
    //
    var rpp_opts = Array(
        { 'elm': 'option', 'value': 10, 'textNode': '10' },
        { 'elm': 'option', 'value': 20, 'textNode': '20' },
        { 'elm': 'option', 'value': 30, 'textNode': '30' },
        { 'elm': 'option', 'value': 50, 'textNode': '50' }
    );
    var fieldset_elms = Array(
        { 'elm': 'legend', 'textNode': 'Data Selection Parameters' },
        { 'elm': 'label', 'textNode': 'Employee Department:' },
        {
            'elm': 'select',
            'id': 'department',
            'name': 'department',
            'events': [
                { 'event': 'change', 'function': emp_tbl_fun },
                { 'event': 'change', 'function': update_data }
            ]
        },
        { 'elm': 'br' },
        { 'elm': 'div', 'id': 'time-range-inputs' },
        { 'elm': 'label', 'textNode': 'Results Per Page:' },
        {
            'elm': 'select',
            'id': 'res-per-page',
            'name': 'res-per-page',
            'children': rpp_opts,
            'events': [ { 'event': 'click', 'function': update_data } ]
        },
        { 'elm': 'br' },
        { 'elm': 'br' },
        { 'elm': 'label', 'textNode': 'Show Deleted Records:' },
        {
            'elm': 'input',
            'type': 'checkbox',
            'id': 'show-deleted',
            'name': 'entry_status',
            'events': [
                {'event': 'click', 'function': function() {
                    AFW_HUB.FormUtil.toggle_checkbox_value(this, '', 'submitted');
                }},
                { 'event': 'click', 'function': update_data }
            ]
        }
    );
    fieldset.add_children(fieldset_elms);
    //
    // adding buttons
    var button = document.createElementWithAttr('BUTTON', {
        'id': 'show-employee-table'
    });
    button.addEventListener('click', emp_tbl_fun);
    button.addEventListener('click', toggle_emp_tbl);
    button.textContent = 'Show Employee Table';
    input_div.appendChild(button);
    button = button.cloneNode();
    button.id = 'get-all-emp-data';
    button.addEventListener('click', data_tbl_fun);
    button.addEventListener('click', toggle_data_tbl);
    button.textContent = 'Get All Employee Data';
    input_div.appendChild(button);
    button = button.cloneNode();
    button.id = 'update-employee-table';
    button.className = 'hidden-elm';
    input_div.appendChild(button);
    button = button.cloneNode();
    button.id = 'update-employee-data-table';
    input_div.appendChild(button);
    input_div.appendChild(
        document.createElementWithAttr('DIV', {
            'id': 'employee-table-div',
            'class': 'hidden-elm'
        })
    );
    //
    // adding the time range inputs
    var update_fun = show_update_button.bind(
        null,
        'update-employee-data-table',
        'employee-data-table',
        'Update Employee Data Table'
    );
    var input_args = { output_id: 'time-range-inputs', update_fun: update_fun };
    create_time_range_inputs(input_args);
    //
    // populating department dropbox
    var dropbox_args = {
        'dropbox_id': 'department',
        'text_format': '%department_name%',
        'value_format': '%department%',
        'placeholder': 'All',
        'placeholder_value': '.',
        'placeholder_disabled': false,
        database: 'misc_database',
        'sql_args': {
            'table': 'departments',
            'cols': [ 'department', 'department_name' ],
            'where': [ [ 'department_type', 'REGEXP', '(^|%)employees(%|$)' ] ],
            'order_by': [ [ 'department_name', 'ASC' ] ]
        }
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    //
    // checking URL data to see if an entry form needs generated
    if (url_data.gen_entry) {
        // creating form
        var args = [
            url_data.emp_internal_id,
            url_data.department,
            url_data.entry_id
        ];
        get_data_entry_form.apply(null, args);
    }
}
//
function view_employee_data_page() {
    //
    // setting page elements
    AFW_HUB.Utilities.add_class('hidden-elm', 'nav-container');
    AFW_HUB.Utilities.add_class('hidden-elm', 'table-div');
    AFW_HUB.Utilities.add_class('print-container', 'content-div');
    AFW_HUB.Utilities.add_class('print-container', 'content-header');
    document.getElementById('page-header').textContent = 'View Employee Data';
    document.getElementById('content-div').id = 'form-div';
    document.getElementById('content-header').id = 'form-header';
    //
    create_employee_data_page();
}
//
function employee_data_maintenance_page() {
    //
    // setting page elements
    AFW_HUB.Utilities.add_class('hidden-elm', 'nav-container');
    AFW_HUB.Utilities.add_class('hidden-elm', 'table-div');
    document.getElementById('page-header').textContent = 'Employee Data Maintenance';
    document.getElementById('content-div').id = 'form-div';
    document.getElementById('content-header').id = 'form-header';
    //
    create_employee_data_page();
}
//
// departmental data entry page setup functions
function setup_occupancy_data_entry() {
    //
    document.getElementById('tab-clicked').value = 'occupancy_data_entry';
    document.getElementById('department').value = 'general';
    document.getElementById('page-header').textContent = 'Occupancy Data Entry';
    document.getElementById('form-header').textContent = '';
    document.getElementById('form-div').removeAll();
    //
    enter_data_emp_table(1, 'emp_last_name', 'ASC', 'general');
}
//
function setup_drivers_data_entry() {
    //
    document.getElementById('tab-clicked').value = 'drivers_data_entry';
    document.getElementById('department').value = 'transportation';
    document.getElementById('page-header').textContent = 'Drivers Data Entry';
    document.getElementById('form-header').textContent = '';
    document.getElementById('form-div').removeAll();
    //
    enter_data_emp_table(1, 'emp_last_name', 'ASC', 'transportation');
}
//
function setup_freight_logistics() {
    //
    document.getElementById('tab-clicked').value = 'freight_backhaul_entry';
    document.getElementById('department').value = 'freight_backhaul';
    document.getElementById('page-header').textContent = 'Freight Logistics';
    document.getElementById('form-header').textContent = '';
    document.getElementById('form-div').removeAll();
    //
    AFW_HUB.Utilities.add_class('hidden-elm', 'table-div');
    get_data_entry_form(null, 'freight_backhaul', null, null);
}
//
function setup_receiving_data_entry() {
    //
    document.getElementById('tab-clicked').value = 'warehouse_receiving_data_entry';
    document.getElementById('department').value = 'warehouse_receiving';
    document.getElementById(
        'page-header'
    ).textContent = 'Warehouse Receiving Data Entry';
    document.getElementById('form-header').textContent = '';
    document.getElementById('form-div').removeAll();
    //
    enter_data_emp_table(1, 'emp_last_name', 'ASC', 'warehouse_receiving');
}
//
function setup_shipping_data_entry() {
    //
    document.getElementById('tab-clicked').value = 'shipping_data_entry';
    document.getElementById('department').value = 'warehouse_shipping';
    document.getElementById(
        'page-header'
    ).textContent = 'Warehouse Shipping Data Entry';
    document.getElementById('form-header').textContent = '';
    document.getElementById('form-div').removeAll();
    //
    enter_data_emp_table(1, 'emp_last_name', 'ASC', 'warehouse_shipping');
}
//
////////////////////////////////////////////////////////////////////////////////
/////////////////////// Table Generation functions /////////////////////////////
//
// generic function to create an employee table
function create_employee_table(args) {
    //
    var table_args = {
        table_name: 'employee_table',
        data_sql_args: args.data_sql_args,
        table_output_id: args.table_output_id || 'table-fieldset',
        table_attr: { id: 'employee-table' },
        row_id_prefix: 'emp-row-',
        row_onclick: args.row_onclick,
        head_row_args: {
            sort_col: args.sort_col,
            sort_dir: args.sort_dir,
            sort_onclick: args.sort_onclick
        },
        page_nav_args: {
            curr_page: args.curr_page,
            sort_col: args.sort_col,
            sort_dir: args.sort_dir,
            tot_pages_shown: 9,
            num_per_page: 10,
            page_nav_div_attr: { id: 'emp-table-page-nav' },
            id_prefix: 'emp',
            link_onclick: args.link_onclick
        }
    };
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
// this function creates the employee table for each of the data entry pages
function enter_data_emp_table(page, sort_col, sort_dir, department) {
    var data_sql_args = {
            sub_query: {
                '%employee_data_last_entry%': {
                    cmd: 'SELECT',
                    cols: [
                        'MAX(employee_data.creation_timestamp) AS `last_data_entry`',
                        'emp_internal_id'
                    ],
                    table: 'employee_data',
                    where: [ [ 'entry_status', 'LIKE', 'submitted' ] ],
                    group_by: 'emp_internal_id'
                }
            },
            where: [
                [ 'employee_info.emp_status', 'LIKE', 'active' ],
                [ 'employee_info.department', 'REGEXP', department ]
            ],
            order_by: [ [ sort_col, sort_dir ] ]
        },
        //
        args = {
            data_sql_args: data_sql_args,
            curr_page: page,
            sort_col: sort_col,
            sort_dir: sort_dir,
            row_onclick: [
                {
                    func: get_data_entry_form,
                    args: [ '%emp_internal_id%', department, null, undefined ]
                }
            ],
            sort_onclick: [
                {
                    func: enter_data_emp_table,
                    args: [ 1, '%column_name%', '%sort_dir%', department ]
                }
            ],
            link_onclick: [
                {
                    func: enter_data_emp_table,
                    args: [ '%page%', sort_col, sort_dir, department ]
                }
            ]
        };
    //
    create_employee_table(args);
}
//
// creates the employee table for the edit_emp_data  and view emp data tabs
function data_maintenance_emp_table(page, sort_col, sort_dir) {
    //
    // creating argument objects
    var args,
        data_sql_args,
        department = document.getElementById('department').value,
        update_button = document.getElementById('update-employee-table');
    //
    // adding an event listener to the update-employee-table button
    update_button = update_button.cloneNode(true);
    update_button.addEventListener(
        'click',
        data_maintenance_emp_table.bind(null, page, sort_col, sort_dir)
    );
    document.safeAppendChild(update_button, true);
    //
    data_sql_args = {
        where: [
            [ 'department', 'REGEXP', department ],
            [ 'emp_status', 'LIKE', 'active' ]
        ],
        order_by: [ [ sort_col, sort_dir ] ]
    };
    //
    args = {
        data_sql_args: data_sql_args,
        table_output_id: 'employee-table-div',
        curr_page: page,
        sort_col: sort_col,
        sort_dir: sort_dir,
        row_onclick: [
            { func: employee_data_table, args: [ 1, 'date', 'DESC', '%emp_internal_id%' ] },
            {
                func: AFW_HUB.Utilities.remove_class,
                args: [ 'hidden-elm', 'table-div' ]
            }
        ],
        sort_onclick: [
            {
                func: data_maintenance_emp_table,
                args: [ 1, '%column_name%', '%sort_dir%' ]
            }
        ],
        link_onclick: [
            {
                func: data_maintenance_emp_table,
                args: [ '%page%', sort_col, sort_dir ]
            }
        ]
    };
    //
    create_employee_table(args);
}
//
// generates the data table for the edit emp data and view emp data tabs
function employee_data_table(page, sort_col, sort_dir, emp_internal_id) {
    var data_table_args,
        data_sql_args,
        time_div = document.getElementById('time-range-inputs'),
        update_button = document.createElement('BUTTON'),
        ts_obj = to_and_from_timestamps(time_div),
        department = document.getElementById('department').value,
        show_deleted = document.getElementById('show-deleted').checked;
    //
    // adding an event listener to the update-employee-data-table button
    update_button.id = 'update-employee-data-table';
    update_button.type = 'button';
    update_button.className = 'hidden-elm';
    update_button.addEventListener(
        'click',
        employee_data_table.bind(null, page, sort_col, sort_dir, emp_internal_id)
    );
    document.safeAppendChild(update_button, true);
    //
    // creating data sql
    data_sql_args = {
        where: [
            [ 'employee_info.emp_internal_id', 'REGEXP', emp_internal_id ],
            [ 'employee_data.date', 'BETWEEN', [ ts_obj.from_ts, ts_obj.to_ts ] ],
            [ 'employee_data.department', 'REGEXP', department ]
        ],
        order_by: [ [ sort_col, sort_dir ] ]
    };
    if (!show_deleted) {
        data_sql_args.where.push([ 'entry_status', 'LIKE', 'submitted' ]);
    }
    //
    // setting up table argument object
    data_table_args = {
        table_name: 'employee_data',
        data_sql_args: data_sql_args,
        table_output_id: 'table-div',
        table_attr: { id: 'employee-data-table' },
        row_id_prefix: 'emp-data-row-',
        row_onclick: [
            {
                func: get_data_entry_form,
                args: [ '%emp_internal_id%', '%department%', '%entry_id%', null ]
            }
        ],
        head_row_args: {
            sortable: true,
            sort_col: sort_col,
            sort_dir: sort_dir,
            sort_onclick: [
                {
                    func: employee_data_table,
                    args: [ 1, '%column_name%', '%sort_dir%', emp_internal_id ]
                }
            ]
        },
        page_nav_args: {
            curr_page: page,
            sort_col: sort_col,
            sort_dir: sort_dir,
            tot_pages_shown: 9,
            num_per_page: Number(document.getElementById('res-per-page').value),
            page_nav_div_attr: { id: 'employee-data-table-page-nav' },
            id_prefix: 'emp-data',
            link_onclick: [
                {
                    func: employee_data_table,
                    args: [ '%page%', sort_col, sort_dir, emp_internal_id ]
                }
            ]
        }
    };
    //
    AFW_HUB.TableUtil.create_standard_table(data_table_args);
}
//
////////////////////////////////////////////////////////////////////////////
/////////////////////// Form initialization ////////////////////////////////
//
// Handles creation of a data entry form
function get_data_entry_form(emp_internal_id, department, entry_id, form) {
    var url_data = AFW_HUB.Navigation.parse_url(),
        form_div = document.getElementById('form-div'),
        form_args,
        args,
        callback;
    //
    // loading form
    if (form == undefined) {
        callback = get_data_entry_form.bind(null, emp_internal_id, department, entry_id);
        args = [department + '_data_form', form_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null, args);
        return;
    }
    if (department === 'freight_backhaul') {
        get_backhaul_form(form, entry_id);
        return;
    }
    //
    // setting up args to either fetch a data entry or employe info
    form_args = {
        function: AFW_HUB.FormUtil.populate_form,
        args: {
            form_id: form.id,
            trigger_events: true,
            sql_args: {
                where: [
                    ['employee_data.entry_id', 'LIKE', entry_id],
                ]
            },
            data_preprocessor: function(args, response) {
                add_data_entry_form_listeners(form, department, response);
                document.getElementById('date').value = new Date().yyyymmdd();
            },
            add_callback: function() {
                var event = document.createEvent('HTMLEvents');
                event.initEvent('blur', true, false);
                document.getElementById('total').dispatchEvent(event);
            }
        }
    };
    if (entry_id === null) {
        form_args = {
            function: function get_emp_info(args, response) {
                var sql_args,
                    fetch_cb = get_emp_info.bind(null, args);
                if (response === undefined) {
                    sql_args = {
                        cmd: 'SELECT',
                        table: 'employee_info',
                        where: [
                            ['emp_internal_id', 'LIKE', emp_internal_id]
                        ]
                    };
                    AFW_HUB.Ajax.fetch_db([sql_args], ['form_data'], null, fetch_cb);
                    return;
                }
                //
                args.data_preprocessor = function(args, resp) {
                    resp.form_data = response.form_data;
                    args.meta_data_only = undefined;
                    add_data_entry_form_listeners(form, department, resp);
                    document.getElementById('date').value = new Date().yyyymmdd();
                };
                AFW_HUB.FormUtil.populate_form(args);
            },
            args: {
                form_id: form.id,
                trigger_events: true,
                meta_data_only: 1,
                sql_args: {}
            }
        };
    }
    // setting up args to populate form elements
    args = {
        methods: [
            {
                function: AFW_HUB.FormUtil.populate_dropbox_options,
                args: {
                    dropbox_id: 'attendance-error',
                    value_format: '%absence_value%',
                    text_format: '%absence_kind%',
                    database: 'misc_database',
                    sql_args: {
                        cmd: 'SELECT',
                        table: 'attendance',
                        cols: [ 'absence_value', 'absence_kind' ]
                    }
                }
            },
            {
                function: AFW_HUB.FormUtil.populate_dropbox_options,
                args: {
                    dropbox_id: 'error-code',
                    value_format: '%reason%',
                    text_format: '%description%',
                    placeholder: 'NO ERRORS',
                    placeholder_value: '0',
                    placeholder_disabled: false,
                    add_opts_val: [ '99' ],
                    add_opts_text: [ 'OTHER' ],
                    database: 'misc_database',
                    sql_args: {
                        cmd: 'SELECT',
                        table: 'error_codes',
                        cols: [ 'reason', 'description' ]
                    },
                    add_callback: function(args, dropbox, response) {
                        args.dropbox_id = 'error-code2';
                        args.add_callback = function() {};
                        AFW_HUB.FormUtil.populate_dropbox_options(args, response);
                        args.dropbox_id = 'error-code3';
                        AFW_HUB.FormUtil.populate_dropbox_options(args, response);
                        args.dropbox_id = 'error-code4';
                        AFW_HUB.FormUtil.populate_dropbox_options(args, response);
                    }
                }
            },
            form_args
        ]
    };
    //
    callback = function(args, response) {
        if (url_data.sub_tab == 'data_maintenance') {
            modify_employee_data_entry(form, response);
        } else if (url_data.sub_tab == 'view_data') {
            view_employee_data_entry(form, response);
        }
    };
    //
    AFW_HUB.FormUtil.populate_multiple_containers(args, callback);
}
//
// this creates the driver frieght backhaul form
function get_backhaul_form(form, entry_id) {
    var form_args,
        sql_args,
        callback;
    //
    // setting up args to either fetch a data entry or employe info
    form_args = {
        form_id: form.id,
        trigger_events: true,
        sql_args: {
            where: [
                ['employee_data.entry_id', 'LIKE', entry_id],
            ]
        },
        data_preprocessor: function(args, response) {
            var sub_button = document.getElementById('submit-data-form'),
                add_po = document.getElementById('add-po-fieldset'),
                recalc = recalc_freight_backhaul_form,
                func;
            response.all_form_data = response.form_data;
            response.form_data = [response.form_data[0]];
            add_backhaul_form_listeners(form, form, response);
            //
            func = submit_employee_data_form.bind(null, form, recalc, 'create');
            sub_button.addEventListener('click', func);
            func = create_haul_fields.bind(null, form, response);
            add_po.addEventListener('click', func);
            //
            document.getElementById('date').value = new Date().yyyymmdd();
        },
        add_callback: function(args, form, response) {
            var url_data = AFW_HUB.Navigation.parse_url(),
                event = document.createEvent('HTMLEvents'),
                i;
            args.form_id = undefined;
            args.data_preprocessor = function() {};
            args.add_callback = function() {};
            for (i = 0; i < response.all_form_data.length; i++) {
                response.form_data = [response.all_form_data[i]];
                args.form = create_haul_fields(form, response);
                AFW_HUB.FormUtil.populate_form(args, response);
            }
            //
            event.initEvent('blur', true, false);
            document.getElementById('total-savings').dispatchEvent(event);
            //
            if (url_data.sub_tab === 'data_maintenance') {
                modify_employee_data_entry(form, response);
                modify_backhaul_entry(form);
            } else if (url_data.sub_tab === 'view_data') {
                view_employee_data_entry(form, response);
                view_backhaul_entry();
            }
        }
    };
    if (entry_id === null) {
        form_args.meta_data_only = 1;
        sql_args = {
            cmd: 'SELECT',
            table: 'employee_info',
            where: [ [ 'emp_id', 'LIKE', '999999' ] ]
        };
        callback = function(response) {
            var fetch_cb = function(resp) {
                response.form_data = resp.form_data;
                form_args.meta_data_only = undefined;
                AFW_HUB.FormUtil.populate_form(form_args, response);
            };
            AFW_HUB.Ajax.fetch_db([sql_args], ['form_data'], null, fetch_cb);
        };
        AFW_HUB.FormUtil.fetch_form_data(form.name, {meta_data_only: 1}, callback);

    }
    else {
        AFW_HUB.FormUtil.populate_form(form_args);
    }
}
//
// handles viewing of an employee data entry
function view_employee_data_entry(form, response) {
    var header = 'Viewing Record: {entry_id}: of {emp_first_name} {emp_last_name}',
        submit_button = document.getElementById('submit-data-form'),
        hide_button = document.createElementWithAttr('BUTTON', {type: 'button'}),
        edit_button = hide_button.cloneNode(),
        print_button = hide_button.cloneNode(),
        all_children = form.allSubNodes(),
        i;
    header = header.format(response.form_data[0]);
    document.getElementById('form-header').textContent = header;
    document.getElementById('emp-first-name').focus();
    //
    // setting all fields to readonly
    for (i = 0; i < all_children.length; i++) {
        if (all_children[i].nodeType != 1) {
            continue;
        }
        all_children[i].readOnly = true;
        //
        if (!all_children[i].type) {
            all_children[i].type = '';
        }
        if (all_children[i].type.toUpperCase() == 'CHECKBOX') {
            AFW_HUB.Utilities.add_class('readonly-field', all_children[i]);
        }
        if (all_children[i].nodeName == 'SELECT') {
            AFW_HUB.Utilities.add_class('readonly-field', all_children[i]);
        }
    }
    //
    // creating a hide entry button
    hide_button.textContent = 'Hide Entry';
    AFW_HUB.Utilities.add_class('do-not-print', hide_button);
    hide_button.addEventListener('click', function() {
        document.getElementById('form-header').removeAll();
        document.getElementById('form-div').removeAll();
    });
    submit_button.parentNode.replaceChild(hide_button, submit_button);
    //
    edit_button.textContent = 'Edit Record';
    edit_button.addEventListener('click', view_to_edit_entry);
    hide_button.parentNode.appendChild(edit_button);
    //
    print_button.textContent = 'Print Record';
    print_button.addEventListener('click', window.print.bind(window));
    hide_button.parentNode.appendChild(print_button);
}
//
// handles modification of a regular data
function modify_employee_data_entry(form, response) {
    var header = 'Modifying Record: {entry_id}: of {emp_first_name} {emp_last_name}',
        department = form.department.value,
        recalc_fun = window['recalc_' + department + '_form'],
        sub_func = function(act) {
            submit_employee_data_form(form, recalc_fun, act);
        },
        mod_button = document.getElementById('submit-data-form').cloneNode(),
        del_button = document.createElementWithAttr('BUTTON', {type: 'button'}),
        res_button = del_button.cloneNode(),
        hide_button = del_button.cloneNode();
    header = header.format(response.form_data[0]);
    mod_button.textContent = 'Update Record';
    del_button.textContent = 'Delete Record';
    res_button.textContent = 'Restore Record';
    hide_button.textContent = 'Hide Record';
    document.getElementById('form-header').textContent = header;
    document.getElementById('emp-first-name').focus();
    //
    // adding submission event listeners
    mod_button.addEventListener('click', sub_func.bind(null, 'modify'));
    res_button.addEventListener('click', sub_func.bind(null, 'restore'));
    del_button.addEventListener('click', sub_func.bind(null, 'delete'));
    hide_button.addEventListener('click', function() {
        document.getElementById('form-header').removeAll();
        document.getElementById('form-div').removeAll();
    });
    //
    form.addNodes([ mod_button, del_button, res_button, hide_button ]);
    //
    if (response.form_data[0].entry_status !== 'submitted') {
        AFW_HUB.Utilities.add_class('hidden-elm', del_button);
        AFW_HUB.Utilities.remove_class('hidden-elm', res_button);
    }
    else {
        AFW_HUB.Utilities.add_class('hidden-elm', res_button);
        AFW_HUB.Utilities.remove_class('hidden-elm', del_button);
    }
}
//
// sets up the form to view a backhaul entry
function view_backhaul_entry() {
    var po_container = document.getElementById('haul-info'),
        fieldsets = po_container.querySelectorAll('.haul-fieldset'),
        hide_button = document.createElementWithAttr('BUTTON', {type: 'button'}),
        hide_show_func, i;
    hide_button.textContent = 'Hide PO';
    document.getElementById('add-po-fieldset').remove();
    //
    // creating toggle function
    hide_show_func = function(fieldset, button) {
        var class_fun = AFW_HUB.Utilities.add_class,
            i;
        //
        if (fieldset.dataset.show) {
            class_fun = AFW_HUB.Utilities.remove_class;
            fieldset.dataset.show = '';
            button.textContent = 'Hide PO';
        } else {
            fieldset.dataset.show = '1';
            button.textContent = 'Show PO';
        }
        //
        for (i = 1; i < fieldset.children.length - 1; i++) {
            class_fun('hidden-elm', fieldset.children[i]);
        }
    };
    //
    // creating a hide entry button
    for (i = 0; i < fieldsets.length; i++) {
        hide_button = hide_button.cloneNode(true);
        hide_button.addEventListener('click', hide_show_func.bind(null, fieldsets[i], hide_button));
        fieldsets[i].replaceChild(hide_button, fieldsets[i].lastChild);
    }
}
//
// sets up the form to modify a freight backhaul data entry
function modify_backhaul_entry(form) {
    var po_container = document.getElementById('haul-info'),
        fieldsets = po_container.querySelectorAll('.haul-fieldset'),
        mod_button = document.createElementWithAttr('BUTTON', {type: 'button'}),
        del_button = mod_button.cloneNode(),
        res_button = mod_button.cloneNode(),
        func, input, i;
    mod_button.textContent = 'Modify PO';
    del_button.textContent = 'Delete PO';
    res_button.textContent = 'Restore PO';
    //
    // creating a hide entry button
    for (i = 0; i < fieldsets.length; i++) {
        input = fieldsets[i].querySelector('[name=po_status]');
        mod_button = mod_button.cloneNode(true);
        del_button = del_button.cloneNode(true);
        res_button = res_button.cloneNode(true);
        //
        del_button.id = 'del-haul-' + i;
        res_button.id = 'res-haul-' + i;
        func = update_haul_field.bind(null, form, fieldsets[i], 'modify');
        mod_button.addEventListener('click', func);
        func = update_haul_field.bind(null, form, fieldsets[i], 'delete');
        del_button.addEventListener('click', func);
        func = update_haul_field.bind(null, form, fieldsets[i], 'restore');
        res_button.addEventListener('click', func);
        //
        if (input.value !== 'submitted') {
            AFW_HUB.Utilities.add_class('hidden-elm', del_button);
            AFW_HUB.Utilities.remove_class('hidden-elm', res_button);
        }
        else {
            AFW_HUB.Utilities.add_class('hidden-elm', res_button);
            AFW_HUB.Utilities.remove_class('hidden-elm', del_button);
        }
        //
        fieldsets[i].replaceChild(mod_button, fieldsets[i].lastChild);
        fieldsets[i].addNodes([del_button, res_button]);
    }
}
//
// handles the transitions from viewing an enrty to editing one
function view_to_edit_entry() {
    var url_data = AFW_HUB.Navigation.parse_url();
    //
    // parsing URL to initialize data object
    delete url_data.full_url;
    delete url_data.url_data;
    url_data.tab = 'employee_data_entry';
    url_data.sub_tab = 'data_maintenance';
    url_data.gen_entry = 'true';
    //
    // getting form data
    url_data.emp_internal_id = document.getElementById('emp-internal-id').value;
    url_data.department = document.getElementById(
        'employee-data-form'
    ).department.value;
    url_data.entry_id = document.getElementById('entry-id').value;
    //
    location.href = AFW_HUB.Navigation.build_url(url_data);
}
