/*global AFW_HUB*/
////////////////////////////////////////////////////////////////////////////////
//////////  This file holds functions associated with employee       ///////////
//////////  setup, data entry and maintence                          ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
//
///////////////////// Data entry form validation ///////////////////////
//
// adds event handlers to data entry forms
function add_data_entry_form_listeners(form, department, response) {
    var recalc_fun = function() {},
        sub_recalc,
        elm_events,
        event_args,
        elm,
        event,
        i;
    //
    // setting department recalculation function
    if (department == 'general') {
        recalc_fun = recalc_general_form;
    } else if (department == 'transportation') {
        recalc_fun = recalc_transportation_form;
        add_transportation_event_listeners();
    } else if (department == 'warehouse_receiving') {
        recalc_fun = recalc_warehouse_receiving_form;
    } else if (department == 'warehouse_shipping') {
        add_warehouse_shipping_event_listeners();
        recalc_fun = recalc_warehouse_shipping_form;
    }
    sub_recalc = recalc_fun;
    recalc_fun = recalc_fun.bind(null, form, null);
    //
    // adding element specific events
    elm_events = [
        {
            id: 'no-additional-pay',
            event: function(elm) {
                var input = document.getElementById('additional-pay');
                input.readOnly = elm.checked;
                input.value = elm.checked ? 0 : input.value;
            }
        },
        {
            id: 'no-pay-deductions',
            event: function(elm) {
                var input = document.getElementById('pay-deductions');
                input.readOnly = elm.checked;
                input.value = elm.checked ? 0 : input.value;
            }
        },
        { id: 'attendance-error', event: setup_off_day.bind(null, form) },
        { id: 'attendance-error', event: recalc_fun },
        {
            id: 'error-code',
            event: function(elm) {
                document.getElementById('error-code2').disabled =
                    elm.value + '' === '0';
            }
        },
        {
            id: 'error-code2',
            event: function(elm) {
                document.getElementById('error-code3').disabled =
                    elm.value + '' === '0';
            }
        },
        {
            id: 'error-code3',
            event: function(elm) {
                document.getElementById('error-code4').disabled =
                    elm.value + '' === '0';
            }
        },
        {
            id: 'submit-data-form',
            event: submit_employee_data_form.bind(null, form, sub_recalc, 'create')
        }
    ];

    //
    // adding event handlers to the form before adding the defaults
    for (i = 0; i < elm_events.length; i++) {
        (elm = document.getElementById(elm_events[i].id)),
            (event = elm.primaryEventString());
        elm.addEventListener(event, elm_events[i].event.bind(null, elm));
    }
    //
    // adding default functions to all form elements
    event_args = {
        meta_array: response.meta_data,
        add_default_events: function(elm) {
            var event = elm.primaryEventString();
            if (!event) {
                return;
            }
            if (elm.nodeName.toLowerCase() == 'button') {
                return;
            }
            //
            elm.addEventListener(
                event,
                AFW_HUB.Utilities.remove_class.bind(null, 'boxshadow-red', elm)
            );
            if (event == 'click') {
                elm.addEventListener(event, recalc_fun);
            } else {
                elm.addEventListener('blur', recalc_fun);
            }
        }
    };
    AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
}
//
// function to add the several transportation form specific event handlers
function add_transportation_event_listeners() {
    var elm_events, elm, event, i;
    //
    function vacation_day(elm) {
        if (elm.value == 'vacation') {
            document.getElementById('hourly-pay-only').checked = true;
            document.getElementById('hourly-pay-only').dispatchPrimaryEvent();
        }
    }

    function overnight_click(checkbox) {
        var input = document.getElementById('per-diem');
        AFW_HUB.FormUtil.toggle_checkbox_value(checkbox, 1, 0);
        AFW_HUB.FormUtil.disable_if_checked(checkbox, input, true);
        document.getElementById('pre-inspection').dispatchPrimaryEvent();
        document.getElementById('post-inspection').dispatchPrimaryEvent();
    }

    function set_orig(elm) {
        var orig_elm = document.getElementById(elm.id + '-orig');
        orig_elm.value = elm.value;
    }

    function rem_comp_pay(checkbox) {
        var pay_type = checkbox.id.match('rem-(.*?)-pay')[1];
        var rate = document.getElementById(pay_type + '-rate');
        var orig_rate = document.getElementById(pay_type + '-rate-orig');
        if (checkbox.checked) {
            orig_rate.value = rate.value;
            rate.value = 0;
        } else {
            rate.value = orig_rate.value;
        }
        AFW_HUB.FormUtil.disable_if_checked(checkbox, rate);
    }

    function only_hourly_pay(checkbox) {
        var boxes = ['rem-base-pay', 'rem-case-pay', 'rem-stop-pay'];
        //
        document.getElementById('hourly-pay-rate').disabled = !checkbox.checked;
        for (var i = 0; i < boxes.length; i++) {
            var elm = document.getElementById(boxes[i]);
            elm.checked = checkbox.checked;
            elm.dispatchPrimaryEvent();
        }
        document.getElementById('pre-inspection').dispatchPrimaryEvent();
        document.getElementById('post-inspection').dispatchPrimaryEvent();
    }

    function inspection_pay(checkbox) {
        var over_night = Number(document.getElementById('over-night').value);
        checkbox.value = 0;
        if (checkbox.checked) {
            checkbox.value = 3.5 + over_night * 3.5;
        }
    }
    //
    elm_events = [
        { id: 'attendance-error', event: vacation_day },
        { id: 'base-rate', event: set_orig },
        { id: 'case-rate', event: set_orig },
        { id: 'stop-rate', event: set_orig },
        {
            id: 'edit-levels',
            event: function(elm) {
                document.getElementById('base-rate').readOnly = !elm.checked;
                document.getElementById('case-rate').readOnly = !elm.checked;
                document.getElementById('stop-rate').readOnly = !elm.checked;
            }
        },
        { id: 'over-night', event: overnight_click },
        { id: 'rem-case-pay', event: rem_comp_pay },
        { id: 'rem-base-pay', event: rem_comp_pay },
        { id: 'rem-stop-pay', event: rem_comp_pay },
        { id: 'pre-inspection', event: inspection_pay },
        { id: 'post-inspection', event: inspection_pay },
        { id: 'hourly-pay-only', event: only_hourly_pay }
    ];
    //
    for (i = 0; i < elm_events.length; i++) {
        elm = document.getElementById(elm_events[i].id);
        event = elm.primaryEventString();
        elm.addEventListener(event, elm_events[i].event.bind(null, elm));
    }
}
//
// adds extra shipping form event listeners
function add_warehouse_shipping_event_listeners() {
    function toggle_cases_and_pallets() {
        var area = document.getElementById('area').value,
            cases_inp = document.getElementById('num-cases'),
            pallets_inp = document.getElementById('num-pallets');
        //
        if (area == 'loading') {
            cases_inp.disabled = true;
            pallets_inp.disabled = false;
            AFW_HUB.Utilities.add_class('hidden-elm', cases_inp.parentNode);
            AFW_HUB.Utilities.remove_class('hidden-elm', pallets_inp.parentNode);
        }
        else {
            cases_inp.disabled = false;
            pallets_inp.disabled = true;
            AFW_HUB.Utilities.remove_class('hidden-elm', cases_inp.parentNode);
            AFW_HUB.Utilities.add_class('hidden-elm', pallets_inp.parentNode);
        }
        recalc_warehouse_shipping_form(cases_inp.form, null);
    }
    document.getElementById('area').addEventListener('change', toggle_cases_and_pallets);
}
//
// adds all backhaul form event listeners
function add_backhaul_form_listeners(form, container, response) {
    var event_args;
    //
    event_args = {
        meta_array: response.meta_data,
        add_default_events: function(elm) {
            var event = elm.primaryEventString();
            if (!event) {
                return;
            }
            if (elm.nodeName.toLowerCase() === 'button') {
                return;
            }
            //
            elm.addEventListener(
                event,
                AFW_HUB.Utilities.remove_class.bind(null, 'boxshadow-red', elm)
            );
            if (event === 'click') {
                elm.addEventListener(
                    event,
                    recalc_freight_backhaul_form.bind(null, form, null)
                );
            } else {
                elm.addEventListener(
                    'blur',
                    recalc_freight_backhaul_form.bind(null, form, null)
                );
            }
        }
    };
    //
    // adding default functions to all form elements in container
    AFW_HUB.FormUtil.add_standard_event_listeners(container, event_args);
}
//
// creates a fieldset for a single backhaul PO
function create_haul_fields(form, response) {
    var container = document.getElementById('haul-info'),
        fieldset = document.createElement('FIELDSET'),
        entry_num = document.getElementById('number-of-fields').value,
        elements;
    //
    function remove_haul() {
        this.remove();
    }
    entry_num = Number(entry_num);
    //
    elements = [
        { elm: 'legend', textNode: 'Haul ' + (entry_num + 1) },
        { elm: 'label', textNode: 'PO Number' },
        { elm: 'input', type: 'text', name: 'po_number' },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Pick Up Point' },
        { elm: 'input', type: 'text', name: 'pickup_point' },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Freight Allowance' },
        {
            elm: 'input',
            type: 'text',
            name: 'freight_allowance',
            placeholder: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Pallet Charges' },
        {
            elm: 'input',
            type: 'text',
            name: 'pallet_charge',
            placeholder: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        //
        { elm: 'input', name: 'internal_id', type: 'hidden' },
        { elm: 'input', name: 'po_status', type: 'hidden', value: 'submitted' },
        {
            elm: 'button',
            id: 'remove-haul-' + entry_num,
            type: 'button',
            textNode: 'Remove Haul',
            events: [{ event: 'click', function: remove_haul.bind(fieldset) }]
        }
    ];
    //
    // stepping through form_elements array to construct fieldset element
    fieldset.id = 'haul-info-' + entry_num;
    AFW_HUB.Utilities.add_class('haul-fieldset', fieldset);
    fieldset.add_children(elements);
    add_backhaul_form_listeners(form, fieldset, response);
    //
    container.safeAppendChild(fieldset);
    document.getElementById('number-of-fields').value = entry_num + 1;
    //
    return fieldset;
}
//
// handles modifying form inputs for a vacation day
function setup_off_day(form) {
    var form_elements = form.querySelectorAll('INPUT,TEXTAREA,SELECT'),
        attendance = document.getElementById('attendance-error').value,
        disable = { vacation: 1, holiday: 1, absent: 1, sick: 1, day_off: 1 },
        elm,
        i;
    disable = disable[attendance];
    //
    // disabling or enabling elements based on attendance status
    for (i = 0; i < form_elements.length; i++) {
        elm = form_elements[i];
        if (elm.dataset.no_toggle_absent) {
            continue;
        }
        if (elm.id.match(/^edit-/)) {
            continue;
        }
        if (disable && elm.disabled) {
            elm.dataset.toggle_absent = '';
            continue;
        }
        // only enabling elements that were initially disabled
        if (!disable && !elm.dataset.toggle_absent) {
            continue;
        }
        elm.dataset.toggle_absent = '1';
        elm.disabled = disable;
    }
    //
    // setting hours to 8 if it is a vacation day or 8 for absence
    if (disable) {
        elm = document.getElementById('hours');
        elm.value = attendance.match(/vacation|holiday/) ? 8 : 0;
        AFW_HUB.Utilities.remove_class_all('boxshadow-red');
    }
}
//
// this recalculates all of the fields on the general form
function recalc_general_form(form, main_callback) {
    var form_values, callback;
    main_callback = main_callback || function() {};
    //
    if (this) {
        document.getElementById('total').value = this.form_values.total;
        form_values = AFW_HUB.FormUtil.get_all_form_values(form);
        form_values.total = this.form_values.total;
        AFW_HUB.FormUtil.round_form_inputs(form);
        main_callback.call({ form_values: form_values });
        return;
    }
    //
    // getting all form values and processing them based on data type
    form_values = AFW_HUB.FormUtil.get_all_form_values(form, {
        disabled_default: 0
    });
    form_values.total = form_values.additional_pay - form_values.pay_deductions;
    //
    // handling overtime
    callback = function() {
        recalc_general_form.call(this, form, main_callback);
    };
    //
    calc_employee_overtime(form, form_values, callback, null);
}
//
// this recalculates all of the fields on the driver form
function recalc_transportation_form(form, main_callback) {
    var form_values,
        backhaul_rate = 10,
        hourly_pay = 0,
        incentive_pay = 0,
        reimbursement = 0;
    (main_callback = main_callback || function() {}), //
        (form_values = AFW_HUB.FormUtil.get_all_form_values(form, {
            disabled_default: 0
        }));
    //
    // handling hourly pay only checkbox
    if (document.getElementById('hourly-pay-only').checked) {
        backhaul_rate = 0;
        hourly_pay = form_values.hourly_pay_rate * form_values.hours;
    }
    //
    // adjusting values if overnight
    if (Number(form_values.over_night) === 1) {
        form_values.base_rate = 2 * form_values.base_rate;
    }
    //
    // calculating pay values
    incentive_pay = form_values.per_diem;
    incentive_pay += backhaul_rate * form_values.num_backhauls;
    incentive_pay += form_values.base_rate * form_values.num_routes;
    incentive_pay += form_values.case_rate * form_values.num_cases;
    incentive_pay += form_values.stop_rate * form_values.num_stops;
    incentive_pay += form_values.pre_inspection + form_values.post_inspection;
    reimbursement += form_values.hotel_amount;
    reimbursement += form_values.toll_amount;
    reimbursement += form_values.truck_fuel * form_values.cost_per_gallon;
    reimbursement += form_values.reefer_fuel * form_values.cost_per_gallon;
    reimbursement += form_values.fuel_def * form_values.def_cost_per_gallon;
    form_values.total = hourly_pay + incentive_pay;
    form_values.total += form_values.additional_pay;
    form_values.total -= form_values.pay_deductions;
    //
    // outputting values
    document.getElementById('incentive-pay').value = incentive_pay;
    document.getElementById('hourly-pay').value = hourly_pay;
    document.getElementById('reimbursement').value = reimbursement;
    document.getElementById('total').value = form_values.total;
    //
    // rounding displayed values and executing callback function
    form_values = AFW_HUB.FormUtil.get_all_form_values(form);
    AFW_HUB.FormUtil.round_form_inputs(form);
    main_callback.call({ form_values: form_values });
}
//
// this recalculates all of the fields on the receving form
function recalc_warehouse_receiving_form(form, main_callback) {
    var form_values, callback, name, sql_args, ot_callback, mph_callback,
        lead_receiver = document.getElementById('lead-receiver').value || '0';
    main_callback = main_callback || function() {};
    //
    if (this) {
        form_values = AFW_HUB.FormUtil.get_all_form_values(form);
        form_values.total = this.form_values.total;
        document.getElementById('total').value = this.form_values.total;
        AFW_HUB.FormUtil.round_form_inputs(form);
        main_callback.call({ form_values: form_values });
        return;
    }
    //
    // getting all form values and processing them based on data type
    form_values = AFW_HUB.FormUtil.get_all_form_values(form, {
        disabled_default: 0
    });
    //
    // doing initial calculations
    form_values.total_moves = 0;
    form_values.total_units = 0;
    for (name in form_values) {
        if (name.match(/^(?!total).+_moves$/)) {
            form_values.total_moves += form_values[name];
        }
        if (name.match(/^(?!total).+_units$/)) {
            form_values.total_units += form_values[name];
        }
    }
    //
    // doing production and pay
    form_values.prod_time = form_values.hours - form_values.indirect / 60;
    form_values.moves_hour = form_values.total_moves / form_values.prod_time;
    form_values.units_hour = form_values.total_units / form_values.prod_time;
    form_values.total = form_values.additional_pay - form_values.pay_deductions;
    //
    if (!isFinite(form_values.moves_hour)) {
        form_values.moves_hour = 0;
    }
    if (!isFinite(form_values.units_hour)) {
        form_values.units_hour = 0;
    }
    //
    // updating display values
    document.getElementById('prod-time').value = form_values.prod_time;
    document.getElementById('total-moves').value = form_values.total_moves;
    document.getElementById('total-units').value = form_values.total_units;
    document.getElementById('moves-hour').value = form_values.moves_hour;
    document.getElementById('units-hour').value = form_values.units_hour;
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'receiving_move_rates',
        order_by: [['minimum_mph', 'DESC']]
    };
    //
    // defining the callback to calculate OT and move pay
    // ot_callback = function() {
    //     recalc_warehouse_receiving_form.call(this, form, main_callback);
    // };
    //
    mph_callback = function(response) {
        calc_move_rate(form_values, response.data);
        // overtime can only relly be calc'd with a full week of data now,
        // probably should just drop it from here
        //calc_employee_overtime(form, form_values, ot_callback, null);
        recalc_warehouse_receiving_form.call({form_values: form_values}, form, main_callback);
    };
    //
    AFW_HUB.Ajax.fetch_db([sql_args], ['data'], 'misc_database', mph_callback);
}
//
// this handles all the the validation and calculations for the warehouse form
function recalc_warehouse_shipping_form(form, main_callback) {
    var form_values, sql_args, ot_callback, sel_callback,
        lead_picker = document.getElementById('lead-picker').value || '0';
    main_callback = main_callback || function() {};
    //
    if (this) {
        form_values = AFW_HUB.FormUtil.get_all_form_values(form);
        form_values.total = this.form_values.total;
        //
        document.getElementById('total').value = form_values.total;
        AFW_HUB.FormUtil.round_form_inputs(form);
        main_callback.call({ form_values: form_values });
        return;
    }
    //
    // getting all form values and processing them based on data type
    form_values = AFW_HUB.FormUtil.get_all_form_values(form, {
        disabled_default: 0
    });
    //
    // calculating some values
    form_values.prod_time = form_values.hours - form_values.indirect / 60;
    form_values.cases_hr = form_values.num_cases / form_values.prod_time;
    form_values.cases_hr = Math.ceil10(form_values.cases_hr, 0);
    form_values.total = (form_values.incentive_pay || 0) + form_values.additional_pay - form_values.pay_deductions;
    //
    document.getElementById('prod-time').value = form_values.prod_time;
    document.getElementById('cases-hr').value = form_values.cases_hr;
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'selection_rates',
        where: [
            ['area', 'LIKE', form_values.area],
        ],
        order_by: [['min_cases_hr', 'DESC']]
    };
    //
    // defining the callback to calculate OT and case pay
    // ot_callback = function() {
    //     recalc_warehouse_shipping_form.call(this, form, main_callback);
    // };
    //
    sel_callback = function(response) {
        calc_sel_rate(form_values, response.data);
        // ot pay has to be calculated at the report level since hourly pay needs averages
        //calc_employee_overtime(form, form_values, ot_callback, null);
        recalc_warehouse_shipping_form.call({form_values: form_values}, form, main_callback);

    };
    //
    AFW_HUB.Ajax.fetch_db([sql_args], ['data'], 'misc_database', sel_callback);
}
//
// this validates the backhaul form and calculates fields
function recalc_freight_backhaul_form(form, main_callback) {
    var max_num_fields = Number(document.getElementById('number-of-fields').value),
        total_freight_allowance = 0,
        total_pallet_charges = 0,
        freight_charge,
        total_savings,
        form_values,
        fieldset,
        i;
    main_callback = main_callback || function() {};
    //
    // getting initial set of form values
    form_values = AFW_HUB.FormUtil.get_all_form_values(form);
    freight_charge = form_values.freight_charge;
    //
    // checking all of the haul fields
    for (i = 0; i < max_num_fields + 1; i++) {
        fieldset = document.getElementById('haul-info-' + i);
        if (fieldset) {
            form_values = AFW_HUB.FormUtil.get_all_form_values(fieldset);
            if (form_values.po_status !== 'submitted') {
                continue;
            }
            //
            total_freight_allowance += form_values.freight_allowance;
            total_pallet_charges += form_values.pallet_charge;
        }
    }
    //
    // outputting data
    total_savings = total_freight_allowance - freight_charge - total_pallet_charges;
    document.getElementById('total-savings').value = total_savings;
    //
    // rounding displayed values and executing callback function
    form_values = AFW_HUB.FormUtil.get_all_form_values(form);
    AFW_HUB.FormUtil.round_form_inputs(form);
    main_callback.call({ form_values: form_values });
}
//
// calculates employee overtime pay
function calc_employee_overtime(form, form_values, callback, response) {
    //
    var curr_date = new Date(form_values.date + ' 00:00'),
        wk_start = new Date(form_values.date + ' 00:00'),
        from_ts,
        to_ts,
        sql_args,
        data,
        pay_rate = form_values.hourly_pay_rate,
        hours = Math.round10(form_values.hours, 2),
        total_hours = 0,
        hourly_amt = 0,
        i;
    //
    curr_date.setDate(curr_date.getDate() - 1);
    wk_start.setDate(wk_start.getDate() - wk_start.getDay());
    from_ts = wk_start.yyyymmdd();
    to_ts = curr_date.yyyymmdd();
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'employee_data',
        cols: ['emp_internal_id', 'date', 'hours', 'hourly_pay_rate'],
        where: [
            ['emp_internal_id', 'LIKE', form_values.emp_internal_id],
            ['date', 'BETWEEN', [from_ts, to_ts]],
            ['entry_status', 'LIKE', 'submitted']
        ]
    };
    //
    // updating total pay with returned data
    if (response) {
        data = response.wk_data;
        //
        // totaling hours for the week
        for (i = 0; i < data.length; i++) {
            total_hours += Number(data[i].hours);
        }
        //
        if (form_values.attendance == 'holiday') {
            hourly_amt = pay_rate * hours;
        } else if (total_hours >= 40) {
            hourly_amt = 1.5 * pay_rate * hours;
        } else if (total_hours + hours > 40) {
            hourly_amt = pay_rate * (40 - total_hours);
            hourly_amt += 1.5 * pay_rate * (total_hours + hours - 40);
        } else {
            hourly_amt = pay_rate * hours;
        }
        //
        form_values.total += hourly_amt;
        document.getElementById('hourly-pay').value = hourly_amt;
        //
        callback.call({ form_values: form_values });
        return;
    }
    //
    callback = calc_employee_overtime.bind(null, form, form_values, callback);
    AFW_HUB.Ajax.fetch_db([sql_args], ['wk_data'], null, callback);
}
//
// validates and calculates a selection rate for warehouse dayshift
function calc_move_rate(form_values, level_data) {
    var error = form_values.error_code,
        attend_error = form_values.attendance_error,
        hourly_rate,
        min_mph,
        i;
    //
    // calculating move pay unless there was an error, the column dollars_per_move
    // really should be renamed
    hourly_rate = form_values.hourly_pay_rate;
    if (!document.getElementById('edit-hourly-pay-rate').checked && level_data.length > 0) {
      hourly_rate = level_data[level_data.length - 1].dollars_per_move;
      if (attend_error == 'none' && error == '0') {
          //
          // determining hourly rate based on moves per hour
          for (i = 0; i < level_data.length; i++) {
              min_mph = Number(level_data[i].minimum_mph);
              if (form_values['moves_hour'] >= min_mph) {
                  hourly_rate = Number(level_data[i].dollars_per_move);
                  break;
              }
          }
      }
      form_values.hourly_pay_rate = hourly_rate;
    }
    form_values.hourly_amt = hourly_rate * Math.round10(form_values.hours, 2);
    document.getElementById('hourly-pay-rate').value = hourly_rate;
    document.getElementById('hourly-pay').value = form_values.hourly_amt;
    //
    // updating form_values
    form_values.total += form_values.hourly_amt;
}
//
// validates and calculates a selection rate for warehouse data
function calc_sel_rate(form_values, sel_data) {
    var min_cases_hr;
    // selection rate now is just a percentage of a defined
    // minimum and no longer effects pay automatically here
    if (sel_data.length !== 0) {
        form_values.sel_rate = form_values.cases_hr / sel_data[0].min_cases_hr
        document.getElementById('sel-rate').value = form_values.sel_rate
    }
}
//
// handles validation of a regular data form
function submit_employee_data_form(form, recalc_fun, action) {
    var errors,
        callback,
        form_values = (this && this.form_values) || null,
        act = 'modification',
        attendance = document.getElementById('attendance-error').value,
        error_code = document.getElementById('error-code').value,
        department = document.getElementById(form.id).department.value;
    //
    // checking if there is an error_code/ attendance_error and requiring comments
    document.getElementById('comments').dataset.skip_auto_validate = '';
    if (attendance === 'none' && error_code === '0') {
        document.getElementById('comments').dataset.skip_auto_validate = '1';
        AFW_HUB.Utilities.remove_class('boxshadow-red', 'comments');
    }
    errors = AFW_HUB.FormUtil.auto_validate(form);
    if (errors) {
        return;
    }
    //
    // if there are no errors performing final calculations
    if (!this) {
        callback = function submission_callback() {
            submit_employee_data_form.call(this, form, recalc_fun, action);
        };
        recalc_fun.call(null, form, callback);
        return;
    }
    //
    // checking submission type
    delete form_values.entry_status;
    form_values.data_entry_id = form_values.entry_id;
    form_values.employee_data_last_modified = Date.current_ts();
    form_values.employee_data_last_modified_by = window.sessionStorage.username;
    if (action === 'create') {
        act = 'creation';
        delete form_values.entry_id;
        delete form_values.internal_id;
        delete form_values.employee_data_last_modified;
        delete form_values.employee_data_last_modified_by;
        form_values.data_entry_id = '@entry_id';
        form_values.entering_user = window.sessionStorage.username;
        form_values.entry_status = 'submitted';
        form_values.entry_id_set_statement = '1';
        form_values['%insert_all%'] = '1';
    } else if (action === 'delete') {
        act = 'deletion';
        form_values.entry_status = 'deleted';
    } else if (action === 'restore') {
        act = 'restoration';
        form_values.entry_status = 'submitted';
    }
    if (department === 'general') {
        delete form_values.data_entry_id;
    }
    //
    if (!confirm('Confirm ' + act + ' of data entry.')) {
        recalc_fun(form);
        return;
    }
    //
    if (department === 'freight_backhaul') {
        form_values = process_freight_backhaul_form_data(act, form_values);
    }
    callback = reset_employee_data_form.bind(null, act);
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
//
// reorganizes form values to properly handle multiple POs
function process_freight_backhaul_form_data(act, form_values) {
    var max_fields = document.getElementById('number-of-fields').value,
        carrier = form_values.carrier,
        freight_charge = form_values.freight_charge,
        total_savings = form_values.total_savings,
        haul_values = {},
        fieldset,
        key,
        i;
    max_fields = Number(max_fields) + 1;
    form_values = [form_values];
    //
    for (i = 0; i < max_fields; i++) {
        fieldset = document.getElementById('haul-info-' + i);
        if (!fieldset) {
            continue;
        }
        haul_values = AFW_HUB.FormUtil.get_all_form_values(fieldset);
        haul_values.carrier = carrier;
        haul_values.freight_charge = freight_charge;
        haul_values.total_savings = total_savings;
        //
        if (haul_values.internal_id === '') {
            delete haul_values.internal_id;
            haul_values.data_entry_id = form_values[0].entry_id;
            haul_values.po_status = 'submitted';
        }
        //
        if (act === 'creation') {
            haul_values.data_entry_id = '@entry_id';
            haul_values.po_status = 'submitted';
        }
        //
        form_values.push(haul_values);
    }
    // unsetting any PO specific values
    fieldset = document.getElementById('haul-info-0');
    haul_values = AFW_HUB.FormUtil.get_all_form_values(fieldset, {
        disabled_default: 1
    });
    haul_values.carrier = carrier;
    haul_values.freight_charge = freight_charge;
    haul_values.total_savings = total_savings;
    delete form_values[0].data_entry_id;
    for (key in haul_values) {
        delete form_values[0][key];
    }
    //
    return form_values;
}
//
// resets the page after submitting an employee form
function reset_employee_data_form(act) {
    var update_button = document.getElementById('update-employee-data-table'),
        tab_clicked = document.getElementById('tab-clicked').value;

    //
    alert('Successful ' + act + ' of entry.');
    //
    // diverting logic if it is the logistics entry tab
    if (tab_clicked === 'freight_backhaul_entry') {
        setup_freight_logistics();
        return;
    }
    //
    if (act !== 'creation') {
        update_button.dispatchPrimaryEvent();
    }
    //
    if (document.getElementById('employee-table')) {
        document.getElementById('employee-table').dispatchPrimaryEvent();
    }
    if (document.getElementById('employee-data-table')) {
        document.getElementById('employee-data-table').dispatchPrimaryEvent();
    }
    document.getElementById('form-div').removeAll();
    document.getElementById('form-header').textContent = '';
}
//
// this function handles updates to a single PO
function update_haul_field(form, fieldset, action) {
    var form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        haul_values = AFW_HUB.FormUtil.get_all_form_values(fieldset),
        errors = false,
        act = 'modification',
        status,
        callback;
    //
    recalc_freight_backhaul_form(form);
    errors = AFW_HUB.FormUtil.auto_validate(fieldset);
    if (errors) {
        return;
    }
    if (!confirm('Confirm ' + act + ' of haul field.')) {
        recalc_freight_backhaul_form(form);
        return;
    }
    //
    // action specific settings
    delete haul_values.data_entry_id;
    if (action === 'create') {
        act = 'creation';
        delete haul_values.internal_id;
        haul_values.data_entry_id = form_values.entry_id;
        haul_values.po_status = 'submitted';
        form_values['%insert_all%'] = '1';
    } else if (action === 'delete') {
        act = 'deletion';
        haul_values.po_status = 'deleted';
    } else if (action === 'restore') {
        act = 'restoration';
        haul_values.po_status = 'submitted';
    }
    form_values = {
        employee_data_last_modified: Date.current_ts(),
        employee_data_last_modified_by: window.sessionStorage.username,
        entry_id: form_values.entry_id
    };
    form_values = [form_values, haul_values];
    status = haul_values.po_status;
    //
    // setting up reset callback
    callback = function() {
        var suff = fieldset.id.match(/\d+$/)[0];
        //
        alert('Sucessful ' + act + ' of haul entry.');
        //
        fieldset.querySelector('[name=po_status]').value = status;
        //
        if (status === 'submitted') {
            AFW_HUB.Utilities.remove_class('hidden-elm', 'del-haul-' + suff);
            AFW_HUB.Utilities.add_class('hidden-elm', 'res-haul-' + suff);
        } else {
            AFW_HUB.Utilities.remove_class('hidden-elm', 'res-haul-' + suff);
            AFW_HUB.Utilities.add_class('hidden-elm', 'del-haul-' + suff);
        }
        // updating data table and form
        document.getElementById('update-employee-data-table').dispatchPrimaryEvent();
        recalc_freight_backhaul_form(form);
    };
    //
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
