/* global AFW_HUB */
////////////////////////////////////////////////////////////////////////////////
//////////////       This file holds the functions that are          ///////////
//////////////       associated only with the marketing page         ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
//
/////////////////////// Marketing Primary Functions ////////////////////////////
//
// handles anything that should execute when the page loads
function marketing_page() {
    //
    // adding nav buttons
    var url_data = AFW_HUB.Navigation.parse_url(),
        nav_bar = document.getElementById('nav-container'),
        nav_buttons = {
            vendor_broker_setup: {
                text: 'Vendor/Broker Setup',
                onclick: vendor_broker_setup
            },
            contact_information: {
                text: 'Contact Information',
                onclick: vendor_broker_contact_info
            },
            marketing_commitment: {
                text: 'Update Marketing Commitment',
                onclick: update_marketing_commitment
            },
            services_performed: {
                text: 'Update Services Performed',
                onclick: update_marketing_services_performed
            },
            food_show: {
                text: 'Food Show Information',
                onclick: update_food_show_info
            },
            growth: { text: 'Growth Programs', onclick: update_growth },
            payments: { text: 'Vendor Payments', onclick: update_payments },
            overview: { text: 'Overview', onclick: create_dashboard }
        },
        button = document.createElementWithAttr('BUTTON', { class: 'big' }),
        span = document.createElementWithAttr('SPAN', { id: 'last-archived' }),
        i,
        btn,
        sql_args,
        callback;
    //
    i = 0;
    button.style.position = 'relative';
    for (btn in nav_buttons) {
        button = button.cloneNode();
        button.id = btn;
        //
        // adding event handler to reload page with sub-tab
        button.addEventListener(
            'click',
            function(sub_tab) {
                delete url_data['full_url'];
                delete url_data['url_data'];
                url_data.sub_tab = sub_tab;
                location.href = AFW_HUB.Navigation.build_url(url_data);
            }.bind(this, btn)
        );
        //
        button.textContent = nav_buttons[btn].text;
        nav_bar.appendChild(button);
        if (i == 3) {
            nav_bar.appendChild(document.createElement('BR'));
        }
        i++;
    }
    //
    // adding an archive button
    button = button.cloneNode();
    button.id = 'archive';
    button.textContent = 'Archive Data';
    button.addEventListener('click', archive_data.bind(null, true));
    span.className = 'nav-button-subtitle';
    span.textContent = 'Last Archived: 0000-00-00';
    button.appendChild(span);
    nav_bar.appendChild(button);
    //
    // updating last modified archive label
    sql_args = {
        cmd: 'SELECT',
        cols: [ 'archive_timestamp' ],
        table: 'marketing_data_archive',
        order_by: [ [ 'archive_timestamp', 'DESC' ] ],
        limit: [ '0', '1' ]
    };
    callback = function(response) {
        var timestamp = response.timestamp[0]['archive_timestamp'];
        timestamp = 'Last Archived: ' + timestamp;
        document.getElementById('last-archived').textContent = timestamp;
    };
    AFW_HUB.Ajax.fetch_db([ sql_args ], [ 'timestamp' ], null, callback);
    //
    // checking perms for each button
    for (btn in nav_buttons) {
        AFW_HUB.Navigation.check_user_perms('marketing.' + btn);
    }
    AFW_HUB.Navigation.check_user_perms('marketing.archive');
    //
    // checking url for sub_tab to navigate to, defaulting to overview if available
    if (url_data.hasOwnProperty('sub_tab')) {
        nav_buttons[url_data['sub_tab']].onclick.call(null);
    } else if (document.getElementById('overview')) {
        document.getElementById('overview').dispatchPrimaryEvent();
    }
}
//
// removes all grandchildren of the main-container
function clear_maincontainer() {
    //
    var childNodes = document.getElementById('main-container').childNodes;
    for (var i = 0; i < childNodes.length; i++) {
        if (childNodes[i].nodeType == 1) {
            childNodes[i].removeAll();
        }
    }
}
//
// sets the page for creation or modification of a vendor
function vendor_broker_setup() {
    //
    document.getElementById('tab-clicked').value = 'vendor_broker_setup';
    clear_maincontainer();
    //
    document.getElementById('page-header').textContent = 'Vendor Broker Setup';
    //
    create_vendor_broker_table_filter('input-div');
    //
    create_vendor_broker_table(1, 'broker', 'ASC');
}
//
// sets the page for creation or modifcation of contact information
function vendor_broker_contact_info() {
    //
    document.getElementById('tab-clicked').value = 'vendor_broker_contact_info';
    clear_maincontainer();
    //
    document.getElementById(
        'page-header'
    ).textContent = 'Vendor Broker Contact Information';
    //
    create_vendor_broker_table_filter('input-div');
    //
    create_contact_vendor_broker_table(1, 'broker', 'ASC');
}
//
// sets the page for the modification of a vendors food show info
function update_food_show_info() {
    //
    document.getElementById('tab-clicked').value = 'update_food_show_info';
    clear_maincontainer();
    //
    document.getElementById('page-header').textContent = 'Food Show Information';
    //
    create_food_show_table_filter('input-div');
    //
    create_food_show_table(1, 'broker', 'ASC');
}
//
// sets the page for updatin marketing informat
function update_marketing_commitment() {
    //
    //
    document.getElementById('tab-clicked').value = 'update_marketing_commitment';
    clear_maincontainer();
    //
    document.getElementById(
        'page-header'
    ).textContent = 'Marketing Commitment Information';
    //
    create_marketing_commitment_table_filter('input-div');
    //
    create_marketing_commitment_table(1, 'broker', 'ASC');
}
//
// sets the page for updating services performed
function update_marketing_services_performed() {
    //
    document.getElementById(
        'tab-clicked'
    ).value = 'update_marketing_services_performed';
    clear_maincontainer();
    //
    document.getElementById('page-header').textContent = 'Services Performed';
    //
    create_services_performed_table_filter('input-div');
    //
    create_services_performed_table(1, 'broker', 'ASC');
}
//
// sets the page for updating a vendors growth info
function update_growth() {
    //
    document.getElementById('tab-clicked').value = 'update_growth';
    clear_maincontainer();
    //
    document.getElementById('page-header').textContent = 'Growth Information';
    //
    create_growth_table_filter('input-div');
    //
    create_growth_table(1, 'broker', 'ASC');
}
//
// sets the page for updating/ adding a payment
function update_payments() {
    clear_maincontainer();
    //
    document.getElementById('tab-clicked').value = 'update_payments';
    document.getElementById('page-header').textContent = 'Payment Information';
    //
    create_payment_table_filter('input-div', null);
    create_payments_vend_table(1, 'broker', 'ASC');
}
//
// sets the page for viewing overall marketing data and reports
function create_dashboard() {
    //
    var input_div = document.getElementById('input-div'),
        table_div = document.getElementById('table-div'),
        content_div = document.getElementById('content-div'),
        params_fieldset = document.createElement('FIELDSET'),
        totals_fieldset = document.createElement('FIELDSET'),
        totals = [
            'marketing-level-commitment',
            'a-la-carte-commitment',
            'growth-program',
            'food-show-commitment',
            'miscellaneous-charges',
            'additional-fees',
            'overall-balance'
        ],
        fieldset = document.createElement('FIELDSET'),
        legend = document.createElement('LEGEND'),
        table = document.createElement('TABLE'),
        button = document.createElement('BUTTON'),
        update_onclick = show_update_button.bind(
            null,
            'create-report',
            'marketing-report',
            'Apply Changes'
        ),
        broker_dropbox_args,
        level_dropbox_args,
        elements,
        children,
        booth_opts,
        text,
        i;
    //
    // initializing page
    clear_maincontainer();
    document.getElementById('page-header').textContent = 'Marketing Dashboard';
    document.getElementById('tab-clicked').value = 'marketing_dashboard';
    //
    // styling main fieldsets to float beside each other
    table_div.style.clear = 'both';
    params_fieldset.style.width = '45%';
    params_fieldset.style.float = 'left';
    totals_fieldset.style.width = '45%';
    totals_fieldset.style.float = 'right';
    input_div.addNodes([ params_fieldset, totals_fieldset ]);
    //
    // creating the overview table
    //
    legend = legend.cloneNode();
    legend.textContent = 'Active Vendor Totals';
    totals_fieldset.addNodes([ legend, table ]);
    //
    // setting up header row
    children = [
        { elm: 'th' },
        { elm: 'th', textNode: 'Charges' },
        { elm: 'th', textNode: 'Payments' }
    ];
    elements = [ { elm: 'tr', children: children } ];
    //
    // adding row for each total
    for (i = 0; i < totals.length; i++) {
        text = 'Total ' + totals[i].replace(/-/g, ' ').toTitleCase();
        children = [
            { elm: 'td', id: 'total-' + totals[i], textNode: text },
            {
                elm: 'td',
                id: 'charge-' + totals[i],
                style: { color: 'rgb(235,0,0)' }
            },
            { elm: 'td', id: 'credit-' + totals[i] }
        ];
        elements.push({ elm: 'tr', children: children });
    }
    elements[elements.length - 1].style = { fontWeight: 'bold' };
    table.add_children(elements);
    calculate_dashboard_totals();
    //
    // setting up the input-div for reporting
    //
    elements = [
        {
            elm: 'h4',
            textNode: 'Report Type:',
            style: { marginTop: '5px', marginBottom: '5px' }
        },
        {
            elm: 'label',
            textNode: 'Accounting',
            style: { minWidth: 'auto', paddingLeft: '10px' }
        },
        {
            elm: 'input',
            type: 'radio',
            name: 'report-type',
            value: 'marketing-accounting'
        },
        {
            elm: 'label',
            textNode: 'Food Show',
            style: { minWidth: 'auto', paddingLeft: '10px' }
        },
        {
            elm: 'input',
            type: 'radio',
            name: 'report-type',
            value: 'marketing-food-show'
        },
        {
            elm: 'label',
            textNode: 'Services',
            style: { minWidth: 'auto', paddingLeft: '10px' }
        },
        {
            elm: 'input',
            type: 'radio',
            name: 'report-type',
            value: 'marketing-services'
        },
        {
            elm: 'h4',
            textNode: 'Report Parameters:',
            style: { marginTop: '10px', marginBottom: '5px' }
        }
    ];
    params_fieldset.add_children(elements);
    //
    // setting up param report elements, all initially disabled
    fieldset = fieldset.cloneNode();
    fieldset.id = 'report-parameters';
    fieldset.style.width = 'auto';
    fieldset.style.border = 'none';
    AFW_HUB.Utilities.add_class('hidden-elm', fieldset);
    children = [
        { elm: 'option', value: 'ASC', textNode: 'Ascending' },
        { elm: 'option', value: 'DESC', textNode: 'Descending' }
    ];
    booth_opts = [
        { elm: 'option', value: '', textNode: 'All' },
        { elm: 'option', value: '[1-9]', textNode: 'Any' },
        { elm: 'option', value: '^0$', textNode: 'None' },
        { elm: 'option', value: '0.5', textNode: 'Half' },
        { elm: 'option', value: '1.0', textNode: 'Full' }
    ];
    elements = [
        { elm: 'label', textNode: 'Preset Report' },
        {
            elm: 'select',
            id: 'preset-report',
            events: [
                { event: 'change', function: update_onclick },
                {
                    event: 'change',
                    function: marketing_report_data_columns.bind(null, false, true)
                }
            ]
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'Vendors Only' },
        {
            elm: 'input',
            type: 'checkbox',
            name: 'vendor',
            value: '.+',
            checked: true,
            events: [
                {
                    event: 'click',
                    function: function() {
                        AFW_HUB.FormUtil.toggle_checkbox_value(this, '.+', '.*');
                        update_onclick();
                    }
                }
            ]
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'Report Only Broker' },
        {
            elm: 'select',
            id: 'broker',
            name: 'broker',
            events: [ { event: 'change', function: update_onclick } ]
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'Marketing Level' },
        {
            elm: 'select',
            id: 'marketing-level',
            name: 'marketing_vendor_broker_table.marketing_level',
            events: [ { event: 'change', function: update_onclick } ]
        },
        {
            elm: 'br',
            class: 'hidden-elm',
            'data-report_type': 'marketing_food_show'
        },
        {
            elm: 'label',
            textNode: 'Booth Size',
            class: 'hidden-elm',
            'data-report_type': 'marketing_food_show'
        },
        {
            elm: 'select',
            id: 'booth-size',
            name: 'booth_size',
            class: 'hidden-elm',
            'data-report_type': 'marketing_food_show',
            disabled: true,
            children: booth_opts,
            events: [ { event: 'change', function: update_onclick } ]
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'Sort Direction' },
        {
            elm: 'select',
            id: 'sort-dir',
            name: 'sort-dir',
            children: children,
            events: [ { event: 'change', function: update_onclick } ]
        },
        { elm: 'br' },
        {
            elm: 'button',
            id: 'toggle-column-selection',
            class: 'medium',
            textNode: 'Show Column Selection',
            events: [
                {
                    event: 'click',
                    function: marketing_report_data_columns.bind(null, true, false)
                }
            ]
        },
        {
            elm: 'button',
            id: 'create-report',
            class: 'medium',
            textNode: 'Create Report',
            events: [ { event: 'click', function: init_marketing_report } ]
        },
        {
            elm: 'button',
            id: 'print-report',
            class: 'medium hidden-elm',
            textNode: 'Print Report',
            events: [ { event: 'click', function: window.print.bind(window) } ]
        }
    ];
    fieldset.add_children(elements);
    params_fieldset.appendChild(fieldset);
    //
    // adding event handlers to the report type buttons
    elements = params_fieldset.querySelectorAll('[name="report-type"]');
    for (i = 0; i < elements.length; i++) {
        elements[i].addEventListener(
            'click',
            function(container) {
                AFW_HUB.Utilities.remove_class('hidden-elm', container);
                create_report_type_params(this, container);
                marketing_report_data_columns(false, true);
            }.bind(elements[i], fieldset)
        );
    }
    //
    // adding additional page elements
    button = button.cloneNode();
    button.id = 'restore-defaults';
    button.textContent = 'Restore Defaults';
    button.addEventListener('click', function() {
        marketing_report_data_columns(false, true);
        show_update_button('create-report', 'marketing-report', 'Apply Changes');
        AFW_HUB.Utilities.add_class('hidden-elm', 'restore-defaults');
    });
    table_div.appendChild(document.createElement('BR'));
    table_div.appendChild(button);
    table_div.appendChild(document.createElement('BR'));
    content_div.appendChild(document.createElement('BR'));
    AFW_HUB.Utilities.add_class('hidden-elm', table_div);
    AFW_HUB.Utilities.add_class('hidden-elm', button);
    //
    // populating dropboxes
    broker_dropbox_args = {
        dropbox_id: 'broker',
        text_format: '%broker%',
        value_format: '%broker%',
        placeholder: 'All',
        placeholder_value: '.*',
        placeholder_disabled: false,
        placeholder_selected: true,
        sql_args: {
            table: 'marketing_vendor_broker_table',
            cols: [ 'TRIM(`broker`) AS `broker`' ],
            where: [ [ 'broker', 'REGEXP', '[0-9A-z]+' ] ],
            order_by: [ [ 'broker', 'ASC' ] ],
            group_by: 'broker'
        },
        add_opts_val: [ '^$' ],
        add_opts_text: [ 'Brokerless' ]
    };
    level_dropbox_args = {
        dropbox_id: 'marketing-level',
        text_format: '%marketing_level%',
        value_format: '%marketing_level%',
        placeholder: 'All',
        placeholder_value: '.*',
        placeholder_disabled: false,
        placeholder_selected: true,
        database: 'misc_database',
        sql_args: {
            table: 'marketing_tiers',
            cols: [ 'marketing_level' ],
            order_by: [ [ 'commitment', 'ASC' ] ]
        },
        add_opts_val: [ 'gold|silver|bronze' ],
        add_opts_text: [ 'Any' ]
    };
    AFW_HUB.FormUtil.populate_dropbox_options(broker_dropbox_args);
    AFW_HUB.FormUtil.populate_dropbox_options(level_dropbox_args);
}
//
// handles archival of all marketing
function archive_data(active_only) {
    //
    // if active_only is true only active vendors are
    // archived otherwise everything is selected
    //
    var cont, sql_args, sql_arr = [], key_arr = [];
    cont = confirm('Confirm Archival of Marketing Data.');
    if (!cont) {
        return;
    }
    //
    // create SQL statement
    sql_args = {
        cmd: 'SELECT',
        table: 'table_meta_data',
        cols: [ 'column_name', 'data_type', 'in_tables' ],
        where: [
            [ 'column_type', 'REGEXP', 'static' ],
            //['data_type', 'REGEXP', 'archive']
            [ 'in_tables', 'REGEXP', 'marketing.+' ]
        ]
    };
    sql_arr.push(sql_args);
    key_arr.push('meta_data');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'table_meta_data',
        cols: [ 'column_name' ],
        where: [
            [ 'in_tables', 'REGEXP', 'marketing_data_archive' ],
            [ 'column_name', 'REGEXP', 'marketing.+' ]
        ]
    };
    sql_arr.push(sql_args);
    key_arr.push('archive_cols');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_vendor_broker_table',
        inner_join: [
            [
                'marketing_accounting',
                'marketing_vendor_broker_table.internal_id',
                'marketing_accounting.vendor_internal_id'
            ],
            [
                'marketing_alacarte_commitment',
                'marketing_vendor_broker_table.internal_id',
                'marketing_alacarte_commitment.vendor_internal_id'
            ],
            [
                'marketing_food_show',
                'marketing_vendor_broker_table.internal_id',
                'marketing_food_show.vendor_internal_id'
            ],
            [
                'marketing_services_performed',
                'marketing_vendor_broker_table.internal_id',
                'marketing_services_performed.vendor_internal_id'
            ]
        ],
        where: []
    };
    if (active_only) {
        sql_args.where.push([
            'marketing_vendor_broker_table.vendor_status',
            'LIKE',
            'active'
        ]);
    }
    sql_arr.push(sql_args);
    key_arr.push('data');
    //
    sql_args = {
        cmd: 'SELECT',
        cols: [ 'marketing_growth.*' ],
        table: 'marketing_growth',
        inner_join: [
            [
                'marketing_vendor_broker_table',
                'marketing_vendor_broker_table.internal_id',
                'marketing_growth.vendor_internal_id'
            ]
        ],
        where: []
    };
    if (active_only) {
        sql_args.where.push([
            'marketing_vendor_broker_table.vendor_status',
            'LIKE',
            'active'
        ]);
        sql_args.where.push([ 'marketing_growth.program_status', 'LIKE', 'active' ]);
    }
    sql_arr.push(sql_args);
    key_arr.push('growth_data');
    //
    sql_args = {
        cmd: 'SELECT',
        cols: [ 'marketing_payments.*' ],
        table: 'marketing_payments',
        inner_join: [
            [
                'marketing_vendor_broker_table',
                'marketing_vendor_broker_table.internal_id',
                'marketing_payments.vendor_internal_id'
            ]
        ],
        where: []
    };
    if (active_only) {
        sql_args.where.push([
            'marketing_vendor_broker_table.vendor_status',
            'LIKE',
            'active'
        ]);
        sql_args.where.push([
            'marketing_payments.payment_status',
            'LIKE',
            'active'
        ]);
    }
    sql_arr.push(sql_args);
    key_arr.push('payment_data');
    //
    AFW_HUB.Ajax.fetch_db(sql_arr, key_arr, null, create_archive_upload);
}
//
////////////////////// Marketing Secondary Functions ///////////////////////////
//
// handles vendor broker setup form generation
function vendor_broker_setup_form(internal_id, row_id, form) {
    var content_div = document.getElementById('content-div'),
        header = document.getElementById('content-header'),
        sub_button = document.getElementById('submit-vendor-broker-form'),
        sub_func = function(act) {
            submit_vendor_broker_form(form, act);
        },
        dropbox_args, form_args, row_data,
        args, button, callback;
    //
    // loading form
    if (!form) {
        callback = vendor_broker_setup_form.bind(null, internal_id, row_id);
        args = ['setup_vendor_broker', content_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null, args);
        return;
    }
    //
    // setting up args to populate form
    form_args = {
        form_id: form.id,
        trigger_events: true,
        meta_data_only: 1,
        sql_args: {
            where: [
                ['marketing_vendor_broker_table.internal_id',
                 'LIKE',
                 internal_id]
            ]
        },
        data_preprocessor: function(args, response) {
            var event_args = {meta_array: response.meta_data};
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
            sub_button.addEventListener('click', sub_func.bind(null, 'creation'));
        }
    };
    dropbox_args = {
        dropbox_id: 'broker',
        text_format: '%broker%',
        value_format: '%broker%',
        placeholder: 'Select Broker',
        sql_args: {
            cmd: 'SELECT',
            table: 'marketing_vendor_broker_table',
            cols: [ 'DISTINCT(TRIM(`broker`)) AS `broker`' ],
            where: [ [ 'TRIM(`broker`)', 'NOT REGEXP', '^$' ] ],
            order_by: [ [ 'broker', 'ASC' ] ]
        },
        add_opts_val: [ 'other' ],
        add_opts_text: [ 'Other' ],
        add_callback: function() {
            AFW_HUB.FormUtil.populate_form(form_args);
        }
    };
    //
    // additional setup for vendor creation or modficaton
    if (internal_id !== null) {
        row_data = document.getElementById(row_id).dataset;
        row_data.broker = row_data.broker || '(None)';
        row_data.vendor = row_data.vendor || '(None)';
        header.textContent = 'Modifying  Broker: {broker} - Vendor: {vendor}';
        header.textContent = header.textContent.format(row_data);
        //
        // adding buttons
        button = sub_button.cloneNode();
        button.textContent = 'Modify Vendor/Broker';
        button.addEventListener('click', sub_func.bind(null, 'modification'));
        form.safeAppendChild(button);
        //
        button = sub_button.cloneNode();
        button.id = 'del-button';
        button.textContent = 'Delete Vendor/Broker';
        button.addEventListener('click', sub_func.bind(null, 'deletion'));
        form.safeAppendChild(button);
        //
        button = sub_button.cloneNode();
        button.id = 'res-button';
        button.textContent = 'Restore Vendor/Broker';
        button.addEventListener('click', sub_func.bind(null, 'restoration'));
        form.safeAppendChild(button);
        //
        form_args.meta_data_only = undefined;
        form_args.add_callback = function(args) {
            //
            if (args.data_arr.vendor_status === 'active') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'res-button');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'del-button');
            }
        };
    }
    //
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
}
//
// handles initial generation of the contact info form
function create_contact_info_form(contact_internal_id, vendor_id, row_id, form) {
    var content_div = document.getElementById('content-div'),
        header = document.getElementById('content-header'),
        sub_button = document.getElementById('submit-contact-info-form'),
        toggle_cb = document.getElementById('toggle-vendor'),
        toggle_func = AFW_HUB.FormUtil.disable_if_checked,
        sub_func = function(act) {
            submit_marketing_contact_info_form(form, act);
        },
        form_args, dropbox_args, row_data, args, callback, elm;
    //
    // loading form
    if (!form) {
        callback = create_contact_info_form;
        callback = callback.bind(null, contact_internal_id, vendor_id, row_id);
        args = ['marketing_contact_info_form', content_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null, args);
        return;
    }
    //
    // adding events to toggle vendor editability
    elm = document.getElementById('vendor-select');
    toggle_cb.addEventListener('click', toggle_func.bind(null, toggle_cb, elm, true));
    elm = document.getElementById('vendor-internal-id');
    toggle_cb.addEventListener('click', toggle_func.bind(null, toggle_cb, elm, false));
    //
    // populating vendor change dropbox
    dropbox_args = {
        dropbox_id: 'vendor-select',
        placeholder: 'Select Vendor',
        text_format: '%broker% - %vendor%',
        value_format: '%internal_id%',
        sql_args: {
            cmd: 'SELECT',
            table: 'marketing_vendor_broker_table',
            cols: [ 'broker', 'vendor', 'internal_id' ],
            where: [ [ 'vendor_status', 'LIKE', 'active' ] ],
            order_by: [ [ 'broker', 'ASC' ], [ 'vendor', 'ASC' ] ]
        }
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    //
    // setting up form args
    form_args = {
        form_id: form.id,
        trigger_events: true,
        sql_args: {
            where: [
                [
                    'marketing_contact_information.contact_internal_id',
                    'LIKE',
                    contact_internal_id
                ]
            ]
        },
        data_preprocessor: function(args, response) {
            var event_args = {meta_array: response.meta_data};
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
            sub_button.addEventListener('click', sub_func.bind(null, 'creation'));
        }
    };
    //
    // setting page if it is a modification
    if (contact_internal_id !== null) {
        row_data = document.getElementById(row_id).dataset;
        header.textContent = "Updating {contact_name}'s Information".format(row_data);
        //
        // adding buttons
        elm = sub_button.cloneNode();
        elm.textContent = 'Modify Contact';
        elm.addEventListener('click', sub_func.bind(null, 'modification'));
        form.safeAppendChild(elm);
        //
        elm = sub_button.cloneNode();
        elm.id = 'del-button';
        elm.textContent = 'Delete Contact';
        elm.addEventListener('click', sub_func.bind(null, 'deletion'));
        form.safeAppendChild(elm);
        //
        elm = sub_button.cloneNode();
        elm.id = 'res-button';
        elm.textContent = 'Restore Contact';
        elm.addEventListener('click', sub_func.bind(null, 'restoration'));
        form.safeAppendChild(elm);
        //
        form_args.add_callback = function(args) {
            //
            if (args.data_arr.contact_status !== 'active') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'del-button');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'res-button');
            }
        };
        AFW_HUB.FormUtil.populate_form(form_args);
    } else {
        args = {
            cmd: 'SELECT',
            cols: ['broker', 'vendor', '`internal_id` AS `vendor_internal_id`'],
            table: 'marketing_vendor_broker_table',
            where: [
                ['internal_id', 'LIKE', vendor_id]
            ]
        };
        callback = function(response) {
            form_args.data_preprocessor = function(args, resp) {
                resp.form_data = response.form_data;
                args.meta_data_only = undefined;
                var event_args = {meta_array: resp.meta_data};
                AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
                sub_button.addEventListener('click', sub_func.bind(null, 'creation'));
            };
            AFW_HUB.FormUtil.populate_form(form_args);
        };
        form_args.meta_data_only = 1;
        AFW_HUB.Ajax.fetch_db([args], ['form_data'], null, callback);
    }
}
//
//
function create_food_show_form(vendor_internal_id, row_id, form) {
    var content_div = document.getElementById('content-div'),
        header = document.getElementById('content-header'),
        row_data = document.getElementById(row_id).dataset,
        msg = 'Updating  Broker: {broker} - Vendor: {vendor} Food Show Information',
        set_suggested_price, callback, form_args, event_args, args;
    //
    set_suggested_price = function() {
        var size = document.getElementById('booth-size').value,
            fee = Number.parse(document.getElementById('food-show-booth-fee').value);
        //
        if (size === '0') {
            fee = 0;
        } else if (size === '0.5') {
            fee = 1100;
        }
        fee = Math.round10(fee, 2, true);
        document.getElementById('suggested-booth-price').value = '$ ' + fee;
    };
    //
    // loading form
    if (!form) {
        callback = create_food_show_form.bind(null, vendor_internal_id, row_id);
        args = ['marketing_food_show_form', content_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null, args);
        return;
    }
    //
    // setting page header
    row_data['broker'] = row_data['broker'] || '(None)';
    row_data['vendor'] = row_data['vendor'] || '(None)';
    header.textContent = msg.format(row_data);
    //
    // setting up args to add event listeners and populate form
    event_args = {
        add_default_events: function(elm) {
            var event = elm.primaryEventString(),
                remove_class = AFW_HUB.Utilities.remove_class,
                validate = validate_marketing_food_show_form;
            //
            if (elm.nodeName.toLowerCase() == 'button') {
                return;
            }
            if (!event) {
                return;
            }
            //
            elm.addEventListener(
                event,
                remove_class.bind(null, 'boxshadow-red', elm)
            );
            elm.addEventListener('blur', validate.bind(null, form));
            //
            if (elm.id === 'booth-size') {
                elm.addEventListener(event, set_suggested_price);
            }
        }
    };
    //
    form_args = {
        form_id: form.id,
        trigger_events: true,
        sql_args: {
            where: [
                [
                    'marketing_vendor_broker_table.internal_id',
                    'LIKE',
                    vendor_internal_id
                ]
            ]
        },
        data_preprocessor: function(args, response) {
            var sub_button = document.getElementById('submit-food-show-form'),
                sub_func = submit_marketing_food_show_form.bind(null ,form);
            event_args.meta_array = response.meta_data;
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
            sub_button.addEventListener('click', sub_func);
        },
        add_callback: function(args) {
            var sql_args = {
                    cmd: 'SELECT',
                    cols: ['food_show_booth_fee'],
                    table: 'marketing_tiers',
                    where: [
                        ['marketing_level', 'LIKE', args.data_arr.marketing_level]
                    ]
                },
                fetch_cb = function(response) {
                    var elm = document.getElementById('food-show-booth-fee');
                    elm.value = response[0][0].food_show_booth_fee;
                    set_suggested_price();
                    validate_marketing_food_show_form(form);
                };
            AFW_HUB.Ajax.fetch_db([sql_args], [0], 'misc_database', fetch_cb);
        }
    };
    AFW_HUB.FormUtil.populate_form(form_args);
}
//
//
function create_growth_form(vendor_internal_id, row_id, form) {
    var content_div = document.getElementById('content-div'),
        header = document.getElementById('content-header'),
        hide_button = document.getElementById('hide-growth-form'),
        row_data = document.getElementById(row_id).dataset,
        msg = 'Updating  Broker: {broker} - Vendor: {vendor}, Growth Information',
        args, form_args, callback;
    //
    // loading form
    if (!form) {
        callback = create_growth_form.bind(null, vendor_internal_id, row_id);
        args = ['marketing_growth_form', content_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null ,args);
        return;
    }
    //
    // setting page header
    row_data['broker'] = row_data['broker'] || '(None)';
    row_data['vendor'] = row_data['vendor'] || '(None)';
    header.textContent = msg.format(row_data);
    //
    // setting up args to populate the form
    form_args = {
        form_id: form.id,
        trigger_events: true,
        sql_args: {
            where: [
                ['marketing_vendor_broker_table.internal_id',
                 'LIKE',
                 vendor_internal_id]
            ]
        },
        data_preprocessor: function(args, response) {
            var add_button = document.getElementById('add-growth-fieldset'),
                func;
            // spoofing vendor data if there isn't at least one growth program
            if (response.form_data.length === 0) {
                response.form_data = [Object.clone(row_data)];
                response.form_data[0].vendor_internal_id = row_data.internal_id;
                response.all_form_data = [];
            }
            else {
                response.all_form_data = response.form_data;
                response.form_data = [response.form_data[0]];
            }
            //
            add_growth_form_events(form, form, response);
            func = create_growth_fieldset.bind(null, form, response);
            add_button.addEventListener('click', func);
        },
        add_callback: function(args, form, response) {
            var event = document.createEvent('HTMLEvents'),
                i;
            event.initEvent('blur', true, false);
            args.form_id = undefined;
            args.data_preprocessor = function() {};
            args.add_callback = function() {};
            document.getElementById('growth-paid').value = row_data.growth_paid;
            //
            for (i = 0; i < response.all_form_data.length; i++) {
                response.form_data = [response.all_form_data[i]];
                args.form = create_growth_fieldset(form, response);
                AFW_HUB.FormUtil.populate_form(args, response);
                mod_growth_program_buttons(form, args.form);
            }
            //
            event.initEvent('blur', true, false);
            document.getElementById('growth-remaining').dispatchEvent(event);
        }
    };
    //
    // adding event to clear form from page and force a table update
    hide_button.addEventListener('click', function() {
        document.getElementById('vendor-broker-table').dispatchPrimaryEvent();
        document.getElementById('content-div').removeAll();
        document.getElementById('content-header').textContent = '';
    });
    AFW_HUB.FormUtil.populate_form(form_args);
}
//
//
function create_payment_form(vendor_internal_id, payment_internal_id, row_id, form) {
    var header = document.getElementById('content-header'),
        content_div = document.getElementById('content-div'),
        sub_button = document.getElementById('submit-form'),
        sub_func = function(act) {
            submit_payment_form(form, act);
        },
        row_data = document.getElementById(row_id).dataset,
        msg = 'Updating Broker: {broker} - Vendor: {vendor}, Payment Information',
        args, event_args, form_args, button, callback;
    //
    // loading form
    if (!form) {
        callback = create_payment_form.bind(
            null,
            vendor_internal_id,
            payment_internal_id,
            row_id
        );
        args = ['marketing_payments_form', content_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null, args);
        return;
    }
    //
    // setting page header
    row_data.broker = row_data.broker || '(None)';
    row_data.vendor = row_data.vendor || '(None)';
    header.textContent = msg.format(row_data);
    //
    // aync populating account balance fields
    populate_account_balance_fields(form, vendor_internal_id, undefined);
    //
    // adding event handlers
    event_args = {
        add_default_events: function(elm) {
            var event = elm.primaryEventString();
            if (elm.nodeName.toLowerCase() == 'button') {
                return;
            }
            if (!event) {
                return;
            }
            //
            elm.addEventListener(
                event,
                AFW_HUB.Utilities.remove_class.bind(null, 'boxshadow-red', elm)
            );
            elm.addEventListener(
                'blur',
                AFW_HUB.FormUtil.auto_validate.bind(null, form)
            );
            elm.addEventListener(
                'blur',
                AFW_HUB.FormUtil.round_form_inputs.bind(null, form, 2)
            );
        }
    };
    //
    // setting up args to populate the data form
    form_args = {
        form_id: form.id,
        trigger_events: true,
        sql_args: {
            where: [
                ['marketing_payments.payment_internal_id',
                 'LIKE',
                 payment_internal_id]
            ]
        },
        data_preprocessor: function(args, response) {
            if (response.form_data.length === 0) {
                response.form_data = [Object.clone(row_data)];
                response.form_data[0].vendor_internal_id = row_data.internal_id;
                document.getElementById('date').value = new Date().yyyymmdd();
            }
            event_args.meta_array = response.meta_data;
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
            sub_button.addEventListener('click', sub_func.bind(null, 'creation'));
        }
    };
    if (payment_internal_id) {
        button = sub_button.cloneNode();
        button.textContent = 'Modify Payment';
        button.addEventListener('click', sub_func.bind(null, 'modification'));
        form.replaceChild(button, sub_button);
        //
        button = sub_button.cloneNode();
        button.id = 'del-button';
        button.textContent = 'Delete Payment';
        button.addEventListener('click', sub_func.bind(null, 'deletion'));
        form.safeAppendChild(button);
        //
        button = sub_button.cloneNode();
        button.id = 'res-button';
        button.textContent = 'Restore Payment';
        button.addEventListener('click', sub_func.bind(null, 'restoration'));
        form.safeAppendChild(button);
        //
        form_args.add_callback = function(args) {
            var event = document.createEvent('HTMLEvents');
            event.initEvent('blur', true, false);
            document.getElementById('amount').dispatchEvent(event);
            //
            if (args.data_arr.payment_status === 'active') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'res-button');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'del-button');
            }
        };
    }
    AFW_HUB.FormUtil.populate_form(form_args);
}
//
//
function create_marketing_commitment_form(vendor_internal_id, row_id, form) {
    var header = document.getElementById('content-header'),
        content_div = document.getElementById('content-div'),
        row_data = document.getElementById(row_id).dataset,
        msg = 'Updating  Broker: {broker} - Vendor: {vendor}, Marketing Commitment',
        sql_args, args, event_args, form_args, dropbox_args, callback;
    //
    // loading form
    if (!form) {
        callback = create_marketing_commitment_form;
        callback = callback.bind(null, vendor_internal_id, row_id);
        args = ['marketing_commitment_form', content_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null, args);
        return;
    }
    //
    // setting header
    row_data.broker = row_data.broker || '(None)';
    row_data.vendor = row_data.vendor || '(None)';
    header.textContent = msg.format(row_data);
    //
    // creating args to populate form
    event_args = {
        add_default_events: function(elm) {
            var event = elm.primaryEventString(),
                rem_class = AFW_HUB.Utilities.remove_class,
                validate = validate_marketing_commitment_form;
            if (elm.nodeName.toLowerCase() === 'button') {
                return;
            }
            if (!event) {
                return;
            }
            //
            elm.addEventListener(event, rem_class.bind(null, 'boxshadow-red', elm));
            elm.addEventListener('blur', validate.bind(null, form, true));
        }
    };
    form_args = {
        form_id: form.id,
        trigger_events: true,
        sql_args: {
            where: [[
                'marketing_vendor_broker_table.internal_id',
                'LIKE',
                vendor_internal_id
            ]]
        },
        add_callback: function() {
            update_commitment_field(form);
            validate_marketing_commitment_form(form, true);
        }
    };
    dropbox_args =  {
        dropbox_id: 'marketing-level',
        text_format: '%marketing_level%',
        value_format: '%marketing_level%',
        placeholder: 'Select Level',
        database: 'misc_database',
        sql_args: {
            cmd: 'SELECT',
            table: 'marketing_tiers',
            cols: [ 'marketing_level' ],
            order_by: [ [ 'marketing_level', 'ASC' ] ]
        },
        add_callback: function() {
            AFW_HUB.FormUtil.populate_form(form_args);
        },
    };
    //
    // fetching alacarte price data and then populating the level dropbox
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_alacarte_prices',
        cols: [ 'service', 'cost' ]
    };
    callback = function(response) {
        form_args.data_preprocessor = function(args, form_response) {
            var sub_button = document.getElementById('submit-marketing-commitment-form'),
                level_inp = document.getElementById('marketing-level'),
                sub_func = submit_marketing_commitment_form.bind(null, form),
                func = update_commitment_field.bind(null, form, undefined);
            form_response.price_data = response.price_data;
            event_args.meta_array = form_response.meta_data;
            //
            create_alacarte_fields(form, form_response);
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
            sub_button.addEventListener('click', sub_func);
            level_inp.addEventListener('change', func);
        };
        //
        AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    };
    AFW_HUB.Ajax.fetch_db([sql_args], ['price_data'], 'misc_database', callback);
}
//
//
function create_services_performed_form(vendor_internal_id, row_id, form) {
    var header = document.getElementById('content-header'),
        content_div = document.getElementById('content-div'),
        row_data = document.getElementById(row_id).dataset,
        msg = 'Updating  Broker: {broker} - Vendor: {vendor}, Services Performed Information',
        form_args,
        event_args,
        sql_args,
        sql_arr = [],
        key_arr = [],
        args,
        callback;
    //
    // loading form
    if (!form) {
        callback = create_services_performed_form;
        callback = callback.bind(null, vendor_internal_id, row_id);
        args = ['services_performed_form', content_div, callback];
        AFW_HUB.FormUtil.load_html_form.apply(null, args);
        return;
    }
    //
    // setting page header
    row_data.broker = row_data.broker || '(None)';
    row_data.vendor = row_data.vendor || '(None)';
    header.textContent = msg.format(row_data);
    //
    // creating sql args
    event_args = {
        add_default_events: function(elm) {
            var event = elm.primaryEventString(),
                rem_class = AFW_HUB.Utilities.remove_class.bind(
                    null,
                    'boxshadow-red',
                    elm
                ),
                validate = validate_services_performed_form.bind(null, form);
            if (elm.nodeName.toLowerCase() == 'button') {
                return;
            }
            if (!event) {
                return;
            }
            //
            elm.addEventListener(event, rem_class);
            elm.addEventListener('blur', validate);
        }
    };
    form_args = {
        form_id: form.id,
        trigger_events: true,
        sql_args: {
            where: [
                [
                    'marketing_vendor_broker_table.internal_id',
                    'LIKE',
                    vendor_internal_id
                ]
            ]
        },
        add_callback: function() {
            validate_services_performed_form(form);
        }
    };
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_alacarte_commitment',
        where: [['vendor_internal_id', 'LIKE', vendor_internal_id]]
    };
    sql_arr.push([sql_args]);
    key_arr.push(['alacarte_data']);
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_tiers',
    };
    sql_arr.push([sql_args]);
    key_arr.push(['marketing_level_data']);
    //
    // setting callback to emulate args passed to populate_form
    callback = function(response) {
        form_args.data_preprocessor = function(args, form_response) {
            var level_data = response.marketing_level_data;
            level_data = level_data.to_object('marketing_level');
            level_data = level_data[form_response.form_data[0].marketing_level];
            //
            form_response.alacarte_data = response.alacarte_data[0];
            form_response.marketing_level_data = level_data;
            event_args.meta_array = form_response.meta_data;
            //
            create_service_performed_fields(form, form_response);
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
            document.getElementById('submit-form')
                .addEventListener('click',
                submit_services_performed_form.bind(null, form));
        };
        AFW_HUB.FormUtil.populate_form(form_args);
    };
    //
    AFW_HUB.Ajax.fetch_db(sql_arr, key_arr, [null, 'misc_database'], callback);
}
//
// generates a marketing payments subqery and then left joins it.
function gen_marketing_payments_subquery(args, main_sql_args, main_cols) {
    var first_bns_day = new Date(
        AFW_HUB.get_constant('FIRST_BUSINESS_DAY')
    ).yyyymmdd(),
        sub_query_args = {
            cmd: 'SELECT',
            table: 'marketing_payments',
            cols: [
                'vendor_internal_id',
                'SUM(`amount`) AS `{}`'.format(args.amount_alias)
            ],
            where: [
                [ 'date', '>=', first_bns_day ],
                [ 'payment_status', 'LIKE', 'active' ],
                [ 'payment_type', 'REGEXP', args.pay_type_regexp ]
            ],
            group_by: 'vendor_internal_id'
        },
        key = '%{}%'.format(args.data_alias),
        sub_query = '{} AS `{}`'.format(key, args.data_alias),
        output_col = 'COALESCE({data_alias}.{amount_alias},0) AS `{amount_alias}`',
        main_table = args.main_table || main_sql_args.table;
    //
    sub_query = [
        sub_query,
        '{}.internal_id'.format(main_table),
        '{}.vendor_internal_id'.format(args.data_alias)
    ];

    main_cols.push(output_col.format(args));
    main_sql_args.sub_query[key] = sub_query_args;
    main_sql_args.left_join.push(sub_query);
}
//
// calculates all of the overall marketing totals
function calculate_dashboard_totals() {
    //
    var sql_args = {
            cmd: 'SELECT',
            cols: [
                'SUM(`level_commitment`) AS `charge-marketing-level-commitment`',
                'SUM(`misc_charges`) AS `charge-miscellaneous-charges`',
                'SUM(`additional_fees`) AS `charge-additional-fees`',
                'SUM(`amount_committed`) AS `charge-a-la-carte-commitment`',
                'SUM(`booth_price`+`electric_fee`+`allowance`) AS `charge-food-show-commitment`',
                'SUM(volume_total + purchases_total + unit_total) AS `charge-growth-program`',
                'SUM(credit_a_la_carte_commitment) AS `credit-a-la-carte-commitment`',
                'SUM(credit_food_show_commitment) AS `credit-food-show-commitment`',
                'SUM(credit_growth_program) AS `credit-growth-program`',
                'SUM(credit_marketing_level_commitment) AS `credit-marketing-level-commitment`',
                'SUM(credit_miscellaneous_charges) AS `credit-miscellaneous-charges`',
                'SUM(credit_additional_fees) AS `credit-additional-fees`'
            ],
            table: 'marketing_vendor_broker_table',
            inner_join: [
                [
                    'marketing_alacarte_commitment',
                    'marketing_vendor_broker_table.internal_id',
                    'marketing_alacarte_commitment.vendor_internal_id'
                ],
                [
                    'marketing_food_show',
                    'marketing_vendor_broker_table.internal_id',
                    'marketing_food_show.vendor_internal_id'
                ],
                [
                    'marketing_accounting',
                    'marketing_vendor_broker_table.internal_id',
                    'marketing_accounting.vendor_internal_id'
                ]
            ],
            left_join: [],
            sub_query: {},
            where: [
            [ 'marketing_vendor_broker_table.vendor_status', 'LIKE', 'active' ]
            ]
        },
        args,
        callback;
    //
    args = {
        cmd: 'SELECT',
        table: 'marketing_growth',
        cols: [
            'SUM(`volume_percent`/100.0 * `ytd_volume`) AS `volume_total`',
            'SUM(`purchase_percent`/100.0 * `ytd_purchases`) AS `purchases_total`',
            'SUM(`dollars_per_unit` * `ytd_units`) AS `unit_total`',
            'marketing_growth.vendor_internal_id'
        ],
        where: [ [ 'program_status', 'LIKE', 'active' ] ],
        group_by: 'marketing_growth.vendor_internal_id'
    };
    sql_args.sub_query['%growth_totals_sub%'] = args;
    sql_args.left_join.push([
        '%growth_totals_sub% AS `growth_totals`',
        'marketing_vendor_broker_table.internal_id',
        'growth_totals.vendor_internal_id'
    ]);
    //
    // building payments SQL sub queries
    args = {
        amount_alias: 'credit_a_la_carte_commitment',
        data_alias: 'a_la_carte_payments',
        pay_type_regexp: '(^a_la_carte$)'
    };
    gen_marketing_payments_subquery(args, sql_args, []);
    //
    args = {
        amount_alias: 'credit_food_show_commitment',
        data_alias: 'food_show_payments',
        pay_type_regexp: '(^food_show$)|(^food_show_allowance$)'
    };
    gen_marketing_payments_subquery(args, sql_args, []);
    //
    args = {
        amount_alias: 'credit_growth_program',
        data_alias: 'growth_payments',
        pay_type_regexp: '(^growth$)'
    };
    gen_marketing_payments_subquery(args, sql_args, []);
    //
    args = {
        amount_alias: 'credit_marketing_level_commitment',
        data_alias: 'marketing_level_payments',
        pay_type_regexp: '(^marketing_level$)'
    };
    gen_marketing_payments_subquery(args, sql_args, []);
    //
    args = {
        amount_alias: 'credit_miscellaneous_charges',
        data_alias: 'miscellaneous_payments',
        pay_type_regexp: '(^miscellaneous)'
    };
    gen_marketing_payments_subquery(args, sql_args, []);
    //
    args = {
        amount_alias: 'credit_additional_fees',
        data_alias: 'additional_fees_payments',
        pay_type_regexp: '(^additional_fees)'
    };
    gen_marketing_payments_subquery(args, sql_args, []);
    //
    callback = function(response) {
        var data_totals = response.data_totals[0], total, element, args, fmt_str;
        //
        // calculating overall totals
        data_totals['charge-overall-balance'] = 0;
        data_totals['credit-overall-balance'] = 0;
        for (total in data_totals) {
            if (total.match(/(?!^.*overall)^charge/)) {
                data_totals['charge-overall-balance'] += Number(data_totals[total]);
            } else if (total.match(/(?!^.*overall)^credit/)) {
                data_totals['credit-overall-balance'] += Number(data_totals[total]);
            }
        }
        //
        // outputting the appropriate totals
        for (total in data_totals) {
            element = document.getElementById(total);
            fmt_str = '%value%';
            if (total.match(/^charge/)) {
                fmt_str = '(%value%)';
            }
            args = [
                data_totals[total],
                'monetary',
                element,
                { format_str: fmt_str, add_commas: true }
            ];
            AFW_HUB.TableUtil.display_table_data.apply(null, args);
        }
    };
    //
    AFW_HUB.Ajax.fetch_db([ sql_args ], [ 'data_totals' ], null, callback);
}
//
// handles generation of additional report parameters
function create_report_type_params(radio_button, container) {
    //
    var report_type = radio_button.value.replace(/-/g, '_'),
        elements = container.allSubNodes(),
        i;
    //
    // unhiding/enabling elements based on report type
    for (i = 0; i < elements.length; i++) {
        if (!elements[i].dataset.report_type) {
            continue;
        }
        if (elements[i].dataset.report_type !== report_type) {
            elements[i].disabled = true;
            AFW_HUB.Utilities.add_class('hidden-elm', elements[i]);
        } else {
            elements[i].disabled = false;
            AFW_HUB.Utilities.remove_class('hidden-elm', elements[i]);
        }
    }
    //
    // updating button CSS classes and removing report
    AFW_HUB.Utilities.remove_class('hidden-elm', 'create-report');
    AFW_HUB.Utilities.add_class('hidden-elm', 'print-report');
    if (document.getElementById('marketing-report')) {
        document.getElementById('marketing-report').remove();
    }
    //
    // re-populating preset dropbox
    var preset_report_args = {
        dropbox_id: 'preset-report',
        text_format: '%preset_name%',
        value_format: '%internal_id%',
        database: 'meta_database',
        sql_args: {
            table: 'table_preset_sql_args',
            cols: [ 'preset_name', 'internal_id' ],
            where: [ [ 'html_table', 'REGEXP', report_type + '_report' ] ]
        },
        placeholder: 'None',
        placeholder_value: '0',
        placeholder_disabled: false
    };
    document.getElementById(preset_report_args.dropbox_id).removeAll();
    AFW_HUB.FormUtil.populate_dropbox_options(preset_report_args);
}
//
// creates and uploads the archive data
function create_archive_upload(response) {
    //
    var meta_array = response.meta_data,
        archive_tables = response.archive_cols,
        meta_data = {},
        vendor_data = {},
        i,
        vend_id,
        col;
    //
    // grouping data by vendor_internal_id
    for (i = 0; i < response.data.length; i++) {
        vend_id = response.data[i]['vendor_internal_id'];
        vendor_data[vend_id] = { growth_programs: [], payments: [] };
        Object.merge(vendor_data[vend_id], response.data[i]);
    }
    // adding in growth data array
    for (i = 0; i < response.growth_data.length; i++) {
        vend_id = response.growth_data[i]['vendor_internal_id'];
        vendor_data[vend_id].growth_programs.push(response.growth_data[i]);
    }
    // adding in payments data array
    for (i = 0; i < response.payment_data.length; i++) {
        vend_id = response.payment_data[i]['vendor_internal_id'];
        vendor_data[vend_id].payments.push(response.payment_data[i]);
    }
    //
    // creating table pattern from archive table column names
    var table_pat = [];
    for (i = 0; i < archive_tables.length; i++) {
        archive_tables[i] = archive_tables[i]['column_name'];
        table_pat.push('(?:^|%)' + archive_tables[i] + '(?:%|$)');
    }
    table_pat = '(' + table_pat.join('|') + ')';
    //
    // reformating meta_array into an object and setting in_tables to single value
    for (i = 0; i < meta_array.length; i++) {
        var match = meta_array[i]['in_tables'].match(table_pat);
        if (match) {
            meta_array[i]['in_tables'] = match[0];
            meta_array[i]['in_tables'] = meta_array[i]['in_tables'].replace(
                /%/g,
                ''
            );
        } else {
            continue;
        }
        meta_data[meta_array[i]['column_name']] = meta_array[i];
    }
    console.log(meta_data);
    //
    // creating SQL INSERT and template
    var sql = 'INSERT INTO `marketing_data_archive` ';
    var cols = archive_tables.concat([
        'vendor_internal_id',
        'vendor_status',
        'archived_by'
    ]);
    var sql_cols = [];
    var sql_vals = [];
    for (i = 0; i < cols.length; i++) {
        sql_cols.push('`' + cols[i] + '`');
        sql_vals.push('\'{' + cols[i] + '}\'');
    }
    var value_pat = '(' + sql_vals.join(',') + '),';
    sql += '(' + sql_cols.join(',') + ') VALUES ';
    //
    // processing vendor data to generate archive value statements
    for (vend_id in vendor_data) {
        //
        // initializing archive data objects
        var archive_entry = {};
        for (i = 0; i < archive_tables.length; i++) {
            archive_entry[archive_tables[i]] = {};
        }
        archive_entry.vendor_internal_id = vend_id;
        archive_entry.vendor_status = vendor_data[vend_id].vendor_status;
        archive_entry.archived_by = window.sessionStorage.username;
        archive_entry.marketing_growth = vendor_data[vend_id].growth_programs;
        archive_entry.marketing_payments = vendor_data[vend_id].payments;
        //
        // seperating data by table and then JSON encoding it
        for (col in meta_data) {
            archive_entry[meta_data[col]['in_tables']][col] = vendor_data[vend_id][col];
        }
        for (i = 0; i < archive_tables.length; i++) {
            archive_entry[archive_tables[i]] = JSON.stringify(
                archive_entry[archive_tables[i]]
            );
            archive_entry[archive_tables[i]] = archive_entry[archive_tables[i]].replace(
                /'/g,
                ''
            );
        }
        sql += value_pat.format(archive_entry);
    }
    sql = sql.replace(/,$/, '');
    //
    // setting up sucess callback
    var success_callback = function() {
        alert('Sucessfully Archived Marketing Data');
        var mod_str = 'Last Modified: ' + Date.current_ts();
        document.getElementById('last-archived').textContent = mod_str;
    };
    //
    AFW_HUB.Ajax.exec_transaction([ sql ], success_callback);
}
//
////////////////////////////////////////////////////////////////////////////////
//////////////////////// Marketing Table Generation ////////////////////////////
//
//
//////////////////////////////// Table Filters /////////////////////////////////
//
// container for common vendor/broker table filtering elements
function standard_vb_filter_elemets(update_function) {
    //
    var status_onlick = function() {
            AFW_HUB.FormUtil.toggle_checkbox_value(this, '.', 'active');
        },
        level_elements = Array(
            { elm: 'option', value: '.', textNode: 'All' },
            { elm: 'option', value: 'a la carte', textNode: 'A La Carte' },
            { elm: 'option', value: 'bronze', textNode: 'Bronze' },
            { elm: 'option', value: 'silver', textNode: 'Silver' },
            { elm: 'option', value: 'gold', textNode: 'Gold' },
            { elm: 'option', value: 'gold|silver|bronze', textNode: 'Any' }
        ),
        form_elements = Array(
            { 'elm': 'legend', textNode: 'Selection Parameters' },
            //
            {
                'elm': 'label',
                'style': { 'width': '15em' },
                'textNode': 'Show Deleted Brokers/Vendors:'
            },
            {
                'elm': 'input',
                id: 'filter-vendor-status',
                'name': 'vendor_status',
                'type': 'checkbox',
                value: 'active',
                'events': [
                    { 'event': 'click', 'function': status_onlick },
                    { 'event': 'click', 'function': update_function }
                ]
            },
            { 'elm': 'br' },
            //
            {
                'elm': 'label',
                'style': { 'width': '15em' },
                textNode: 'Filter by Broker:'
            },
            {
                'elm': 'input',
                id: 'filter-broker',
                'name': 'broker',
                'type': 'text',
                value: '',
                'events': [ { 'event': 'keyup', 'function': update_function } ]
            },
            { 'elm': 'br' },
            //
            {
                'elm': 'label',
                'style': { 'width': '15em' },
                'textNode': 'Filter by Vendor:'
            },
            {
                'elm': 'input',
                id: 'filter-vendor',
                'name': 'vendor',
                'type': 'text',
                value: '',
                'events': [ { 'event': 'keyup', 'function': update_function } ]
            },
            { 'elm': 'br' },
            //
            {
                'elm': 'label',
                'style': { 'width': '15em' },
                'textNode': 'Filter by Marketing Level:'
            },
            {
                'elm': 'select',
                id: 'filter-marketing-level',
                'name': 'marketing_level',
                'children': level_elements,
                'events': [ { 'event': 'change', 'function': update_function } ]
            },
            { 'elm': 'br' }
        );
    //
    return form_elements;
}
//
// creates filtering options for the vendor_broker tabke
function create_vendor_broker_table_filter(output_id) {
    //
    // adding a filter fieldset
    var fieldset = document.createElement('FIELDSET');
    var onclick = create_vendor_broker_table.bind(null, 1, 'broker', 'ASC');
    var form_elements = standard_vb_filter_elemets(onclick);
    //
    fieldset.id = 'selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById(output_id).appendChild(fieldset);
    document.getElementById(output_id).appendChild(document.createElement('BR'));
}
//
//
function create_food_show_table_filter(output_id) {
    //
    // adding a filter fieldset
    var fieldset = document.createElement('FIELDSET');
    var update_function = create_food_show_table.bind(null, 1, 'broker', 'ASC');
    var form_elements = standard_vb_filter_elemets(update_function);
    //
    form_elements.push(
        //
        {
            'elm': 'label',
            'style': { 'width': '15em' },
            'textNode': 'Filter by Booth Size:'
        },
        {
            'elm': 'select',
            id: 'filter-booth-size',
            'name': 'booth_size',
            'events': [ { 'event': 'change', 'function': update_function } ]
        },
        { 'elm': 'br' }
    );
    var select_elements = Array(
        //
        { 'elm': 'option', value: '.', textNode: 'All' },
        { 'elm': 'option', value: '0', textNode: 'None' },
        { 'elm': 'option', value: '0.5', textNode: 'Half' },
        { 'elm': 'option', value: '1.0', textNode: 'Full' }
    );
    //
    fieldset.id = 'selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById(output_id).appendChild(fieldset);
    document.getElementById('filter-booth-size').add_children(select_elements);
    document.getElementById(output_id).appendChild(document.createElement('BR'));
}
//
//
function create_growth_table_filter(output_id) {
    //
    // adding a filter fieldset
    var fieldset = document.createElement('FIELDSET');
    var update_function = create_growth_table.bind(null, 1, 'broker', 'ASC');
    var form_elements = standard_vb_filter_elemets(update_function);
    //
    fieldset.id = 'selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById(output_id).appendChild(fieldset);
}
//
//
function create_payment_table_filter(output_id, update_function) {
    //
    // setting default update function
    if (!update_function) {
        update_function = create_payments_vend_table.bind(null, 1, 'broker', 'ASC');
    }
    //
    // adding a filter fieldset
    var fieldset = document.createElement('FIELDSET'),
        form_elements = standard_vb_filter_elemets(update_function),
        button,
        dropbox_args,
        status_onlick = function() {
            AFW_HUB.FormUtil.toggle_checkbox_value(this, '.', 'active');
        };
    //
    form_elements.push(
        { elm: 'label', textNode: 'Show Deleted Records:' },
        {
            elm: 'input',
            id: 'filter-payment-status',
            name: 'payment_status',
            type: 'checkbox',
            value: 'active',
            events: [
                { event: 'click', function: status_onlick },
                { event: 'click', function: update_function }
            ]
        },
        { elm: 'br' },
        {
            elm: 'label',
            style: { width: '15em' },
            textNode: 'Filter by Payment Type:'
        },
        {
            elm: 'select',
            id: 'filter-payment-type',
            name: 'payment_type',
            disabled: true,
            events: [ { event: 'change', function: update_function } ]
        },
        { elm: 'br' }
    );
    //
    fieldset.id = 'selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById(output_id).safeAppendChild(fieldset);
    //
    // populating payment type dropbox
    dropbox_args = {
        dropbox_id: 'filter-payment-type',
        text_format: '%name%',
        value_format: '%type%$',
        placeholder: 'All',
        placeholder_value: '.',
        placeholder_disabled: false,
        sql_args: {
            table: 'marketing_payments',
            cols: [
                'DISTINCT(`payment_type`) as `type`',
                'REPLACE(`payment_type`, \'_\', \' \') AS `name`'
            ]
        }
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    //
    // adding buttons to create or modify a payment
    button = document.createElementWithAttr('BUTTON', { id: 'new-payment' });
    button.textContent = 'New Vendor Payment';
    button.addEventListener('click', function() {
        document.getElementById('table-div').removeAll();
        document.getElementById('content-header').textContent = '';
        document.getElementById('content-div').removeAll();
        update_function = create_payments_vend_table.bind(null, 1, 'broker', 'ASC');
        create_payment_table_filter(output_id, update_function);
        update_function();
    });
    fieldset.safeAppendChild(button);
    //
    button = document.createElementWithAttr('BUTTON', { id: 'mod-payment' });
    button.textContent = 'Modify Vendor Payment';
    button.addEventListener('click', function() {
        document.getElementById('table-div').removeAll();
        document.getElementById('content-header').textContent = '';
        document.getElementById('content-div').removeAll();
        update_function = create_payments_table.bind(null, 1, 'date', 'DESC');
        create_payment_table_filter(output_id, update_function);
        update_function();
    });
    fieldset.safeAppendChild(button);
}
//
//
function create_marketing_commitment_table_filter(output_id) {
    //
    // adding a filter fieldset
    var fieldset = document.createElement('FIELDSET');
    var update_function = create_marketing_commitment_table.bind(
        null,
        1,
        'broker',
        'ASC'
    );
    var form_elements = standard_vb_filter_elemets(update_function);
    //
    fieldset.id = 'selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById(output_id).appendChild(fieldset);
}
//
//
function create_services_performed_table_filter(output_id) {
    //
    // adding a filter fieldset
    var fieldset = document.createElement('FIELDSET');
    var update_function = create_services_performed_table.bind(
        null,
        1,
        'broker',
        'ASC'
    );
    var form_elements = standard_vb_filter_elemets(update_function);
    //
    fieldset.id = 'selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById(output_id).appendChild(fieldset);
}
//
// creates the filtering fieldset for the contact_info table
function create_contact_info_table_filter(broker, vendor_id, output_id) {
    //
    // adding a filter fieldset
    var fieldset = document.createElement('FIELDSET'),
        table_func = create_contact_info_table.bind(
            null,
            vendor_id,
            1,
            'broker',
            'ASC'
        ),
        form_elements = [
            { elm: 'legend', textNode: 'Contact Selection Parameters' },
            //
            {
                elm: 'label',
                style: { width: '15em' },
                textNode: 'Show Deleted Contacts:'
            },
            {
                elm: 'input',
                id: 'contact-show-deleted',
                name: 'contact_status',
                type: 'checkbox',
                value: '^active$',
                events: [
                    { event: 'click',
                      function: function() {
                          var cb = document.getElementById('contact-show-deleted');
                          AFW_HUB.FormUtil.toggle_checkbox_value(cb, '', '^active$');
                      }},
                    { event: 'click', function: table_func }
                ]
            },
            { elm: 'br' }
        ], elm;
    if (broker !== '') {
        elm = {
            elm: 'input',
            type: 'hidden',
            name: 'marketing_vendor_broker_table.broker',
            value: '^' + broker + '$'
        };
        form_elements.push(elm);
    }
    else {
        elm = {
            elm: 'input',
            type: 'hidden',
            name: 'marketing_vendor_broker_table.internal_id',
            value: '^' + vendor_id + '$'
        };
        form_elements.push(elm);
    }
    //
    fieldset.id = 'contact-selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById(output_id).safeAppendChild(fieldset);
}
//
/////////////////////////// Table Creation Functions ///////////////////////////
//
// sets up the standard vendor_broker_table args
function vendor_broker_table_args(tbl_func, page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var table_args,
        data_sql_args = {
            order_by: [
                [ sort_col, sort_dir ],
                [ 'marketing_vendor_broker_table.broker', 'ASC' ],
                [ 'marketing_vendor_broker_table.vendor', 'ASC' ]
            ]
        },
        params_container = document.getElementById('selection-parameters');
    //
    // adding in where paremeters to data_sql_args
    AFW_HUB.TableUtil.get_table_inputs(data_sql_args, params_container);
    //
    // setting up table args
    table_args = {
        table_name: 'vendors_and_brokers',
        data_sql_args: data_sql_args,
        table_output_id: 'table-div',
        table_attr: { id: 'vendor-broker-table' },
        row_id_prefix: 'vbt-row-',
        head_row_args: {
            sortable: true,
            sort_col: sort_col,
            sort_dir: sort_dir,
            sort_onclick: [
                { func: tbl_func, args: [ 1, '%column_name%', '%sort_dir%' ] }
            ]
        },
        page_nav_args: {
            curr_page: page,
            sort_col: sort_col,
            sort_dir: sort_dir,
            tot_pages_shown: 9,
            num_per_page: 10,
            page_nav_div_attr: { id: 'vbt-table-page-nav' },
            id_prefix: 'vbt',
            link_onclick: [
                { func: tbl_func, args: [ '%page%', sort_col, sort_dir ] }
            ]
        }
    };
    //
    return table_args;
}
//
// creates broker vendor table
function create_vendor_broker_table(page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var tbl_func = create_vendor_broker_table,
        table_args = vendor_broker_table_args(tbl_func, page, sort_col, sort_dir);
    //
    table_args.row_onclick = [
        {
            func: vendor_broker_setup_form,
            args: [ '%internal_id%', '%row_id%', null ]
        }
    ];
    table_args.add_callback = function() {
        var br = document.createElementWithAttr('BR', {
            id: 'create-new-vendor-br'
        });
        var button = document.createElementWithAttr('BUTTON', {
            id: 'create-new-vendor'
        });
        button.addTextNode('Create New Vendor/Broker');
        button.style['margin'] = 'auto';
        button.style['display'] = 'block';
        button.addEventListener(
            'click',
            vendor_broker_setup_form.bind(null, null, null, null)
        );
        document.getElementById('table-div').safeAppendChild(br);
        document.getElementById('table-div').safeAppendChild(button);
    };
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
// creates the vendor/broker table for the contact info page
function create_contact_vendor_broker_table(page, sort_col, sort_dir) {
    var tbl_func = create_contact_vendor_broker_table,
        table_args = vendor_broker_table_args(tbl_func, page, sort_col, sort_dir);
    //
    table_args.row_onclick = [
        {
            func: create_contact_info_table_filter,
            args: [ '%broker%', '%internal_id%', 'table-div' ]
        },
        {
            func: create_contact_info_table,
            args: [
                '%internal_id%', // this and above is the vendor's internal id
                1,
                'marketing_contact_information.contact_name',
                'ASC'
            ]
        }
    ];
    table_args.add_callback = function() {
        var br = document.createElementWithAttr('BR', {
            id: 'contact-table-spacer-br'
        });
        document.getElementById('table-div').safeAppendChild(br);
    };
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
// creates the vendor/ broker contact info table
function create_contact_info_table(vendor_id, page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var params_fieldset = document.getElementById('contact-selection-parameters'),
        table_args,
        data_sql_args = {
            where: [],
            order_by: [
                [ sort_col, sort_dir ],
                [ 'marketing_vendor_broker_table.broker', 'ASC' ],
                [ 'marketing_vendor_broker_table.vendor', 'ASC' ]
            ]
        },
        form_func = create_contact_info_form.bind(
            null,
            null,
            vendor_id,
            null,
            null
        );
    //
    // Adding where params to data_sql_args
    AFW_HUB.TableUtil.get_table_inputs(data_sql_args, params_fieldset);
    //
    // creating table argument object
    table_args = {
        table_name: 'marketing_contact_information',
        data_sql_args: data_sql_args,
        table_output_id: 'table-div',
        table_attr: { 'id': 'vendor-broker-contact-info-table' },
        row_id_prefix: 'vbc-info-row-',
        row_onclick: [
            {
                func: create_contact_info_form,
                args: [
                    '%contact_internal_id%',
                    '%vendor_internal_id%',
                    '%row_id%',
                    null
                ]
            }
        ],
        head_row_args: {
            sortable: true,
            sort_col: sort_col,
            sort_dir: sort_dir,
            sort_onclick: [
                {
                    func: create_contact_info_table,
                    args: [
                        '%vendor_internal_id%',
                        1,
                        '%column_name%',
                        '%sort_dir%'
                    ]
                }
            ]
        },
        page_nav_args: {
            curr_page: page,
            sort_col: sort_col,
            sort_dir: sort_dir,
            tot_pages_shown: 9,
            num_per_page: 10,
            page_nav_div_attr: { id: 'vbc-info-table-page-nav' },
            id_prefix: 'vbt',
            link_onclick: [
                {
                    func: create_contact_info_table,
                    args: [
                        vendor_id,
                        '%page%',
                        sort_col,
                        sort_dir
                    ]
                }
            ]
        },
        add_callback: function() {
            var br = document.createElementWithAttr('BR', {
                    id: 'create-new-contacta-br'
                }),
                button = document.createElementWithAttr('BUTTON', {
                    id: 'create-new-contact'
                });
            button.addTextNode('Create New Contact');
            button.style.margin = 'auto';
            button.style.display = 'block';
            button.addEventListener('click', form_func);
            document.getElementById('table-div').safeAppendChild(br);
            document.getElementById('table-div').safeAppendChild(button);
        }
    };
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
//
function create_food_show_table(page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var tbl_func = create_food_show_table,
        table_args = vendor_broker_table_args(tbl_func, page, sort_col, sort_dir);
    //
    table_args.row_onclick = [
        {
            func: create_food_show_form,
            args: [ '%internal_id%', '%row_id%', null ]
        }
    ];
    table_args.data_preprocessor = function(args) {
        var data_arr = args.data, i, size;
        for (i = 0; i < data_arr.length; i++) {
            size = data_arr[i]['booth_size'];
            if (size == '1.0') {
                data_arr[i]['booth_size'] = 'Full';
            } else if (size == '0.5') {
                data_arr[i]['booth_size'] = 'Half';
            } else if (size == '0') {
                data_arr[i]['booth_size'] = 'None';
            }
        }
    };
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
//
function create_growth_table(page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var tbl_func = create_growth_table,
        table_args = vendor_broker_table_args(tbl_func, page, sort_col, sort_dir),
        args;
    //
    table_args.data_sql_args.left_join = [];
    table_args.data_sql_args['sub_query'] = {
        '%growth_data_table%': {
            cmd: 'SELECT',
            table: 'marketing_growth',
            cols: [
                '`vendor_internal_id`',
                'MAX(marketing_growth.growth_last_modified) AS `growth_last_updated`'
            ],
            where: [ [ 'marketing_growth.program_status', 'LIKE', 'active' ] ],
            group_by: '`vendor_internal_id`'
        }
    };
    //
    args = {
        amount_alias: 'growth_paid',
        data_alias: 'growth_payments',
        pay_type_regexp: '(^growth$)'
    };
    gen_marketing_payments_subquery(args, table_args.data_sql_args, []);
    delete table_args.data_sql_args.left_join;
    //
    table_args.row_onclick = [
        {
            func: create_growth_form,
            args: [ '%internal_id%', '%row_id%', null ]
        }
    ];
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
//
function create_payments_vend_table(page, sort_col, sort_dir) {
    //
    // disabling elements that pertain to the payments table
    document.getElementById('filter-payment-status').disabled = true;
    document.getElementById('filter-payment-type').disabled = true;
    //
    // initializating argument objects
    var tbl_func = create_payments_vend_table,
        table_args = vendor_broker_table_args(tbl_func, page, sort_col, sort_dir);
    //
    table_args.data_sql_args['sub_query'] = {
        '%payments_data_table%': {
            cmd: 'SELECT',
            table: 'marketing_payments',
            cols: [
                '`vendor_internal_id`',
                'MAX(marketing_payments.date) AS `last_payment_date`'
            ],
            where: [ [ 'marketing_payments.payment_status', 'LIKE', 'active' ] ],
            group_by: '`vendor_internal_id`'
        }
    };
    table_args.row_onclick = [
        {
            func: create_payment_form,
            args: [ '%internal_id%', null, '%row_id%', null ]
        }
    ];
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
//
function create_payments_table(page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var table_args,
        data_sql_args = {
            where: [],
            order_by: [
                [ sort_col, sort_dir ],
                [ 'marketing_vendor_broker_table.broker', 'ASC' ],
                [ 'marketing_vendor_broker_table.vendor', 'ASC' ]
            ]
        },
        params_container = document.getElementById('selection-parameters');
    //
    // enabling payments table refinement elements
    document.getElementById('filter-payment-status').disabled = false;
    document.getElementById('filter-payment-type').disabled = false;
    //
    // loading where params into data_sql_args
    AFW_HUB.TableUtil.get_table_inputs(data_sql_args, params_container);
    //
    // creating table argument object
    table_args = {
        table_name: 'marketing_payments',
        data_sql_args: data_sql_args,
        table_output_id: 'table-div',
        table_attr: { id: 'payments-table' },
        row_id_prefix: 'vend-pay-row-',
        row_onclick: [
            {
                func: create_payment_form,
                args: [
                    '%internal_id%',
                    '%payment_internal_id%',
                    '%row_id%',
                    null
                ]
            }
        ],
        head_row_args: {
            sortable: true,
            sort_col: sort_col,
            sort_dir: sort_dir,
            sort_onclick: [
                {
                    func: create_payments_table,
                    args: [ 1, '%column_name%', '%sort_dir%' ]
                }
            ]
        },
        page_nav_args: {
            curr_page: page,
            sort_col: sort_col,
            sort_dir: sort_dir,
            tot_pages_shown: 9,
            num_per_page: 10,
            page_nav_div_attr: { id: 'pag-table-page-nav' },
            id_prefix: 'pgt',
            link_onclick: [
                {
                    func: create_payments_table,
                    args: [ '%page%', sort_col, sort_dir ]
                }
            ]
        }
    };
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
//
function create_marketing_commitment_table(page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var tbl_func = create_marketing_commitment_table,
        table_args = vendor_broker_table_args(tbl_func, page, sort_col, sort_dir);
    //
    table_args.row_onclick = [
        {
            func: create_marketing_commitment_form,
            args: [ '%internal_id%', '%row_id%', null ]
        }
    ];
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
//
function create_services_performed_table(page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var tbl_func = create_services_performed_table,
        table_args = vendor_broker_table_args(tbl_func, page, sort_col, sort_dir);
    //
    table_args.row_onclick = [
        {
            func: create_services_performed_form,
            args: [ '%internal_id%', '%row_id%', null ]
        }
    ];
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
/////////////////////////// Reporting Functions ///////////////////////////////
//
// creates the data columns table for the report page
function marketing_report_data_columns(toggle, reset) {
    var radio = document.querySelectorAll('INPUT[name="report-type"]:checked'),
        table_name = radio[0].value.replace(/-/g, '_') + '_report',
        preset = document.getElementById('preset-report').value || '0',
        button = document.getElementById('toggle-column-selection'),
        disp_div = document.getElementById('table-div'),
        hide_str = 'Hide Column Selection',
        show_str = 'Show Column Selection',
        table_args,
        add_event_handlers,
        callback;
    //
    // toggling table viewability
    if (!toggle) {
        button.value = 'update-button';
    }
    toggle_view_element_button(button, disp_div, hide_str, show_str);
    //
    // resetting table based on preset data
    if (!reset) {
        return;
    }
    //
    add_event_handlers = function(elm) {
        var event = elm.primaryEventString(), func;
        //
        if (!event) {
            return;
        }
        func = function() {
            AFW_HUB.Utilities.remove_class('hidden-elm', 'restore-defaults');
            show_update_button('create-report', 'marketing-report', 'Show Changes');
        };
        elm.addEventListener(event, func);
    };
    table_args = {
        table_name: table_name,
        table_attr: { id: 'report-column-selection' },
        table_output_id: disp_div.id,
        create_sort_row: true,
        add_event_handlers: add_event_handlers,
        add_callback: function() {}
    };
    //
    callback = function(preset_data) {
        table_args.selected_cols = preset_data.cols || [ '*' ];
        AFW_HUB.TableUtil.create_column_selection_table(table_args);
    };
    AFW_HUB.TableUtil.fetch_preset_data(preset, callback);
}
//
// this fetches preset data before generating the rest of the args
function init_marketing_report() {
    //
    // initializing variables
    var preset = document.getElementById('preset-report').value || '0',
        radio = document.querySelectorAll('INPUT[name="report-type"]:checked'),
        display_table = document.getElementById('report-column-selection'),
        display_params = AFW_HUB.TableUtil.process_column_selection_table(
            display_table
        ),
        report_args = {
            table_name: radio[0].value.replace(/-/g, '_') + '_report',
            display_cols: display_params.display_cols,
            table_attr: { id: 'marketing-report', class: 'print-container' },
            table_output_id: 'content-div',
            no_page_nav: true,
            page_nav_args: {},
            head_row_args: { sortable: false },
            sort_by: display_params.sort_col[0] || 'broker',
            sort_dir: document.getElementById('sort-dir').value,
            add_callback: build_marketing_report_totals
        };
    //
    // updating CSS classes
    AFW_HUB.Utilities.add_class('hidden-elm', 'create-report');
    AFW_HUB.Utilities.remove_class('boxshadow-blue', 'create-report');
    AFW_HUB.Utilities.remove_class('hidden-elm', 'print-report');
    //
    // getting preset data
    AFW_HUB.TableUtil.fetch_preset_data(
        preset,
        create_marketing_report.bind(null, report_args)
    );
}
//
// generates SQL statements to fetch marketing report data
function create_marketing_report(report_args, sql_args) {
    //
    var params_container = document.getElementById('report-parameters'), args;
    //
    // adding params into sql args
    sql_args.sub_query = {};
    sql_args.left_join = [];
    sql_args.where = sql_args.where || [];
    sql_args.where.push([
        'marketing_vendor_broker_table.vendor_status',
        'LIKE',
        'active'
    ]);
    sql_args.order_by = [
        [ report_args.sort_by, report_args.sort_dir ],
        [ 'marketing_vendor_broker_table.broker', 'ASC' ],
        [ 'marketing_vendor_broker_table.vendor', 'ASC' ]
    ];
    AFW_HUB.TableUtil.get_table_inputs(sql_args, params_container);
    //
    // adding an report type specific changes to the args and sub-queries
    if (report_args.table_name == 'marketing_food_show_report') {
        args = {
            amount_alias: 'food_show_paid',
            data_alias: 'food_show_payments',
            pay_type_regexp: '(^food_show$)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'food_show_allowance_paid',
            data_alias: 'food_show_allowance_payments',
            pay_type_regexp: '(^food_show_allowance$)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
    } else if (report_args.table_name == 'marketing_accounting_report') {
        //
        args = {
            cmd: 'SELECT',
            cols: [
                'SUM(`volume_percent`/100.0 * `ytd_volume` + `purchase_percent`/100.0 * `ytd_purchases` + `dollars_per_unit` * `ytd_units`) AS `growth_total_due`',
                'marketing_growth.vendor_internal_id'
            ],
            table: 'marketing_growth',
            where: [ [ 'program_status', 'LIKE', 'active' ] ],
            group_by: 'marketing_growth.vendor_internal_id'
        };
        sql_args.sub_query['%growth_program_totals%'] = args;
        //
        args = {
            amount_alias: 'marketing_level_paid',
            data_alias: 'marketing_level_payments',
            pay_type_regexp: '(^marketing_level$)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'a_la_carte_paid',
            data_alias: 'a_la_carte_payments',
            pay_type_regexp: '(^a_la_carte$)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'food_show_paid',
            data_alias: 'food_show_payments',
            pay_type_regexp: '(^food_show$)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'food_show_allowance_paid',
            data_alias: 'food_show_allowance_payments',
            pay_type_regexp: '(^food_show_allowance$)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'growth_paid',
            data_alias: 'growth_payments',
            pay_type_regexp: '(^growth$)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'additional_fees_paid',
            data_alias: 'additional_fee_payments',
            pay_type_regexp: '(^additional_fees)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'miscellaneous_paid',
            data_alias: 'miscellaneous_payments',
            pay_type_regexp: '(^miscellaneous)'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
        //
        args = {
            amount_alias: 'total_paid',
            data_alias: 'marketing_total_payments',
            pay_type_regexp: '.*'
        };
        gen_marketing_payments_subquery(args, sql_args, []);
    } else if (report_args.table_name == 'marketing_services_report') {
        //pass
    }
    //
    delete sql_args.left_join;
    report_args.data_sql_args = sql_args;
    //
    AFW_HUB.TableUtil.create_standard_table(report_args);
}
//
// builds a totals row at the bottom of the report
function build_marketing_report_totals(response, table) {
    var data = response.table_data,
        meta_data = response.meta_data.to_object('column_name'),
        row = document.createElement('TR'),
        td = document.createElement('TD'),
        totals = {},
        i,
        column,
        args;
    //
    // initializing the totals row
    for (column in meta_data) {
        totals[column] = 0;
    }
    //
    // summing up all valuess
    for (i = 0; i < data.length; i++) {
        for (column in data[i]) {
            totals[column] += Number(data[i][column]);
        }
    }
    //
    // outputting totals row
    for (column in meta_data) {
        if (isNaN(totals[column])) {
            totals[column] = '';
        }
        td = td.cloneNode();
        args = [
            totals[column],
            meta_data[column].data_type,
            td,
            { format_str: '%value%', add_commas: true }
        ];
        AFW_HUB.TableUtil.display_table_data.apply(null, args);
        row.appendChild(td);
    }
    row.style.fontWeight = 'bold';
    row.firstChild.textContent = 'Totals:';
    table.appendChild(row);
}
