/* global AFW_HUB */
////////////////////////////////////////////////////////////////////////////////
//////////////       This file holds the functions that are          ///////////
//////////////       associated only with marketing forms            ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
//
function growth_fieldset_elements() {
    var elms = [
        //
        { elm: 'label', textNode: 'Volume %' },
        {
            elm: 'input',
            id: 'volume-percent',
            name: 'volume_percent',
            class: 'text-input',
            value: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'Purchases %' },
        {
            elm: 'input',
            id: 'purchase-percent',
            name: 'purchase_percent',
            value: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'Dollars per Unit' },
        {
            elm: 'input',
            id: 'dollars-per-unit',
            name: 'dollars_per_unit',
            value: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        { elm: 'br' },
        { elm: 'label', textNode: 'YTD Volume' },
        {
            elm: 'input',
            id: 'ytd-volume',
            name: 'ytd_volume',
            value: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'YTD Purchases' },
        {
            elm: 'input',
            id: 'ytd-purchases',
            name: 'ytd_purchases',
            value: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        { elm: 'label', textNode: 'YTD Units' },
        {
            elm: 'input',
            id: 'ytd-units',
            name: 'ytd_units',
            value: '0.00',
            'data-data_type': 'float'
        },
        { elm: 'br' },
        { elm: 'br' },
        { elm: 'label', textNode: 'Growth Notes' },
        {
            elm: 'textarea',
            id: 'growth-notes',
            name: 'growth_notes',
            rows: '2',
            cols: '60',
            value: '',
            'data-skip_auto_validate': '1',
            'data-allow_empty': '1'
        },
        {
            elm: 'input',
            type: 'hidden',
            id: 'growth-internal-id',
            name: 'growth_internal_id',
            value: ''
        },
        {
            elm: 'input',
            type: 'hidden',
            id: 'program-status',
            name: 'program_status',
            value: 'active'
        },
        { elm: 'br' },
        {
            elm: 'button',
            type: 'button',
            class: 'js-create-growth-program',
            textNode: 'Create Program'
        },
        {
            elm: 'button',
            type: 'button',
            class: 'js-remove-growth-program',
            textNode: 'Remove Program'
        }
    ];
    //
    return Object.clone(elms);
}
//
// creates the last updated span for different fieldsets
function create_last_modifed_span(name) {
    //
    var span = document.createElement('SPAN');
    var label = document.createElement('LABEL');
    var input = document.createElementWithAttr('INPUT', {
        'name': name,
        'class': 'input-borderless'
    });
    //
    span.style['float'] = 'right';
    label.style['min-width'] = 'initial';
    label.style['color'] = 'rgb(127,127,127)';
    label.style['font-size'] = '14px';
    label.textContent = 'Last Updated:  ';
    input.disabled = true;
    //
    span.addNodes([ label, input ]);
    return span;
}
//
///////////////////////////// Vendor Broker Setup Form /////////////////////////
//
//
function submit_vendor_broker_form(form, action) {
    var errors = AFW_HUB.FormUtil.auto_validate(form),
        form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        callback;
    //
    if (errors) {
        return;
    }
    if (!confirm('Confirm ' + action + ' of Broker/ Vendor.')) {
        return;
    }
    //
    if (action === 'creation') {
        form_values.vendor_status = 'active';
        form_values.vendor_internal_id_set_statement = '1';
        delete form_values.internal_id;
        form_values['%insert_all%'] = '1';
        form_values = [form_values, {vendor_internal_id: '@vendor_id'}];
    }
    if (action === 'deletion') {
        form_values.vendor_status = 'deleted';
    }
    if (action === 'restoration') {
        form_values.vendor_status = 'active';
    }
    //
    callback = reset_vendor_broker_form.bind(null, action);
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
//
// resets the vendor broker from
function reset_vendor_broker_form(action) {
    //
    document.getElementById('vendor-broker-table').dispatchPrimaryEvent();
    alert('Successful ' + action + ' of Vendor/ Broker.');
    //
    // recreating form
    if (action === 'creation') {
        vendor_broker_setup_form(null, null, null);
    } else {
        document.getElementById('content-div').removeAll();
        document.getElementById('content-header').textContent = '';
    }
}
//
/////////////////////////// Vendor Broker Contact Form /////////////////////////
//
//
function submit_marketing_contact_info_form(form, act) {
    var errors = AFW_HUB.FormUtil.auto_validate(form),
        form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        callback;
    //
    if (errors) {
        return;
    }
    if (!confirm('Confirm ' + act + ' of ' + form_values.contact_name)) {
        return;
    }
    //
    if (act === 'creation') {
        form_values.contact_status = 'active';
    }
    if (act === 'deletion') {
        form_values.contact_status = 'deleted';
    }
    if (act === 'restoration') {
        form_values.contact_status = 'active';
    }
    //
    callback = reset_marketing_contact_info_form.bind(null, act);
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
//
// resets the contact info from
function reset_marketing_contact_info_form(act) {
    var table = document.getElementById('vendor-broker-contact-info-table');
    table.dispatchPrimaryEvent();
    //
    alert('Successful ' + act + ' of contact.');
    document.getElementById('content-div').removeAll();
    document.getElementById('content-header').textContent = '';
}
//
//////////////////////// Marketing Food Show Validation ////////////////////////
//
function validate_marketing_food_show_form(form) {
    //
    var form_values = AFW_HUB.FormUtil.get_all_form_values(form), total_price;
    //
    total_price = form_values['booth_price'];
    total_price += form_values['electric_fee'];
    total_price += form_values['allowance'];
    document.getElementById('total-booth-price').value = total_price;
    //
    AFW_HUB.FormUtil.round_form_inputs(form, 2);
}
//
function submit_marketing_food_show_form(form) {
    var errors = AFW_HUB.FormUtil.auto_validate(form),
        form_values = AFW_HUB.FormUtil.get_all_form_values(form);
    validate_marketing_food_show_form(form);
    if (errors) {
        return;
    }
    //
    if (!confirm('Confirm update of food show information.')) {
        return;
    }
    //
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, reset_food_show_form);
}
//
function reset_food_show_form() {
    //
    document.getElementById('vendor-broker-table').dispatchPrimaryEvent();
    alert('Successful update of food show information.');
    //
    document.getElementById('content-div').removeAll();
    document.getElementById('content-header').textContent = '';
}
//
/////////////////////////////// Growth Validation /////////////////////////////
//
//
// generates the growth fieldsets
function create_growth_fieldset(form, response) {
    var n = Number(document.getElementById('growth-field-count').value),
        fieldset = document.createElement('FIELDSET'),
        legend = document.createElement('LEGEND'),
        last_modified = create_last_modifed_span('growth_last_modified'),
        button = document.createElementWithAttr('BUTTON', {type: 'button'}),
        create_func = setup_growth_program.bind(null, form, fieldset, 'creation'),
        elm_array, elm, i;
    n += 1;
    //
    // copying fieldset elements and adding event handlers
    elm_array = growth_fieldset_elements();
    for (i = 0; i < elm_array.length; i++) {
        elm = elm_array[i];
        if (elm.hasOwnProperty('id')) {
            elm['id'] = 'growth-program-{}-{}'.format(n, elm['id'] );
        }
    }
    //
    // setting up fieldset child nodes
    fieldset.id = 'growth-program-' + n;
    fieldset.addNodes([ legend, last_modified ]);
    fieldset.add_children(elm_array);
    legend.textContent = 'Growth Program ' + n;
    document.getElementById('growth-fieldsets-div').appendChild(fieldset);
    //
    // adding event listeners
    add_growth_form_events(form, fieldset, response);
    //
    // setting up buttons to create or remove the program
    button = fieldset.querySelector('.js-remove-growth-program');
    button.addEventListener('click', fieldset.remove.bind(fieldset));
    //
    button = fieldset.querySelector('.js-create-growth-program');
    button.addEventListener('click', create_func);
    //
    // incrementing counter and returning fieldset
    document.getElementById('growth-field-count').value = n;
    return fieldset;
}
//
// shows and hides a growth program fieldset
function show_hide_growth_program(button, fieldset) {
    var status = fieldset.querySelector('[name=program_status]').value,
        sub_nodes = fieldset.querySelectorAll('fieldset > *'),
        class_func = AFW_HUB.Utilities.remove_class,
        i;
    //
    if (!fieldset.dataset.hidden) {
        class_func = AFW_HUB.Utilities.add_class;
        button.textContent = 'Show ' + status.toTitleCase() + ' Program';
        fieldset.dataset.hidden = '1';
    }
    else {
        button.textContent = 'Hide ' + status.toTitleCase() + ' Program';
        fieldset.dataset.hidden = '';
    }
    //
    // looping through elements to show or hide them
    for (i = 2; i < sub_nodes.length - 1; i++) {
        class_func('hidden-elm', sub_nodes[i]);
    }
}
//
// handles addition of buttons to a growth program fieldset
function mod_growth_program_buttons(form, fieldset) {
    var status = fieldset.querySelector('[name=program_status]').value,
        crt_button = fieldset.querySelector('.js-create-growth-program'),
        rem_button = fieldset.querySelector('.js-remove-growth-program'),
        sub_func = function(act) {
            setup_growth_program(form, fieldset, act);
        },
        button;
    crt_button.className = '';
    //
    // modfying buttons to handle an existing fieldset
    button = crt_button.cloneNode();
    AFW_HUB.Utilities.add_class('js-update-growth-progam', button);
    button.textContent = 'Update Program';
    button.addEventListener('click', sub_func.bind(null, 'modification'));
    fieldset.replaceChild(button, crt_button);
    //
    button = crt_button.cloneNode();
    AFW_HUB.Utilities.add_class('js-delete-growth-progam', button);
    button.textContent = 'Delete Program';
    button.addEventListener('click', sub_func.bind(null, 'deletion'));
    fieldset.replaceChild(button, rem_button);
    //
    button = crt_button.cloneNode();
    AFW_HUB.Utilities.add_class('js-restore-growth-progam', button);
    button.textContent = 'Restore Program';
    button.addEventListener('click', sub_func.bind(null, 'restoration'));
    fieldset.safeAppendChild(button);
    //
    button = crt_button.cloneNode();
    button.textContent = 'Hide Program';
    button.addEventListener(
        'click',
        show_hide_growth_program.bind(null, button, fieldset)
    );
    fieldset.safeAppendChild(button);
    //
    if (status !== 'active') {
        button = fieldset.querySelector('.js-delete-growth-progam');
        AFW_HUB.Utilities.add_class('hidden-elm', button);
        show_hide_growth_program(fieldset.lastChild, fieldset);
    } else {
        button = fieldset.querySelector('.js-restore-growth-progam');
        AFW_HUB.Utilities.add_class('hidden-elm', button);
    }
}
//
// adds event handlers to the specified container for the growth form
function add_growth_form_events(form, container, response) {
    var event_args = {
        meta_array: response.meta_data,
        add_default_events: function(elm) {
            var add_class = AFW_HUB.Utilities.add_class,
                rem_class = AFW_HUB.Utilities.remove_class,
                validate = validate_growth_form.bind(null, form, true),
                event = elm.primaryEventString();
            //
            if (elm.nodeName.toLowerCase() === 'button') {
                return;
            }
            if (!event) {
                return;
            }
            //
            elm.addEventListener(event, rem_class.bind(null, 'boxshadow-red', elm));
            elm.addEventListener('keyup', function(container) {
                var button = container.querySelector('.js-update-growth-progam');
                if (button === null) { return;}
                if (button.parentNode !== elm.parentNode) { return;}
                add_class('boxshadow-blue', button);
            }.bind(null, container));
            elm.addEventListener('blur', validate);
        }
    };
    AFW_HUB.FormUtil.add_standard_event_listeners(container, event_args);
}
//
// updates proram totals and checks for errors
function validate_growth_form(form, truncate) {
    var error = AFW_HUB.FormUtil.auto_validate(form),
        precision = truncate ? AFW_HUB.get_constant('STD_PRECISION') : 8,
        conatiner = document.getElementById('growth-fieldsets-div'),
        fieldsets = conatiner.querySelectorAll('FIELDSET'),
        growth_paid = Number(document.getElementById('growth-paid').value),
        growth_due = 0,
        growth_values, growth_rem, i;
    //
    // validating and summing values from the growth fieldsets
    for (i = 0; i < fieldsets.length; i++) {
        growth_values = AFW_HUB.FormUtil.get_all_form_values(fieldsets[i]);
        //
        // converting into percents
        growth_values.volume_percent /= 100;
        growth_values.purchase_percent /= 100;
        //
        growth_due += growth_values.volume_percent * growth_values.ytd_volume;
        growth_due += growth_values.purchase_percent * growth_values.ytd_purchases;
        growth_due += growth_values.dollars_per_unit * growth_values.ytd_units;
    }
    //
    // outputting values
    growth_rem = growth_due - growth_paid;
    document.getElementById('growth-due').value = growth_due;
    document.getElementById('growth-remaining').value = growth_rem;
    //
    // rounding form and returning error
    AFW_HUB.FormUtil.round_form_inputs(form, precision);
    return error;
}
//
// submits a single program to the database
function setup_growth_program(form, fieldset, action) {
    var errors = AFW_HUB.FormUtil.auto_validate(fieldset),
        form_values = AFW_HUB.FormUtil.get_all_form_values(fieldset),
        callback;
    form_values.vendor_internal_id = document.getElementById('vendor-internal-id').value;
    //
    if (errors) {
        return;
    }
    if (!confirm('Confirm ' + action + ' of growth program.')) {
        return;
    }
    //
    form_values.growth_last_modified = Date.current_ts();
    form_values.growth_last_modified_by = window.sessionStorage.username;
    if (action === 'creation') {
        form_values.program_status = 'active';
        delete form_values.growth_last_modified;
        delete form_values.growth_last_modified_by;
    } else if (action === 'deletion') {
        form_values.program_status = 'deleted';
    } else if (action === 'restoration') {
        form_values.program_status = 'active';
    }
    //
    callback = function() {
        var button;
        // updating some elements
        fieldset.querySelector('[name=program_status]').value = form_values.program_status;
        fieldset.querySelector('[name=growth_last_modified]').value = Date.current_ts();
        //
        if (action === 'creation') {
            mod_growth_program_buttons(form, fieldset);
        }
        //
        button = fieldset.querySelector('.js-update-growth-progam');
        AFW_HUB.Utilities.remove_class('boxshadow-blue', button);
        if (form_values.program_status !== 'active') {
            button = fieldset.querySelector('.js-delete-growth-progam');
            AFW_HUB.Utilities.add_class('hidden-elm', button);
            show_hide_growth_program(fieldset.lastChild, fieldset);
        } else {
            button = fieldset.querySelector('.js-restore-growth-progam');
            AFW_HUB.Utilities.add_class('hidden-elm', button);
        }
        //
        alert('Successful ' + action + ' of growth program.');
    };
    //
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
//
///////////////////////////// Payments Validation /////////////////////////////
//
// populates the account balance fields on the payment form
function populate_account_balance_fields(form, vend_id, response) {
    var args,
        sql_args,
        sql_arr = [],
        key_arr = [],
        key,
        callback,
        data = {};
    //
    // setting up each sq, statement for the vendor account totals
    sql_args = {
        cmd: 'SELECT',
        cols: [
            'COALESCE(level_commitment, 0) AS `total_due_marketing_level`',
            '((SELECT total_due_marketing_level) - COALESCE(total_paid_marketing_level, 0)) AS `remaining_marketing_level`'
        ],
        table: 'marketing_vendor_broker_table',
        inner_join: [
            ['marketing_accounting',
             'marketing_vendor_broker_table.internal_id',
             'marketing_accounting.vendor_internal_id']
        ],
        left_join: [],
        sub_query: {},
        where: [ [ 'marketing_accounting.vendor_internal_id', 'LIKE', vend_id ] ]
    };
    args = {
        amount_alias: 'total_paid_marketing_level',
        data_alias: 'data',
        pay_type_regexp: '(^marketing_level$)'
    };
    gen_marketing_payments_subquery(args, sql_args, sql_args.cols);
    sql_arr.push(sql_args);
    key_arr.push('marketing_level');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_vendor_broker_table',
        inner_join: [
            ['marketing_food_show',
             'marketing_vendor_broker_table.internal_id',
             'marketing_food_show.vendor_internal_id']
        ],
        cols: [
            'COALESCE(booth_price + electric_fee, 0) AS `total_due_food_show`',
            '(booth_price + electric_fee - COALESCE(total_paid_food_show, 0)) AS `remaining_food_show`'
        ],
        left_join: [],
        sub_query: {},
        where: [ [ 'marketing_food_show.vendor_internal_id', 'LIKE', vend_id ] ]
    };
    args = {
        amount_alias: 'total_paid_food_show',
        data_alias: 'data',
        pay_type_regexp: '(^food_show$)'
    };
    gen_marketing_payments_subquery(args, sql_args, sql_args.cols);
    sql_arr.push(sql_args);
    key_arr.push('foodshow');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_vendor_broker_table',
        inner_join: [
            ['marketing_food_show',
             'marketing_vendor_broker_table.internal_id',
             'marketing_food_show.vendor_internal_id']
        ],
        cols: [
            '`allowance` AS `total_due_food_show_allowance`',
            '(allowance - COALESCE(total_paid_food_show_allowance, 0)) AS `remaining_food_show_allowance`'
        ],
        left_join: [],
        sub_query: {},
        where: [ [ 'marketing_food_show.vendor_internal_id', 'LIKE', vend_id ] ]
    };
    args = {
        amount_alias: 'total_paid_food_show_allowance',
        data_alias: 'data',
        pay_type_regexp: '(^food_show_allowance$)'
    };
    gen_marketing_payments_subquery(args, sql_args, sql_args.cols);
    sql_arr.push(sql_args);
    key_arr.push('foodshow_allowance');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_vendor_broker_table',
        inner_join: [
            ['marketing_alacarte_commitment',
             'marketing_vendor_broker_table.internal_id',
             'marketing_alacarte_commitment.vendor_internal_id']
        ],
        cols: [
            '`amount_committed` AS `total_due_a_la_carte`',
            '(`amount_committed` - COALESCE(total_paid_a_la_carte, 0)) AS `remaining_a_la_carte`'
        ],
        left_join: [],
        sub_query: {},
        where: [
            [ 'marketing_alacarte_commitment.vendor_internal_id', 'LIKE', vend_id ]
        ]
    };
    args = {
        amount_alias: 'total_paid_a_la_carte',
        data_alias: 'data',
        pay_type_regexp: '(^a_la_carte$)'
    };
    gen_marketing_payments_subquery(args, sql_args, sql_args.cols);
    sql_arr.push(sql_args);
    key_arr.push('alacarte');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_growth',
        cols: [
            'SUM(`volume_percent`/100.0 * `ytd_volume`) AS `volume_total`',
            'SUM(`purchase_percent`/100.0 * `ytd_purchases`) AS `purchases_total`',
            'SUM(`dollars_per_unit` * `ytd_units`) AS `unit_total`',
            'marketing_growth.vendor_internal_id'
        ],
        where: [ [ 'marketing_growth.vendor_internal_id', 'LIKE', vend_id ] ],
        group_by: 'marketing_growth.vendor_internal_id'
    };
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_vendor_broker_table',
        inner_join: [
            ['%growth_totals_sub% AS `growth_data`',
             'marketing_vendor_broker_table.internal_id',
             'growth_data.vendor_internal_id']
        ],
        cols: [
            '(volume_total + purchases_total + unit_total) AS `total_due_growth`',
            '(volume_total + purchases_total + unit_total - COALESCE(total_paid_growth,0)) AS `remaining_growth`'
        ],
        left_join: [],
        sub_query: { '%growth_totals_sub%': sql_args }
    };
    args = {
        amount_alias: 'total_paid_growth',
        data_alias: 'data',
        pay_type_regexp: '(^growth$)',
    };
    gen_marketing_payments_subquery(args, sql_args, sql_args.cols);
    sql_arr.push(sql_args);
    key_arr.push('growth');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_vendor_broker_table',
        inner_join: [
            ['marketing_accounting',
             'marketing_vendor_broker_table.internal_id',
             'marketing_accounting.vendor_internal_id']
        ],
        cols: [
            'COALESCE(misc_charges, 0) AS `total_due_miscellaneous`',
            '((SELECT total_due_miscellaneous) - COALESCE(total_paid_miscellaneous, 0)) AS `remaining_miscellaneous`'
        ],
        left_join: [],
        sub_query: {},
        where: [ [ 'marketing_accounting.vendor_internal_id', 'LIKE', vend_id ] ]
    };
    args = {
        amount_alias: 'total_paid_miscellaneous',
        data_alias: 'data',
        pay_type_regexp: '(^miscellaneous$)'
    };
    gen_marketing_payments_subquery(args, sql_args, sql_args.cols);
    sql_arr.push(sql_args);
    key_arr.push('miscellaneous');
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'marketing_vendor_broker_table',
        inner_join: [
            ['marketing_accounting',
             'marketing_vendor_broker_table.internal_id',
             'marketing_accounting.vendor_internal_id']
        ],
        cols: [
            'COALESCE(additional_fees, 0) AS `total_due_additional_fees`',
            '((SELECT total_due_additional_fees) - COALESCE(total_paid_additional_fees, 0)) AS `remaining_additional_fees`'
        ],
        left_join: [],
        sub_query: {},
        where: [ [ 'marketing_accounting.vendor_internal_id', 'LIKE', vend_id ] ]
    };
    args = {
        amount_alias: 'total_paid_additional_fees',
        data_alias: 'data',
        pay_type_regexp: '(^additional_fees)'
    };
    gen_marketing_payments_subquery(args, sql_args, sql_args.cols);
    sql_arr.push(sql_args);
    key_arr.push('additional_fees');
    //
    if (response === undefined) {
        callback = populate_account_balance_fields.bind(null, form, vend_id);
        AFW_HUB.Ajax.fetch_db(sql_arr, key_arr, null, callback);
        return;
    }
    //
    // processing returned data into a single object
    for (key in response) {
        Object.merge(data, response[key][0]);
    }
    // outputting it to the totals fields
    for (key in data) {
        document.getElementById(key.replace(/_/g, '-')).value = data[key];
    }
    AFW_HUB.FormUtil.round_form_inputs(form, 2);
}
//
//
function submit_payment_form(form, action) {
    var errors = AFW_HUB.FormUtil.auto_validate(form),
        form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        callback;
    //
    if (errors) {
        return;
    }
    if (!confirm('Confirm ' + action + ' of Payment.')) {
        return;
    }
    //
    form_values.payment_last_modified = Date.current_ts();
    form_values.payment_last_modified_by = window.sessionStorage.username;
    if (action === 'creation') {
        form_values.entering_user = window.sessionStorage.username;
        form_values.payment_status = 'active';
        delete form_values.payment_last_modified;
        delete form_values.payment_last_modified_by;
    }
    if (action === 'deletion') {
        form_values.payment_status = 'deleted';
    }
    if (action === 'restoration') {
        form_values.payment_status = 'active';
    }
    //
    callback = reset_payments_form.bind(null, action);
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
//
//
function reset_payments_form(act) {
    var table = document.getElementById('table-div').querySelector('TABLE');
    table.dispatchPrimaryEvent();
    //
    alert('Successful ' + act + ' of Payment.');
    //
    document.getElementById('content-div').removeAll();
    document.getElementById('content-header').textContent = '';
}
//
///////////////////// Marketing commitment validation /////////////////////////
//
// creates the alacarte fields based on database data
function create_alacarte_fields(form, response) {
    var output_div = document.getElementById('alacarte-fields'),
        price_data = response.price_data.to_object('service'),
        calc_total,
        label,
        input,
        select,
        textarea,
        level,
        price,
        total,
        br,
        entry,
        i,
        col,
        attr,
        node_list = [];
    //
    // defining function to auto update itemized totals
    calc_total = function(quantity, price, total) {
        var val = Number.parse(quantity.value) * Number.parse(price.value);
        if (quantity.disabled) {
            val = 0;
        }
        total.value = '$ ' + val.toFixed(AFW_HUB.get_constant('STD_PRECISION'));
    };
    //
    // defining template nodes
    label = document.createElementWithAttr('LABEL', { style: { width: '12rem' } });
    input = document.createElementWithAttr('INPUT', { style: { width: '7rem' } });
    br = document.createElement('BR');
    //
    attr = { style: { width: '7rem' }, value: 0, disabled: true };
    attr.style.textAlign = 'right';
    price = document.createElementWithAttr('INPUT', attr);
    attr.value = '$ 0.00';
    total = document.createElementWithAttr('INPUT', attr);
    //
    attr = { style: { minWidth: 'initial', width: '8.3rem' } };
    select = document.createElementWithAttr('SELECT', attr);
    select.add_children([
        { elm: 'OPTION', value: 0, textNode: 'NO' },
        { elm: 'OPTION', value: 1, textNode: 'YES' }
    ]);
    //
    for (i = 0; i < response.meta_data.length; i++) {
        col = response.meta_data[i];
        if (price_data[col.column_name] === undefined) { continue;}
        col.cost = price_data[col.column_name].cost;
        //
        label = label.cloneNode();
        label.textContent = col.display_name;
        node_list.push(label);
        //
        // outputting select or text input depending on data type
        if (col.data_type.match(/bool/)) {
            level = select.cloneNode(true);
            //
            entry = select.cloneNode(true);
            entry.addEventListener(
                'change',
                calc_total.bind(null, entry, price, total)
            );
        } else {
            level = input.cloneNode();
            //
            entry = input.cloneNode(true);
            entry.addEventListener(
                'keyup',
                calc_total.bind(null, entry, price, total)
            );
        }
        //
        // defining element properties and price values
        level.disabled = true;
        level.dataset.column = 'level_commitment';
        level.dataset.column_name = col.column_name;
        //
        price = price.cloneNode();
        price.value = '$ ' + Math.round10(col.cost, 2, true);
        total = total.cloneNode();
        total.dataset.column = 'total_value';
        total.dataset.column_name = col.column_name;
        //
        entry.id = col.column_name.replace(/_/g, '-');
        entry.name = col.column_name;
        entry.dataset.data_type = col.data_type;
        node_list.push(entry, level, price, total, br.cloneNode());
    }
    //
    // adding notes field
    label = label.cloneNode();
    label.textContent = 'Notes';
    attr = {
        id: 'alacarte-commitment-notes',
        name: 'alacarte_commitment_notes',
        'data-allow_empty': '1',
        rows: 2, cols: 60
    };
    textarea = document.createElementWithAttr('TEXTAREA', attr);
    node_list.push(label, textarea, br.cloneNode());
    output_div.addNodes(node_list);
}
//
// updates the marketing level commitment field
function update_commitment_field(form, response) {
    var level = document.getElementById('marketing-level').value,
        suggested = document.getElementById('suggested-level-commitment'),
        elements = form.querySelectorAll('[data-column=level_commitment]'),
        sql_args,
        data,
        i,
        args;
    //
    if (response === undefined) {
        sql_args = {
            cmd: 'SELECT',
            table: 'marketing_tiers',
            where: [ [ 'marketing_level', 'LIKE', level ] ]
        };
        args = [[sql_args], [0], 'misc_database'];
        args.push(update_commitment_field.bind(null, form));
        AFW_HUB.Ajax.fetch_db.apply(null, args);
        return;
    }
    //
    data = response[0][0];
    args = [data.commitment, 'float', suggested];
    args.push({ format_str: '$ %value%', add_commas: true });
    suggested.value = AFW_HUB.TableUtil.display_table_data.apply(null, args);
    //
    for (i = 0; i < elements.length; i++) {
        elements[i].value = data[elements[i].dataset.column_name];
    }
    validate_marketing_commitment_form(form, true);
}
//
// recalculates and checks the alacarte form for errors
function validate_marketing_commitment_form(form, truncate) {
    var error = AFW_HUB.FormUtil.auto_validate(form),
        form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        total_fields = form.querySelectorAll('[data-column=total_value]'),
        precision = truncate ? AFW_HUB.get_constant('STD_PRECISION') : 8,
        total = 0,
        i;
    //
    // getting form values
    // calculating total a la carte charges
    for (i = 0; i < total_fields.length; i++) {
        total += Number.parse(total_fields[i].value);
    }
    /*new_blarg pushes total of component value up to a-la-carte commitment in the
    event no value is entered. Credit must be entered to offset charges if components
    are included in the marketing program.
    */
    //start new_blarg -
    if (total > form_values['amount_committed']) {
        document.getElementById('amount-committed').value = total;
        form_values['amount_committed'] = total;
    }
    //end new blarg
    var bal_rem = form_values['amount_committed'] - total;
    // total commitment must be the initial commitment plus all charges above
    // and beyond that commitment, minus any credits
    var tot_com;
    //begin conditional statements
    if (bal_rem < 0) {
        tot_com = form_values['level_commitment'] - bal_rem;
        tot_com += form_values['amount_committed'];
    } else {
        tot_com = form_values['level_commitment'] + form_values['amount_committed'];
    }
    //end conditional statements
    //adding additional fees and subtracting credits
    document.getElementById('balance-remaining').value = bal_rem;
    document.getElementById('total-marketing-commitment').value = tot_com;
    //
    // rounding values to precision
    AFW_HUB.FormUtil.round_form_inputs(form, precision);
    //
    return error;
}
//
// submits form values to the database
function submit_marketing_commitment_form(form) {
    var errors = validate_marketing_commitment_form(form, false),
        form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        username = window.sessionStorage.username,
        args;
    //
    // returning early if there are errors
    if (errors) {
        validate_marketing_commitment_form(form, true);
        return;
    }
    if (!confirm('Confirm update of marketing information.')) {
        return;
    }
    //
    // adding modify dates to form values
    form_values.internal_id = form_values.vendor_internal_id;
    form_values.accounting_last_modified = Date.current_ts();
    form_values.accounting_last_modified_by = username;
    form_values.alacarte_commitment_last_modified = Date.current_ts();
    form_values.alacarte_commitment_last_modified_by = username;
    //
    args = [form.name, form_values, reset_marketing_commitment_form];
    AFW_HUB.Ajax.submit_form_values.apply(null, args);
}
//
// resets the page after a sucessful submision
function reset_marketing_commitment_form() {
    //
    alert('Successful update of Vendor/ Broker marketing commitment information.');
    //
    document.getElementById('vendor-broker-table').dispatchPrimaryEvent();
    document.getElementById('content-div').removeAll();
    document.getElementById('content-header').textContent = '';
}
//
//////////////// Marketing services performed validation //////////////////////
//
//
function create_service_performed_fields(form, response) {
    var output_div = document.getElementById('services-performed-fields'),
        calc_remaining,
        label,
        input,
        select,
        level,
        alacarte,
        remaining,
        br,
        entry,
        i,
        col,
        attr,
        node_list = [];
    //
    // defining function to auto update itemized totals
    calc_remaining = function(input, level, alacarte, remaining) {
        var total = Number(alacarte.value) + Number(level.value);
        remaining.value = total - Number(input.value);
    };
    //
    // defining template nodes
    label = document.createElementWithAttr('LABEL', { style: { width: '12rem' } });
    input = document.createElementWithAttr('INPUT', { style: { width: '7rem' } });
    br = document.createElement('BR');
    //
    attr = { style: { minWidth: 'initial', width: '8.3rem' } };
    select = document.createElementWithAttr('SELECT', attr);
    select.add_children([
        { elm: 'OPTION', value: 0, textNode: 'NO' },
        { elm: 'OPTION', value: 1, textNode: 'YES' }
    ]);
    //
    for (i = 0; i < response.meta_data.length; i++) {
        col = response.meta_data[i];
        if (response.marketing_level_data[col.column_name] === undefined) {
            continue;
        }
        if (response.alacarte_data[col.column_name] === undefined) {
            continue;
        }
        //
        label = label.cloneNode();
        label.textContent = col.display_name;
        node_list.push(label);
        //
        // outputting select or text input
        if (col.data_type.match('bool')) {
            level = select.cloneNode(true);
            alacarte = select.cloneNode(true);
            remaining = input.cloneNode();
            entry = select.cloneNode(true);
            entry.addEventListener(
                'change',
                calc_remaining.bind(null, entry, level, alacarte, remaining)
            );
        } else {
            level = input.cloneNode();
            alacarte = input.cloneNode();
            remaining = input.cloneNode();
            entry = input.cloneNode();
            entry.addEventListener(
                'keyup',
                calc_remaining.bind(null, entry, level, alacarte, remaining)
            );
        }
        //
        // generating standard calculation and display elements
        level.disabled = true;
        level.dataset.column = 'level_commitment';
        level.dataset.column_name = col.column_name;
        level.value = response.marketing_level_data[col.column_name];
        alacarte.disabled = true;
        alacarte.value = response.alacarte_data[col.column_name];
        remaining.disabled = true;
        //
        entry.id = col.column_name.replace(/_/g, '-');
        entry.name = col.column_name;
        entry.dataset.data_type = col.data_type;
        node_list.push(entry, level, alacarte, remaining, br.cloneNode());
    }
    output_div.addNodes(node_list);
}
//
//
function validate_services_performed_form(form) {
    return AFW_HUB.FormUtil.auto_validate(form);
}
//
//
function submit_services_performed_form(form) {
    var errors = validate_services_performed_form(form),
        form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        username = window.sessionStorage.username,
        args;
    //
    // returning early if there are errors
    if (errors) {
        validate_services_performed_form(form);
        return;
    }
    if (!confirm('Confirm update of services performed.')) {
        return;
    }
    //
    // adding modify dates to form values
    form_values.services_performed_last_modified = Date.current_ts();
    form_values.services_performed_last_modified_by = username;
    //
    args = [form.name, form_values, reset_services_performed_form];
    AFW_HUB.Ajax.submit_form_values.apply(null, args);
}
//
//
function reset_services_performed_form() {
    //
    alert('Successful update of Vendor/ Broker services performed information.');
    //
    document.getElementById('vendor-broker-table').dispatchPrimaryEvent();
    document.getElementById('content-div').removeAll();
    document.getElementById('content-header').textContent = '';
}
