/*global AFW_HUB*/
////////////////////////////////////////////////////////////////////////////////
//////////////       This file holds the driver functions that are   ///////////
//////////////       associated with only the meat shop              ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
///////////////////// Page setup functions /////////////////////////////
//
// sets up the main aspects of the meat shop page
function meat_shop_page() {
    var nav_container = document.getElementById('nav-container'),
        nav_buttons = [
            { id: 'item_maintenance', func: item_maintenance },
            { id: 'update_stock', func: update_stock },
            { id: 'modify_stock_change_record', func: mod_change_records },
            { id: 'inventory_report', func: meat_shop_inventory_report },
            { id: 'stock_change_report', func: stock_change_report }
        ],
        button = document.createElementWithAttr('BUTTON', { class: 'big' }),
        clicked = AFW_HUB.Navigation.click_single_button(),
        i;
    //
    for (i = 0; i < nav_buttons.length; i++) {
        button = button.cloneNode();
        button.id = nav_buttons[i].id;
        button.addEventListener('click', nav_buttons[i].func);
        button.textContent = button.id.replace(/_/g, ' ').toTitleCase();
        //
        nav_container.appendChild(button);
    }
    //
    // checking perms for each button
    for (i = 0; i < nav_buttons.length; i++) {
        AFW_HUB.Navigation.check_user_perms('meat_shop.' + nav_buttons[i].id);
    }
    if (clicked) {
        AFW_HUB.Utilities.add_class('hidden-elm', 'home_page');
        AFW_HUB.Utilities.add_class('hidden-elm', clicked);
    }
}
//
// this sets up the page if the item maintence button is clicked
function item_maintenance() {
    //
    reset_meat_page();
    document.getElementById('tab-clicked').value = 'item-maintenance';
    //
    var update_fun = gen_item_table.bind(null, 1, 'item_number', 'ASC');
    var head_str = 'Click on an item in the table to modfy it or click ' +
        'the button below the table to create a new item.';
    var head_elements = [ { elm: 'H4', textNode: head_str } ];
    //
    document.getElementById('input-div').add_children(head_elements);
    gen_item_table_filter(update_fun);
    gen_item_table(1, 'item_number', 'ASC');
}
//
// this sets up the page if stock changes is clicked
function update_stock() {
    //
    reset_meat_page();
    document.getElementById('tab-clicked').value = 'stock-changes';
    //
    var update_fun = gen_item_table.bind(null, 1, 'item_number', 'ASC');
    var head_elements = [
        {
            elm: 'H4',
            textNode: 'Click on an item in the table to update it\'s quantity.'
        }
    ];
    //
    //
    document.getElementById('input-div').add_children(head_elements);
    gen_item_table_filter(update_fun);
    gen_item_table(1, 'item_number', 'ASC');
}
//
// this sets up the page for modifying a stock change record
function mod_change_records() {
    //
    reset_meat_page();
    document.getElementById('tab-clicked').value = 'mod-stock-changes';
    //
    var update_fun = gen_stock_table.bind(null, 1, 'creation_timestamp', 'DESC');
    var head_elements = [
        { elm: 'H4', textNode: 'Click on a record in the table to modify it.' },
        //
        {
            elm: 'fieldset',
            id: 'param-container',
            style: { padding: '0px' },
            children: [ { elm: 'legend', textNode: 'Selection Parameters' } ]
        }
    ];
    //
    document.getElementById('input-div').add_children(head_elements);
    gen_stock_change_filter(update_fun);
    gen_stock_table(1, 'creation_timestamp', 'DESC');
}
//
// handles page generation for an inventory report
function meat_shop_inventory_report() {
    //
    // clearing elements inside main-container div
    reset_meat_page();
    var header = 'Meat Shop Inventory Report';
    document.getElementById('page-header').textContent = header;
    //
    var update_fun = create_inventory_report.bind(null, '', '');
    gen_item_table_filter(update_fun);
    var sort_fieldset = document.getElementById('sorting-parameters');
    var elements = [
        { elm: 'label', style: { width: '7em' }, textNode: 'Sort Column:' },
        {
            elm: 'select',
            id: 'sort-col',
            name: 'sort-col',
            events: [ { event: 'change', function: update_fun } ]
        },
        { elm: 'text', textNode: '   ' },
        { elm: 'label', style: { width: '7em' }, textNode: 'Sort Direction' },
        {
            elm: 'select',
            id: 'sort-dir',
            name: 'sort-dir',
            events: [ { event: 'change', function: update_fun } ],
            children: [
                { elm: 'option', value: 'ASC', textNode: 'Ascending' },
                { elm: 'option', value: 'DESC', textNode: 'Descending' }
            ]
        }
    ];
    sort_fieldset.add_children(elements);
    AFW_HUB.Utilities.remove_class('hidden-elm', sort_fieldset);
    //
    var dropbox_args = {
        database: 'meta_database',
        sql_args: {
            cmd: 'SELECT',
            cols: [ 'column_name', 'column_meta_data.display_name' ],
            table: 'column_meta_data',
            inner_join: [
                ['table_meta_data',
                 'column_meta_data.internal_id',
                 'table_meta_data.column_internal_id']
            ],
            where: [
                [ 'use_on_pages', 'REGEXP', '(^|%)meat_shop(%|$)' ],
                [ 'html_table', 'LIKE', 'meat_shop_stock_report' ]
            ],
            order_by: [ [ 'column_name', 'ASC' ] ],
            group_by: 'column_meta_data.internal_id'
        },
        dropbox_id: 'sort-col',
        text_format: '%display_name%',
        value_format: '%column_name%'
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    create_inventory_report('', '');
}
//
// this handles page creation and generation of a stock change report
function stock_change_report() {
    //
    // clearing elements inside main-container div
    reset_meat_page();
    //
    var header = 'Stock Change Record Report';
    var update_fun = show_update_button.bind(
        null,
        'create-report',
        'stock-change-report-table',
        'Show Changes'
    );
    var head_elements = [
        {
            elm: 'fieldset',
            id: 'param-container',
            style: { padding: '0px' },
            children: [ { elm: 'legend', textNode: 'Stock Report Parameters' } ]
        }
    ];
    //
    var button = document.createElementWithAttr('BUTTON', {
        id: 'create-report',
        type: 'button'
    });
    button.addEventListener('click', create_change_report.bind(null, '', ''));
    button.addTextNode('Create Report');
    //
    document.getElementById('page-header').textContent = header;
    document.getElementById('input-div').add_children(head_elements);
    document.getElementById('input-div').appendChild(button);
    gen_stock_change_filter(update_fun);
}
//
// this resets the page after main buttons are pressed
function reset_meat_page() {
    //
    // clearing elements inside main-container div
    var childNodes = document.getElementById('main-container').childNodes;
    for (var i = 0; i < childNodes.length; i++) {
        if (childNodes[i].nodeType == 1) {
            childNodes[i].removeAll();
        }
    }
}
////////////////////////////////////////////////////////////////////////
///////////////////// Table Generation Function ////////////////////////
//
// creates the item table filter
function gen_item_table_filter(update_function) {
    //
    // creating the main params container
    var output_id = 'input-div';
    var param_fieldset = document.createElementWithAttr('FIELDSET');
    param_fieldset.id = 'param-container';
    param_fieldset.style.padding = '0px';
    var elements = [ { elm: 'legend', textNode: 'Selection Parameters' } ];
    param_fieldset.add_children(elements);
    document.getElementById(output_id).appendChild(param_fieldset);
    //
    // adding a filter fieldset
    var status_onlick = function() {
        AFW_HUB.FormUtil.toggle_checkbox_value.call(null, this, '.', '^active$');
    };
    elements = [
        //
        { elm: 'label', textNode: 'Show Deleted Items:' },
        {
            elm: 'input',
            id: 'filter-item-status',
            name: 'item_status',
            type: 'checkbox',
            value: '^active$',
            events: [
                { event: 'click', function: status_onlick },
                { event: 'click', function: update_function }
            ]
        },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Filter by Item Number:' },
        {
            elm: 'input',
            id: 'filter-item-number',
            name: 'item_number',
            type: 'text',
            value: '',
            events: [ { event: 'keyup', function: update_function } ]
        },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Filter by Item Name:' },
        {
            elm: 'input',
            id: 'filter-item-name',
            name: 'item_name',
            type: 'text',
            value: '',
            events: [ { event: 'keyup', function: update_function } ]
        },
        { elm: 'br' }
    ];
    //
    var props = { style: { border: 'none' } };
    var fieldset = document.createElementWithAttr('FIELDSET', props);
    fieldset.id = 'selection-parameters';
    fieldset.add_children(elements);
    param_fieldset.appendChild(fieldset);
    //
    fieldset = document.createElementWithAttr('FIELDSET', props);
    fieldset.id = 'sorting-parameters';
    fieldset.className += ' hidden-elm';
    param_fieldset.appendChild(fieldset);
    //
    document.getElementById(output_id).appendChild(document.createElement('BR'));
}
//
// creates the stock change records filter
function gen_stock_change_filter(update_function) {
    //
    // adding a filter fieldset
    var output_id = 'param-container';
    var fieldset = document.createElement('FIELDSET');
    var status_onlick = function() {
        var box = document.getElementById('filter-entry-status');
        AFW_HUB.FormUtil.toggle_checkbox_value(box, '', 'submitted');
    };
    var sel_params_elements = Array(
        //
        { elm: 'label', textNode: 'Show Deleted Entries:' },
        {
            elm: 'input',
            id: 'filter-entry-status',
            name: 'entry_status',
            type: 'checkbox',
            value: 'submitted',
            events: [
                { event: 'click', function: status_onlick },
                { event: 'click', function: update_function }
            ]
        },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Filter by Item Number:' },
        {
            elm: 'input',
            id: 'filter-item-number',
            name: 'meat_shop_stock.item_number',
            type: 'text',
            value: '',
            events: [ { event: 'keyup', function: update_function } ]
        },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Filter by Item Name:' },
        {
            elm: 'input',
            id: 'filter-item-name',
            name: 'meat_shop_stock.item_name',
            type: 'text',
            value: '',
            events: [ { event: 'keyup', function: update_function } ]
        },
        { elm: 'br' },
        //
        { elm: 'label', textNode: 'Filter by Entering User:' },
        {
            elm: 'input',
            id: 'filter-entering-user',
            name: 'entering_user',
            type: 'text',
            value: '',
            events: [ { event: 'keyup', function: update_function } ]
        },
        { elm: 'br' }
    );
    var time_range_elements = Array({
        elm: 'fieldset',
        id: 'time-range-inputs',
        style: { border: 'none' }
    });
    //
    fieldset.id = 'selection-parameters';
    fieldset.style.border = 'none';
    fieldset.add_children(sel_params_elements);
    document.getElementById(output_id).add_children(time_range_elements);
    document.getElementById(output_id).appendChild(fieldset);
    document.getElementById(output_id).appendChild(document.createElement('BR'));
    //
    var input_args = { output_id: 'time-range-inputs', update_fun: update_function };
    create_time_range_inputs(input_args);
}
//
// generates the table for the item maintanence tab
function gen_item_table(page, sort_col, sort_dir) {
    //
    var params_container = document.getElementById('selection-parameters'),
        item_table_args = {
            table_name: 'meat_shop_items',
            table_output_id: 'table-div',
            table_attr: { id: 'meat-shop-items' },
            data_sql_args: { order_by: [ [ sort_col, sort_dir ] ] },
            head_row_args: {
                sortable: true,
                tables_referenced: [ 'meat_shop_stock' ],
                sort_col: sort_col,
                sort_dir: sort_dir,
                sort_onclick: [
                    {
                        func: gen_item_table,
                        args: [ 1, '%column_name%', '%sort_dir%' ]
                    }
                ]
            },
            page_nav_args: {
                curr_page: page,
                sort_col: sort_col,
                sort_dir: sort_dir,
                tot_pages_shown: 9,
                num_per_page: 10,
                page_nav_div_attr: { id: 'item-table-page-nav' },
                id_prefix: 'item',
                link_onclick: [
                    { func: gen_item_table, args: [ '%page%', sort_col, sort_dir ] }
                ]
            }
        };
    //
    // setting up the data and meta SQL args
    AFW_HUB.TableUtil.get_table_inputs(
        item_table_args.data_sql_args,
        params_container
    );
    //
    // setting tab specifc values
    if (document.getElementById('tab-clicked').value === 'item-maintenance') {
        item_table_args.add_callback = function() {
            var br = document.createElementWithAttr('BR', { id: 'create-spacer' });
            var button = document.createElementWithAttr('BUTTON', {
                id: 'create-new-item',
                type: 'button'
            });
            button.style.display = 'block';
            button.style.margin = 'auto';
            button.addEventListener('click', mod_item.bind(null, null, null));
            button.textContent = 'Create New Item';
            document.getElementById('table-div').safeAppendChild(br);
            document.getElementById('table-div').safeAppendChild(button);
        };
        item_table_args.row_onclick = [
            { func: mod_item, args: [ '%item_number%', null, undefined ] }
        ];
    } else if (document.getElementById('tab-clicked').value === 'stock-changes') {
        item_table_args.table_callback = function() {};
        item_table_args.row_onclick = [
            { func: update_item_stock, args: [ '%row_id%', null, undefined ] }
        ];
    }
    //
    AFW_HUB.TableUtil.create_standard_table(item_table_args);
}
//
// generates the table for modifying a stock change record
function gen_stock_table(page, sort_col, sort_dir) {
    //
    var fieldset = document.getElementById('time-range-inputs'),
        ts_obj = to_and_from_timestamps(fieldset),
        params_container = document.getElementById('selection-parameters'),
        table_args = {
            table_name: 'meat_shop_records',
            table_output_id: 'table-div',
            table_attr: { id: 'meat-shop-records' },
            row_id_prefix: 'record-row',
            row_onclick: [
                { func: update_item_stock, args: [ '%row_id%', '%entry_id%', undefined ] }
            ],
            data_sql_args: {
                cmd: 'SELECT',
                table: 'meat_shop_stock_changes',
                cols: [ 'meat_shop_stock_changes.*', 'meat_shop_stock.item_name' ],
                inner_join: [
                    [
                        'meat_shop_stock',
                        'meat_shop_stock.item_internal_id',
                        'meat_shop_stock.item_internal_id'
                    ]
                ],
                where: [ [ 'date', 'BETWEEN', [ ts_obj.from_ts, ts_obj.to_ts ] ] ],
                order_by: [ [ sort_col, sort_dir ] ]
            },
            head_row_args: {
                sortable: true,
                sort_col: sort_col,
                sort_dir: sort_dir,
                sort_onclick: [
                    {
                        func: gen_stock_table,
                        args: [ 1, '%column_name%', '%sort_dir%' ]
                    }
                ]
            },
            page_nav_args: {
                curr_page: page,
                sort_col: sort_col,
                sort_dir: sort_dir,
                tot_pages_shown: 9,
                num_per_page: 10,
                page_nav_div_attr: { id: 'record-table-page-nav' },
                id_prefix: 'item-record',
                link_onclick: [
                    { func: gen_stock_table, args: [ '%page%', sort_col, sort_dir ] }
                ]
            }
        };
    //
    AFW_HUB.TableUtil.get_table_inputs(table_args.data_sql_args, params_container);
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
////////////////////////////////////////////////////////////////////////
/////////////////////// Item Form Functions ////////////////////////////
//
// This function handles the modifcation of an item
function mod_item(item_number, form) {
    var content_div = document.getElementById('content-div'),
        sub_button = document.getElementById('submit-form'),
        header = 'Creating new Item',
        validate = function(act) {
            validate_item_form(form, act, undefined);
        },
        event_args, form_args, button, callback;
    //
    // loading form
    if (!form) {
        callback = mod_item.bind(null, item_number);
        AFW_HUB.FormUtil.load_html_form('meat_shop_item', content_div, callback);
        return;
    }
    //
    // add event listeners args
    event_args = {
        add_default_events: function(elm) {
            var event = elm.primaryEventString(),
                remove_class = AFW_HUB.Utilities.remove_class,
                round_form_inputs = AFW_HUB.FormUtil.round_form_inputs;
            if (elm.nodeName.toLowerCase() == 'button') {
                return;
            }
            if (!event) {
                return;
            }
            //
            elm.addEventListener(event, remove_class.bind(null, 'boxshadow-red', elm));
            elm.addEventListener('blur', round_form_inputs.bind(null, form, 2)
            );
        }
    };
    //
    // form population args
    form_args = {
        form_id: form.id,
        trigger_events: true,
        meta_data_only: 1,
        sql_args: {
            table: 'meat_shop_stock',
            where: [ [ 'item_number', 'LIKE', item_number ] ]
        },
        data_preprocessor: function(args, response) {
            event_args.meta_array = response.meta_data;
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
        }
    };
    //
    sub_button.addEventListener('click', validate.bind(false, 'creation'));
    if (item_number) {
        header = 'Modfiying Item: ' + item_number;
        document.getElementById('item-number').disabled = true;
        //
        // adding buttons
        button = sub_button.cloneNode();
        button.textContent = 'Update Item';
        button.addEventListener('click', validate.bind(false, 'modification'));
        form.replaceChild(button, sub_button);
        //
        button = sub_button.cloneNode();
        button.id = 'delete-item';
        button.textContent = 'Delete Item';
        button.addEventListener('click', validate.bind(false, 'deletion'));
        form.safeAppendChild(button);
        //
        button = sub_button.cloneNode();
        button.id = 'restore-item';
        button.textContent = 'Restore Item';
        button.addEventListener('click', validate.bind(false, 'restoration'));
        form.safeAppendChild(button);
        //
        // function to show buttons based on status
        form_args.meta_data_only = undefined;
        form_args.add_callback = function() {
            //
            if (document.getElementById('item-status').value !== 'active') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'delete-item');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'restore-item');
            }
        };
    }
    document.getElementById('content-header').textContent = header;
    AFW_HUB.FormUtil.populate_form(form_args);
}
//
// this validates the meat_shop_item form
function validate_item_form(form, act, unique_err) {
    var errors = false, args;
    //
    // checking if the item number is unique
    if (unique_err === undefined) {
        args = [
            document.getElementById('item-number'),
            document.getElementById('item-internal-id'),
            'meat_shop_stock',
            null,
            validate_item_form.bind(window, form, act)
        ];
        AFW_HUB.FormUtil.check_unique.apply(null, args);
        return;
    }
    errors = AFW_HUB.FormUtil.auto_validate(form);
    if (errors) {
        return false;
    }
    //
    submit_item_form(form, act);
}
//
// submits the item form
function submit_item_form(form, act) {
    var form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        callback = reset_item_form.bind(null, act);
    //
    if (!confirm('Confirm ' + act + ' of item.')) {
        return;
    }
    //
    // action based values
    if (act === 'creation') {
        delete form_values.item_internal_id;
        form_values.item_status = 'active';
    }
    else if (act === 'deletion') {
        form_values.item_status = 'inactive';
    }
    else if (act === 'restoration') {
        form_values.item_status = 'active';
    }
    //
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
//
function reset_item_form(act) {
    //
    alert('Successful ' + act + ' of Item.');
    //
    document.getElementById('meat-shop-items').dispatchPrimaryEvent();
    document.getElementById('content-header').textContent = '';
    document.getElementById('content-div').removeAll();
}
////////////////////////////////////////////////////////////////////////
/////////////////////// Stock Form Functions ///////////////////////////
//
// this creates an entry to update the stock in the items table
function update_item_stock(row_id, entry_id, form) {
    var content_div = document.getElementById('content-div'),
        sub_button = document.getElementById('submit-form'),
        header = '',
        validate = function(act) {
            validate_stock_form(form, act);
        },
        quantity_calc = function() {
            var amt = document.getElementById('amount').value;
            var qty = document.getElementById('curr-quantity').value;
            //
            var value = Number(amt) + Number(qty);
            document.getElementById('new-quantity').value = value;
        },
        event_args, form_args, row_data, button, callback;
    row_data = document.getElementById(row_id).dataset;
    //
    // loading form
    if (form === undefined) {
        callback = update_item_stock.bind(null, row_id, entry_id);
        AFW_HUB.FormUtil.load_html_form('stock_change_form', content_div, callback);
        return;
    }
    //
    // add event listeners args
    event_args = {
        add_default_events: function(elm) {
            var event = elm.primaryEventString(),
                remove_class = AFW_HUB.Utilities.remove_class,
                round_form_inputs = AFW_HUB.FormUtil.round_form_inputs;
            if (elm.nodeName.toLowerCase() == 'button') {
                return;
            }
            if (!event) {
                return;
            }
            //
            elm.addEventListener(event, remove_class.bind(null, 'boxshadow-red', elm));
            elm.addEventListener('blur', round_form_inputs.bind(null, form, 2)
            );
        }
    };
    // populate form args
    form_args = {
        form_id: form.id,
        trigger_events: true,
        data_preprocessor: function(args, response) {
            event_args.meta_array = response.meta_data;
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
            document.getElementById('amount').addEventListener('keyup', quantity_calc);
            document.getElementById('amount').addEventListener('blur', quantity_calc);
        }
    };
    //
    // setting page for creation or modification of a stock record
    if (entry_id) {
        header = 'Modfiying Record: {entry_id} for Item: {item_number} - {item_name}';
        header = header.format(row_data);
        //
        // unhiding form elements to help with updating an amount
        AFW_HUB.Utilities.remove_class('hidden-elm', 'orig-amount-label');
        AFW_HUB.Utilities.remove_class('hidden-elm', 'orig-amount');
        document.getElementById('amount-label').textContent = 'New Add/Remove Pounds:';
        //
        // modifying the quantity calc function to handle modifications
        quantity_calc = function() {
            var org_qty = Number(document.getElementById('curr-quantity').value),
                org_amt = Number(document.getElementById('orig-amount').value),
                new_amt = Number(document.getElementById('amount').value),
                new_qty = document.getElementById('new-quantity');
            //
            new_qty.value = org_qty - org_amt + new_amt;
        };
        //
        // adding buttons
        button = sub_button.cloneNode();
        button.textContent = 'Update Record';
        button.addEventListener('click', validate.bind(false, 'modification'));
        form.replaceChild(button, sub_button);
        //
        button = sub_button.cloneNode();
        button.id = 'delete-record';
        button.textContent = 'Delete Record';
        button.addEventListener('click', validate.bind(false, 'deletion'));
        form.safeAppendChild(button);
        //
        button = sub_button.cloneNode();
        button.id = 'restore-record';
        button.textContent = 'Restore Record';
        button.addEventListener('click', validate.bind(false, 'restoration'));
        form.safeAppendChild(button);
        //
        // setting sql ags and form population callback to show buttons
        form_args.sql_args = {
            where: [
                ['meat_shop_stock_changes.entry_id', 'LIKE', entry_id]
            ]
        };
        form_args.add_callback = function() {
            //
            if (document.getElementById('entry-status').value != 'submitted') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'delete-record');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'restore-record');
            }
        };
    }
    else {
        sub_button.addEventListener('click', validate.bind(null, 'creation'));
        form.name = 'meat_shop_item';
        form_args.sql_args = {
            where: [
                ['meat_shop_stock.item_number', 'LIKE', row_data.item_number]
            ]
        };
        form_args.add_callback = function() {
            form.name = 'stock_change_form';
            document.getElementById('date').value = new Date().yyyymmdd();
        };
    }
    document.getElementById('content-header').textContent = header;
    AFW_HUB.FormUtil.populate_form(form_args);
}
//
// this validates the stock change form
function validate_stock_form(form, act) {
    var error = AFW_HUB.FormUtil.auto_validate(form);
    if (error) {
        return;
    }
    submit_stock_form(form, act);
}
//
// handles submission of the stock form
function submit_stock_form(form, act) {
    var msg = 'Confirm ' + act + ' of stock change record',
        orig_amount = document.getElementById('orig-amount').value,
        new_quantity = document.getElementById('new-quantity').value,
        curr_quantity = document.getElementById('curr-quantity').value,
        callback = reset_stock_form.bind(null, act),
        form_values;
    //
    if (!confirm(msg)) {
        return;
    }
    //
    form_values = AFW_HUB.FormUtil.get_all_form_values(form);
    //
    // action specific changes to quantity form values
    form_values.quantity = Number(new_quantity);
    form_values.meat_shop_stock_last_modified = Date.current_ts();
    form_values.meat_shop_stock_last_modified_by = window.sessionStorage.username;
    if (act == 'creation') {
        delete form_values.entry_id;
        delete form_values.meat_shop_stock_last_modified;
        delete form_values.meat_shop_stock_last_modified_by;
        form_values.entry_status = 'submitted';
        form_values.entering_user = window.sessionStorage.username;
    } else if (act == 'deletion') {
        delete form_values.amount;
        form_values.entry_status = 'deleted';
        form_values.quantity = Number(curr_quantity) - Number(orig_amount);
    } else if (act == 'restoration') {
        delete form_values.amount;
        form_values.entry_status = 'submitted';
        form_values.quantity = Number(curr_quantity) + Number(orig_amount);
    }
    //
    AFW_HUB.Ajax.submit_form_values(form.name, form_values, callback);
}
//
// resets page elements after successful submission to the database
function reset_stock_form(act) {
    //
    if (document.getElementById('meat-shop-items')) {
        document.getElementById('meat-shop-items').dispatchPrimaryEvent();
    }
    if (document.getElementById('meat-shop-records')) {
        document.getElementById('meat-shop-records').dispatchPrimaryEvent();
    }
    document.getElementById('content-header').removeAll();
    document.getElementById('content-div').removeAll();
    //
    alert('Successful ' + act + ' of stock change record.');
}
////////////////////////////////////////////////////////////////////////
////////////////////////// Reporting Functions /////////////////////////
//
// this handles inputs for the meat shop reports
function create_inventory_report() {
    //
    var params_container = document.getElementById('selection-parameters'),
        table_args = {
            table_name: 'meat_shop_stock_report',
            table_output_id: 'content-div',
            table_attr: { id: 'meat-shop-report' },
            row_id_prefix: 'report-row',
            no_page_nav: true,
            data_sql_args: {
                cmd: 'SELECT',
                table: 'meat_shop_stock',
                cols: [ 'meat_shop_stock.*', '(quantity*cost_per_lb)`total_value`' ],
                order_by: []
            },
            page_nav_args: {},
            head_row_args: {
                sortable: false,
                tables_referenced: [ 'meat_shop_stock' ]
            },
            add_callback: build_inventory_report_totals
        };
    //
    AFW_HUB.TableUtil.get_table_inputs(table_args.data_sql_args, params_container);
    params_container = document.getElementById('sorting-parameters');
    AFW_HUB.TableUtil.get_table_inputs(table_args.data_sql_args, params_container);
    table_args.data_sql_args.order_by.push([ 'item_number', 'ASC' ]);
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
// this function modifies the table output by create_meat_shop_report
function build_inventory_report_totals(response, table) {
    var data = response.table_data,
        meta_data = response.meta_data.to_object('column_name'),
        row = document.createElement('TR'),
        td = document.createElement('TD'),
        totals = {},
        i,
        column,
        args;
    //
    // initializing the totals row
    for (column in meta_data) {
        totals[column] = 0;
    }
    //
    // summing up all valuess
    for (i = 0; i < data.length; i++) {
        for (column in data[i]) {
            totals[column] += Number(data[i][column]);
        }
    }
    //
    // outputting totals row
    for (column in meta_data) {
        if (isNaN(totals[column])) {
            totals[column] = '';
        }
        td = td.cloneNode();
        args = [
            totals[column],
            meta_data[column].data_type,
            td,
            { format_str: '%value%', add_commas: true }
        ];
        AFW_HUB.TableUtil.display_table_data.apply(null, args);
        row.appendChild(td);
    }
    row.style.fontWeight = 'bold';
    row.firstChild.textContent = 'Totals:';
    table.appendChild(row);
}
//
// generates the table for modifying a stock change record
function create_change_report(sort_col, sort_dir) {
    var fieldset = document.getElementById('time-range-inputs'),
        old_btn = document.getElementById('create-report'),
        new_btn = document.createElement('BUTTON'),
        params_container = document.getElementById('selection-parameters'),
        ts_obj = to_and_from_timestamps(fieldset),
        table_args = {
            table_name: 'meat_shop_stock_change_report',
            table_output_id: 'content-div',
            table_attr: { id: 'stock-change-report-table' },
            no_page_nav: true,
            data_sql_args: {
                cmd: 'SELECT',
                table: 'meat_shop_stock_changes',
                where: [ [ 'date', 'BETWEEN', [ ts_obj.from_ts, ts_obj.to_ts ] ] ],
                order_by: [
                    [ sort_col || 'date', sort_dir || 'DESC' ],
                    [ 'meat_shop_stock.item_number', 'ASC' ]
                ]
            },
            head_row_args: {
                sortable: true,
                tables_referenced: [ 'meat_shop_stock_changes' ],
                sort_col: sort_col,
                sort_dir: sort_dir,
                sort_onclick: [
                    {
                        func: create_change_report,
                        args: [ '%column_name%', '%sort_dir%' ]
                    }
                ]
            },
            page_nav_args: {}
        };
    AFW_HUB.TableUtil.get_table_inputs(table_args.data_sql_args, params_container);
    //
    // creating new get data button to prevent stacking of event handlers
    new_btn.appendChild(document.createTextNode('Create Report'));
    new_btn.id = 'create-report';
    new_btn.className = 'hidden-elm';
    new_btn.addEventListener('click', create_change_report.bind(null, '', ''));
    old_btn.parentNode.replaceChild(new_btn, old_btn);
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
