/* global AFW_HUB */
////////////////////////////////////////////////////////////////////////////////
/////////// This file holds various miscellaneous functions that       /////////
/////////// are widely used throughout the hub                         /////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
//
////////////////////////////////// Suggestions ////////////////////////////////
//
// sets up the suggestion pge
function leave_suggestion_page() {
    var main_container = document.getElementById('main-container'),
        label = document.createElement('LABEL'),
        textarea = document.createElementWithAttr('TEXTAREA', { id: 'description' }),
        button = document.createElement('BUTTON');
    AFW_HUB.Utilities.add_class('hidden-elm', 'head');
    //
    // setting up page
    main_container.removeAll();
    label.textContent = 'Suggestion:';
    //
    textarea.name = 'description';
    textarea.rows = 4;
    textarea.cols = 60;
    //
    button.addEventListener('click', submit_suggestion);
    button.textContent = 'Submit';
    //
    main_container.appendChild(label);
    main_container.appendChild(document.createElement('BR'));
    main_container.appendChild(textarea);
    main_container.appendChild(document.createElement('BR'));
    main_container.appendChild(button);
}
//
// this generates the popup window to leave a suggestion
function enter_suggestion() {
    //
    window.open(
        'entry_page.php?tab=leave_suggestion',
        'suggestionWindow',
        'height=150,width=600,scrollbars=yes'
    );
    window.focus();
}
//
// this takes the user to the table maintence page to view the suggestions
function view_suggestions(status) {
    //
    var sug_params = '&gen_suggestions=true&status=' + status;
    location.href = 'content_page.php?tab=table_maintenance' + sug_params;
}
//
// this submits the suggestion to the database
function submit_suggestion() {
    //
    // getting values
    var form_vals = {
            submission_date: new Date().yyyymmdd(),
            suggestion_description: document.getElementById(
            'description'
        ).value.toLowerCase(),
            entering_user: document.getElementById('user-username').value
        },
        callback;
    //
    callback = function() {
        alert('Thank you for your suggestion!');
        window.close();
    };
    AFW_HUB.Ajax.submit_form_values('leave_suggestion', form_vals, callback);
}
//
//////////////////////////////////////////////////////////////////////////
//
// toggles an element created by a button, updating the button's string
// to reflect visibilty change if the button's value is update
// then the changes are applied instead of toggling visibility
function toggle_view_element_button(button, element, hide_str, show_str) {
    //
    // getting elements if IDs were supplied instead
    if (!button.nodeName) {
        button = document.getElementById(button);
    }
    if (!element.nodeName) {
        element = document.getElementById(element);
    }
    //
    // checking value of button, if update returning with no changes
    if (button.value === 'update') {
        button.value = 'view';
        AFW_HUB.Utilities.remove_class('hidden-elm', element);
        AFW_HUB.Utilities.remove_class('boxshadow-blue', button);
    } else if (button.value === 'update-button') {
        button.value = 'view';
    } else {
        AFW_HUB.Utilities.show_hide(element);
    }
    //
    // updating button string
    if (element.className.match('hidden-elm')) {
        button.textContent = show_str;
    } else {
        button.textContent = hide_str;
    }
}
//
// shows an update viewable data button or updates an existing button
// with a class and string
function show_update_button(button_id, data_table_id, update_str) {
    //
    // making button visible if data table has already been generated
    if (document.getElementById(data_table_id)) {
        AFW_HUB.Utilities.remove_class('hidden-elm', button_id);
        AFW_HUB.Utilities.add_class('boxshadow-blue', button_id);
        if (update_str != '') {
            document.getElementById(button_id).textContent = update_str;
            document.getElementById(button_id).value = 'update';
        }
    }
}
//
// this function specifically populates the year dropboxes
// based on the first_year_with_data value in operations_constants table
function populate_year_dropboxes(dropbox) {
    //
    var curr_year = new Date().getFullYear();
    var first_year = AFW_HUB.get_constant('FIRST_YEAR_WITH_DATA');
    var opt = null;
    for (var y = curr_year; y >= first_year; y--) {
        opt = document.createElementWithAttr('OPTION', { value: y });
        opt.addTextNode(y);
        dropbox.appendChild(opt);
    }
}
//
// creates the elm objects for each option in the time range dropbox
function gen_time_range_options() {
    var ranges = {
            'yesterday': [ new Date(), new Date() ],
            'current-week': [ new Date(), new Date() ],
            'current-pay-period': [ new Date(), new Date() ],
            'previous-pay-period': [ new Date(), new Date() ],
            'last-2-weeks': [ new Date(), new Date() ],
            'last-4-weeks': [ new Date(), new Date() ],
            'current-month': [ new Date(), new Date() ]
        },
        d,
        prop;
    //
    // updating date objects to have proper offsets
    d = null;
    ranges['yesterday'][0].setDate(ranges['yesterday'][0].getDate() - 1);
    d = ranges['current-week'][0];
    ranges['current-week'][0].setDate(d.getDate() - d.getDay());
    ranges['current-week'][1].setDate(d.getDate() + 6);
    d = ranges['last-2-weeks'][0];
    ranges['last-2-weeks'][0].setDate(d.getDate() - d.getDay() - 7);
    ranges['last-2-weeks'][1] = new Date(ranges['current-week'][1]);
    d = ranges['last-4-weeks'][0];
    ranges['last-4-weeks'][0].setDate(d.getDate() - d.getDay() - 21);
    ranges['last-4-weeks'][1] = new Date(ranges['current-week'][1]);
    d = ranges['current-month'][0];
    ranges['current-month'][0].setDate(1);
    ranges['current-month'][0].setDate(d.getDate() - d.getDay());
    d = ranges['current-month'][1];
    ranges['current-month'][1].setDate(1);
    ranges['current-month'][1].setMonth(d.getMonth() + 1);
    ranges['current-month'][1].setDate(d.getDate() + 6 - d.getDay());
    //
    // converting date objects to yyyy-mm-dd strings
    for (prop in ranges) {
        ranges[prop] = [ ranges[prop][0].yyyymmdd(), ranges[prop][1].yyyymmdd() ];
    }
    //
    // adding pay period ranges
    ranges['current-pay-period'] = find_pay_period();
    d = new Date(ranges['current-pay-period'][0]);
    d.setDate(d.getDate() - 1);
    ranges['previous-pay-period'] = find_pay_period(d);
    //
    // generating options array
    var options = [];
    var elm_obj = null;
    for (prop in ranges) {
        elm_obj = {
            'elm': 'OPTION',
            'value': ranges[prop].join('|'),
            'textNode': prop.replace(/-/g, ' ').toTitleCase(),
            'data-start_date': ranges[prop][0],
            'data-end_date': ranges[prop][1]
        };
        if (prop == 'current-pay-period') {
          elm_obj['selected'] = true; // probably want to see the curret PP most of the time
        }
        //
        options.push(elm_obj);
    }
    //
    return options;
}
//
// creates the elm objects for each option in the month dropxes
function gen_month_options() {
    //
    var opts = [
        'January',
        'February',
        'March',
        'April',
        'May',
        'June',
        'July',
        'August',
        'September',
        'October',
        'November',
        'December'
    ];
    for (var i = 0; i < opts.length; i++) {
        opts[i] = { elm: 'OPTION', value: i + 1, textNode: opts[i] };
    }
    //
    return opts;
}
//
// this creates the time range inputs for a page
function create_time_range_inputs(input_args) {
    //
    var output_element = document.getElementById(input_args.output_id);
    var cal_image = 'http://www.afwendling.com/operations/images/calander.png';
    //
    // creating initial elements required
    var label = document.createElement('LABEL');
    var range_select = document.createElementWithAttr('SELECT', {
        name: 'time_range'
    });
    var toggle_button = document.createElementWithAttr('BUTTON', { type: 'button' });
    var date_range_div = document.createElementWithAttr('DIV', {
        class: 'hidden-elm'
    });
    label.textContent = 'Time Range:';
    range_select.add_children(gen_time_range_options(null));
    range_select.addEventListener('change', input_args.update_fun);
    toggle_button.textContent = 'Show Date Range';
    //
    output_element.addNodes([ label, range_select, toggle_button, date_range_div ]);
    //
    // creating date range elements
    var month_opts = gen_month_options();
    var date_range_elms = [
        { elm: 'SPAN', class: 'cal-span hidden-elm' },
        { elm: 'LABEL', style: { width: '4em' }, textNode: 'From:' },
        {
            elm: 'INPUT',
            name: 'st-day',
            style: { 'width': '2em' },
            value: '01',
            disabled: true
        },
        {
            elm: 'SELECT',
            name: 'st-month',
            style: { minWidth: 'initial', margin: '5px' },
            children: month_opts,
            disabled: true
        },
        {
            elm: 'SELECT',
            name: 'st-year',
            style: { minWidth: 'initial', margin: '5px' },
            disabled: true
        },
        { elm: 'IMG', class: 'cal-image', src: cal_image },
        { elm: 'BR' },
        { elm: 'LABEL', style: { 'width': '4em' }, textNode: 'To:' },
        {
            elm: 'INPUT',
            name: 'en-day',
            style: { width: '2em' },
            value: '01',
            disabled: true
        },
        {
            elm: 'SELECT',
            name: 'en-month',
            style: { minWidth: 'initial', margin: '5px' },
            children: month_opts,
            disabled: true
        },
        {
            elm: 'SELECT',
            name: 'en-year',
            style: { minWidth: 'initial', margin: '5px' },
            disabled: true
        },
        { 'elm': 'IMG', 'class': 'cal-image', 'src': cal_image }
    ];
    //
    // outputting date range elms to the div and building node arrays
    date_range_div.add_children(date_range_elms);
    date_range_elms = date_range_div.querySelectorAll('DIV > *');
    var cal_container = date_range_div.querySelector('SPAN');
    var cal_imgs = date_range_div.querySelectorAll('DIV > IMG');
    //
    // adding event handlers to date input/select fields
    var elms = {};
    var event = null;
    for (var i = 0; i < date_range_elms.length; i++) {
        if (!date_range_elms[i].name) {
            continue;
        }
        elms[date_range_elms[i].name] = date_range_elms[i];
        event = date_range_elms[i].primaryEventString();
        date_range_elms[i].addEventListener(event, input_args.update_fun);
    }
    //
    // populating year dropboxes
    populate_year_dropboxes(elms['st-year']);
    populate_year_dropboxes(elms['en-year']);
    //
    // defining JS event handlers
    var toggle_date_range = toggle_view_element_button.bind(
        null,
        toggle_button,
        date_range_div,
        'Hide Date Range',
        'Show Date Range'
    );
    var toggle_inputs = function(container, range_select, node_list) {
        var disable = !!container.className.match(/hidden-elm/);
        range_select.disabled = !disable;
        for (var i = 0; i < node_list.length; i++) {
            node_list[i].disabled = disable;
        }
    };
    toggle_inputs = toggle_inputs.bind(
        null,
        date_range_div,
        range_select,
        date_range_elms
    );
    var show_calendar = AFW_HUB.Utilities.show_hide.bind(null, cal_container);
    var from_img_onclick = create_calander.bind(
        null,
        cal_container,
        0,
        elms['st-day'],
        elms['st-month'],
        elms['st-year']
    );
    var to_img_onclick = create_calander.bind(
        null,
        cal_container,
        0,
        elms['en-day'],
        elms['en-month'],
        elms['en-year']
    );
    //
    // adding event handlers
    toggle_button.addEventListener('click', toggle_date_range);
    toggle_button.addEventListener('click', toggle_inputs);
    cal_imgs[0].addEventListener('click', from_img_onclick);
    cal_imgs[0].addEventListener('click', show_calendar);
    cal_imgs[1].addEventListener('click', to_img_onclick);
    cal_imgs[1].addEventListener('click', show_calendar);
}
//
// handles creation of a calander
function create_calander(out_elm, mon_shift, day_elm, mon_elm, year_elm) {
    mon_shift = Number(mon_shift);
    //
    // defining initial static variables
    var mon_arr = [
        'January',
        'February',
        'March',
        'April',
        'May',
        'June',
        'July',
        'August',
        'September',
        'October',
        'November',
        'December'
    ];
    var dow_arr = [ 'SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT' ];
    var todays_date = new Date();
    var adj_date = new Date(
        todays_date.getFullYear(),
        todays_date.getMonth() + mon_shift,
        1
    );
    //
    // putting selected class on date entries
    AFW_HUB.Utilities.remove_class_all('boxshadow-blue');
    AFW_HUB.Utilities.add_class('boxshadow-blue', day_elm);
    AFW_HUB.Utilities.add_class('boxshadow-blue', mon_elm);
    AFW_HUB.Utilities.add_class('boxshadow-blue', year_elm);
    //
    // building table header
    var cal_table = document.createElementWithAttr('TABLE', {
        id: 'calander',
        class: 'cal-table'
    });
    var row = document.createElement('TR');
    var td = document.createElementWithAttr('TD', {
        class: 'cal-table-month',
        colspan: '7'
    });
    td.style['white-space'] = 'initial';
    var elms = [
        {
            'elm': 'SPAN',
            'style': { 'float': 'left' },
            'textNode': ' <<',
            'events': [
                {
                    'event': 'click',
                    'function': create_calander.bind(
                        null,
                        out_elm,
                        mon_shift - 1,
                        day_elm,
                        mon_elm,
                        year_elm
                    )
                }
            ]
        },
        {
            'elm': 'SPAN',
            'textNode': mon_arr[adj_date.getMonth()] + '  ' + adj_date.getFullYear()
        },
        {
            'elm': 'SPAN',
            'style': { 'float': 'right' },
            'textNode': ' >>',
            'events': [
                {
                    'event': 'click',
                    'function': create_calander.bind(
                        null,
                        out_elm,
                        mon_shift + 1,
                        day_elm,
                        mon_elm,
                        year_elm
                    )
                }
            ]
        }
    ];
    td.add_children(elms);
    row.appendChild(td);
    cal_table.appendChild(row);
    //
    row = row.cloneNode();
    elms = [];
    for (var i = 0; i < dow_arr.length; i++) {
        elms.push({
            'elm': 'TD',
            'class': 'cal-table-wkhead',
            'textNode': dow_arr[i]
        });
    }
    row.add_children(elms);
    cal_table.appendChild(row);
    //
    // building calander table body
    adj_date = new Date(
        adj_date.getFullYear(),
        adj_date.getMonth(),
        1 - adj_date.getDay()
    );
    td = document.createElementWithAttr('TD', { 'class': 'cal-table-day' });
    for (var d = 0; d < 42; d++) {
        //
        if (d % 7 == 0) {
            cal_table.appendChild(row);
            row = row.cloneNode();
        }
        td = document.createElementWithAttr('TD', { 'class': 'cal-table-day' });
        td.dataset.day = adj_date.getDate();
        td.dataset.month = adj_date.getMonth() + 1;
        td.dataset.year = adj_date.getFullYear();
        td.addEventListener(
            'click',
            set_date.bind(td, day_elm, mon_elm, year_elm, out_elm)
        );
        td.textContent = adj_date.getDate();
        //
        row.appendChild(td);
        if (adj_date.yyyymmdd() == Date.current_ts().split(' ')[0]) {
            AFW_HUB.Utilities.add_class('cal-table-today', td);
        }
        adj_date.setDate(adj_date.getDate() + 1);
    }
    out_elm.safeAppendChild(cal_table);
}
//
// upates the actual input fields with the date selection from the calander
function set_date(day_elm, mon_elm, year_elm, cal_container) {
    //
    day_elm.value = this.dataset.day;
    mon_elm.value = this.dataset.month;
    year_elm.value = this.dataset.year;
    //
    // dispatching events to each element and hiding calander
    day_elm.dispatchPrimaryEvent();
    mon_elm.dispatchPrimaryEvent();
    year_elm.dispatchPrimaryEvent();
    //
    AFW_HUB.Utilities.add_class('hidden-elm', cal_container);
    AFW_HUB.Utilities.remove_class_all('boxshadow-blue');
}
//
// checks the parent element for time range values or date values
function to_and_from_timestamps(parent) {
    var from_ts = '';
    var to_ts = '';
    var time_values = AFW_HUB.FormUtil.get_all_form_values(parent);
    //
    // getting time range data
    if (time_values.hasOwnProperty('time_range')) {
        from_ts = time_values['time_range'].split('|')[0] + ' 00:00:00';
        to_ts = time_values['time_range'].split('|')[1] + ' 23:59:59';
    } else {
        // using calander inputs instead
        from_ts = '{st-year}-{st-month}-{st-day} 00:00:00'.format(time_values);
        to_ts = '{en-year}-{en-month}-{en-day} 23:59:59'.format(time_values);
    }
    //
    return { from_ts: from_ts, to_ts: to_ts };
}
//
// this determines the time beginning and end of a pay period for a given date
function find_pay_period(date) {
    var first_business_day = AFW_HUB.get_constant('FIRST_DAY_OF_PAY_PERIOD'),
        st_pp = new Date(first_business_day),
        test_date = st_pp,
        date_adj = 14,
        date_arr,
        ts_arr;
    //
    date = date || new Date();
    if (date instanceof Array) {
        if (date.length < 3) {
            throw new Error('Invalid date array: ' + date);
        }
        date = new Date(Number(date[0]), Number(date[1]) - 1, Number(date[2]));
    } else if (typeof date === 'string' || date instanceof String) {
        date_arr = date.match(/(\d+).(\d+).(\d+)/);
        if (date_arr.length < 4) {
            throw new Error('Invalid date string: ' + date);
        }
        date = new Date(
            Number(date_arr[1]),
            Number(date_arr[2]) - 1,
            Number(date_arr[3])
        );
    }
    if (date < st_pp) {
        date_adj = -14;
    }
    //
    // looping through to find period inclusive of the date
    while (true) {
        ts_arr = [ new Date(test_date) ];
        test_date.setDate(test_date.getDate() + date_adj);
        ts_arr[1] = new Date(test_date);
        ts_arr.sort(function(a, b) {
            return b < a;
        });
        //
        if (date >= ts_arr[0] && date <= ts_arr[1]) {
            break;
        }
    }
    // shifting end date to be saturday instead of sunday
    ts_arr[1].setDate(ts_arr[1].getDate() - 1);
    ts_arr = [ ts_arr[0].yyyymmdd(), ts_arr[1].yyyymmdd() ];
    return ts_arr;
}
