/*eslint-disable camelcase */
/*eslint-disable no-alert */
/*eslint-disable no-use-before-define */
/*eslint-disable global-strict */
////////////////////////////////////////////////////////////////////////////////
//////////////       This file holds the functions that are          ///////////
//////////////       associated only with the sales pages            ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
var AFW_HUB = AFW_HUB || {};
////////////////////////////////////////////////////////////////////////////////
///////////////////// Page Generation Functions ////////////////////////////////
//
// creates the main aspects of the sales page
function sales_page() {
    //fec
    var url_data = AFW_HUB.Navigation.parse_url();
    var nav_buttons = {
        sales_rep_maintenance: rep_maintenance,
        customer_maintenance: customer_maintenance,
        sales_rep_report: rep_report,
        customer_report: customer_report,
        returns_report: returns_report
    };
    //
    var nav_container = document.getElementById('nav-container');
    var button = document.createElementWithAttr('BUTTON', { class: 'big' });
    var i = 0;
    for (var btn in nav_buttons) {
        button = button.cloneNode();
        button.id = btn;
        button.textContent = btn.replace(/_/g, ' ').toTitleCase();
        button.addEventListener('click', nav_buttons[btn].bind(button, true));
        //
        if ((i + 1) % 5 === 0) {
            nav_container.appendChild(document.createElement('BR'));
        }
        nav_container.appendChild(button);
        i++;
    }
    //
    // checking perms for each button
    for (var id in nav_buttons) {
        AFW_HUB.Navigation.check_user_perms('sales.' + id);
    }
    //
    // checking url for sub_tab to navigate to
    if (url_data.hasOwnProperty('sub_tab')) {
        var sub_tab_func = nav_buttons[url_data.sub_tab];
        sub_tab_func.call(null, false);
    }
}
//
// this unhides the rep creation/maintenance buttons and resets the page
function rep_maintenance(reset) {
    var childNodes = document.getElementById('main-container').childNodes,
        fieldset = document.createElement('FIELDSET'),
        status_onclick = function() {
            AFW_HUB.FormUtil.toggle_checkbox_value(this, '.*', '^active$');
        },
        form_elements = [
            { elm: 'legend', textNode: 'Rep Selection Parameters' },
            //
            { elm: 'label', textNode: 'Show Inactive Reps:' },
            {
                elm: 'input',
                id: 'rep-status',
                name: 'rep_status',
                type: 'checkbox',
                value: '^active$',
                events: [
                    { event: 'click', function: status_onclick },
                    {
                        event: 'click',
                        function: create_rep_table.bind(null, 1, 'rep_id', 'ASC')
                    }
                ]
            },
            { elm: 'br' },
            {
                elm: 'button',
                textNode: 'Create New Sales Rep',
                type: 'button',
                events: [
                    {
                        event: 'click',
                        function: setup_sales_rep_form.bind(null, null, null, null)
                    }
                ]
            }
        ],
        i;
    //
    if (reset) {
        reset_sales_page(this);
        return;
    }
    //
    // clearing elements inside main-container div
    for (i = 0; i < childNodes.length; i++) {
        if (childNodes[i].nodeType === 1) {
            childNodes[i].removeAll();
        }
    }
    //
    // adding a show inactive button
    fieldset.id = 'rep-selection-parameters';
    fieldset.add_children(form_elements);
    document.getElementById('input-div').appendChild(fieldset);
    document.getElementById('tab-clicked').value = 'rep-maintenance';
    //
    AFW_HUB.Utilities.remove_class('hidden-elm', 'table-div');
    create_rep_table(1, 'rep_id', 'ASC');
}
//
// this unhides the customer creation/maintenance buttons and resets the page
function customer_maintenance(reset) {
    //
    var table_fun = create_customer_table.bind(null, 1, 'customer_id', 'ASC', null),
        fieldset = create_customer_table_refinement(table_fun);
    //
    if (reset) {
        reset_sales_page(this);
        return;
    }
    //
    // adding refinement fields
    document.getElementById('input-div').appendChild(fieldset);
    document.getElementById('tab-clicked').value = 'customer-maintenance';
    //
    AFW_HUB.Utilities.remove_class('hidden-elm', 'table-div');
    table_fun();
}
//
// this function sets up the page for a sales rep report
function rep_report(reset) {
    //
    var update_fun = function() {
            show_update_button('create-report', 'sales-rep-report', 'Show Changes');
        },
        input_args = { output_id: 'report-inputs', update_fun: update_fun },
        elements = [
            //
            { elm: 'br' },
            { elm: 'br' },
            //
            { elm: 'label', textNode: 'Sales Reps to Show:' },
            {
                elm: 'select',
                id: 'rep-status',
                name: 'rep-status',
                events: [ { event: 'change', function: update_fun } ]
            },
            //
            { elm: 'br' },
            //
            { elm: 'label', textNode: 'Preset Report Options:' },
            {
                elm: 'select',
                id: 'preset-report',
                name: 'preset-report',
                events: [
                    {
                        event: 'change',
                        function: function() {
                            show_update_button(
                                'create-report',
                                'sales-rep-report',
                                'Show Changes'
                            );
                            AFW_HUB.Utilities.remove_class(
                                'hidden-elm',
                                'sel-cols-div'
                            );
                            rep_report_data_columns(false, true, undefined);
                        }
                    }
                ]
            },
            //
            { elm: 'br' },
            { elm: 'br' },
            //
            { elm: 'label', textNode: 'Sorting Direction:' },
            {
                elm: 'select',
                id: 'sort-direction',
                name: 'sort-direction',
                events: [ { event: 'change', function: update_fun } ]
            }
        ],
        fieldset = document.createElement('FIELDSET'),
        legend = document.createElement('LEGEND'),
        dropbox_args;
    //
    if (reset) {
        reset_sales_page(this);
        return;
    }
    //
    // setting up report inputs
    fieldset.id = 'report-inputs';
    legend.textContent = 'Report Parameters';
    document.getElementById('tab-clicked').value = 'sales-rep-report';
    document.getElementById('input-div').appendChild(fieldset);
    AFW_HUB.Utilities.add_class('hidden-elm', 'table-div');
    //
    fieldset.appendChild(legend);
    create_time_range_inputs(input_args);
    fieldset.add_children(elements);
    //
    elements = [
        { elm: 'option', textNode: 'Active', value: 'active' },
        { elm: 'option', textNode: 'All', value: '.+' },
        { elm: 'option', textNode: 'Inactive', value: 'inactive' }
    ];
    document.getElementById('rep-status').add_children(elements);
    //
    elements = [
        { elm: 'option', textNode: 'Ascending', value: 'ASC' },
        { elm: 'option', textNode: 'Descending', value: 'DESC' }
    ];
    document.getElementById('sort-direction').add_children(elements);
    //
    // creating the required buttons and selection cols div
    elements = [
        { elm: 'br' },
        //
        {
            elm: 'button',
            id: 'show-data-sel-cols',
            textNode: 'Show Data Selection Columns',
            events: [
                {
                    event: 'click',
                    function: function() {
                        rep_report_data_columns(true, true, undefined);
                    }
                }
            ]
        },
        //
        {
            elm: 'button',
            id: 'create-report',
            textNode: 'Create Report',
            events: [
                {
                    event: 'click',
                    function: create_rep_report.bind(null, null, undefined)
                }
            ]
        },
        //
        {
            elm: 'button',
            id: 'print-rep-report',
            textNode: 'Print',
            events: [ { event: 'click', function: window.print.bind(window) } ]
        },
        //
        { elm: 'div', id: 'sel-cols-div', class: 'hidden-elm' }
    ];
    document.getElementById('input-div').add_children(elements);
    //
    elements = [
        { elm: 'br' },
        {
            elm: 'button',
            id: 'restore-defaults',
            class: 'hidden-elm',
            textNode: 'Restore Defaults',
            events: [
                {
                    event: 'click',
                    function: function() {
                        rep_report_data_columns(false, true);
                        show_update_button(
                            'create-report',
                            'sales-rep-report',
                            'Show Changes'
                        );
                        AFW_HUB.Utilities.add_class(
                            'hidden-elm',
                            'restore-defaults'
                        );
                    }
                }
            ]
        },
        { elm: 'br' }
    ];
    document.getElementById('sel-cols-div').add_children(elements);
    //
    // hiding rep table and button if only a rep is logged in
    if (window.sessionStorage.department === 'sales_rep') {
        create_rep_report(null, undefined, undefined);
    }
    //
    // populating dropboxes
    dropbox_args = {
        dropbox_id: 'preset-report',
        text_format: '%preset_name%',
        value_format: '%preset_index%',
        placeholder: 'None',
        placeholder_value: '0',
        database: 'meta_database',
        sql_args: {
            table: 'table_preset_sql_args',
            cols: [ 'internal_id', 'preset_name' ],
            where: [ [ 'html_table', 'LIKE', 'sales_rep_commission_report' ] ]
        }
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    //
    rep_report_data_columns(false, true);
}
//
// this function sets up the page for a customer report
function customer_report(reset) {
    //
    if (reset) {
        reset_sales_page(this);
        return;
    }
    document.getElementById('tab-clicked').value = 'customer-report';
    AFW_HUB.Utilities.add_class('hidden-elm', 'table-div');
    //
    alert('WIP');
}
//
// sets up the page for entry and viewing of returns
function returns_report(reset) {
    var returns_funcs = returns_form_funcs(),
        aux_div = returns_funcs.setup_aux_div(),
        table_div = document.getElementById('table-div'),
        msg = 'Select a value from the table below';
    //
    // initializing page
    if (reset) {
        reset_sales_page(this);
        return;
    }
    document.getElementById('tab-clicked').value = 'returns-report';
    document.getElementById('content-header').id = 'main-form-header';
    document.getElementById('content-div').style.clear = 'none';
    document.getElementById('content-div').id = 'main-form-div';
    AFW_HUB.Utilities.remove_class('hidden-elm', 'main-form-div');
    //
    // changing ID's of main container children to not conflict with aux div IDs
    table_div.id = 'main-table-div';
    table_div.style.clear = 'none';
    document.getElementById('table-header').id = 'main-table-header';
    //
    // adding aux div to DOM
    document.getElementById('main-container').insertBefore(aux_div, table_div);
    document.getElementById('auxillary-div-header').textContent = msg;
    //
    // adding action buttons
    generate_returns_buttons();
}
//
// clears elements on the sales returns reort page
function clear_sales_returns_page() {
    var ids = [
            'input-div',
            'main-table-header',
            'main-table-div',
            'main-form-header',
            'main-form-div'
        ],
        i;
    for (i = 0; i < ids.length; i++) {
        document.getElementById(ids[i]).removeAll();
    }
    AFW_HUB.Utilities.add_class('hidden-elm', 'auxillary-div');
    generate_returns_buttons();
}
//
// adds action buttons to the returns form page
function generate_returns_buttons() {
    //
    // adding buttons to determine user action
    var button = document.createElementWithAttr('BUTTON', { class: 'medium' }),
        div = document.getElementById('input-div'),
        returns_tbl_func = create_returns_table.bind(null, null, null, null),
        report_func = setup_returns_report.bind(null, null);
    //
    button.textContent = 'New Return';
    button.addEventListener('click', clear_sales_returns_page);
    button.addEventListener(
        'click',
        setup_returns_form.bind(null, null, null, null)
    );
    div.appendChild(button);
    //
    button = button.cloneNode();
    button.textContent = 'Update Return';
    button.addEventListener('click', clear_sales_returns_page);
    button.addEventListener(
        'click',
        create_returns_table_refinement.bind(null, returns_tbl_func)
    );
    button.addEventListener('click', returns_tbl_func);
    button.addEventListener(
        'click',
        AFW_HUB.Utilities.remove_class.bind(null, 'hidden-elm', 'main-table-div')
    );
    div.appendChild(button);
    //
    button = button.cloneNode();
    button.textContent = 'View Returns';
    button.addEventListener('click', clear_sales_returns_page);
    button.addEventListener(
        'click',
        create_returns_table_refinement.bind(null, report_func)
    );
    button.addEventListener(
        'click',
        create_returns_report_filters.bind(null, report_func)
    );
    button.addEventListener('click', setup_returns_report.bind(null, null));
    div.appendChild(button);
    //
    div.appendChild(document.createElement('BR'));
}
//
// this resets the page after main buttons are pressed
function reset_sales_page(button) {
    //
    location.href = 'content_page.php?tab=sales&sub_tab=' + button.id;
}
////////////////////////////////////////////////////////////////////////////////
//////////////////// Table Generation Functions ////////////////////////////////
//
// this function creates the sales rep table
function create_rep_table(page, sort_col, sort_dir) {
    //
    // initializating argument objects
    var sub_tab = document.getElementById('tab-clicked').value,
        table_args = {
            table_name: 'sales_rep_table',
            table_output_id: 'table-div',
            table_attr: { id: 'sales_rep_table' },
            row_id_prefix: 'rep-row-',
            data_sql_args: {
                cmd: 'SELECT',
                table: 'sales_rep_table',
                where: [],
                order_by: [ [ sort_col, sort_dir ] ]
            },
            head_row_args: {
                sortable: true,
                sort_col: sort_col,
                sort_dir: sort_dir,
                sort_onclick: [
                    {
                        func: create_rep_table,
                        args: [ 1, '%column_name%', '%sort_dir%' ]
                    }
                ]
            },
            page_nav_args: {
                curr_page: page,
                sort_col: sort_col,
                sort_dir: sort_dir,
                tot_pages_shown: 9,
                num_per_page: 15,
                page_nav_div_attr: { id: 'rep-table-page-nav' },
                id_prefix: 'rep',
                link_onclick: [
                    {
                        func: create_rep_table,
                        args: [ '%page%', sort_col, sort_dir ]
                    }
                ]
            }
        };
    //
    // setting sub tab specific arguments
    if (sub_tab === 'rep-maintenance') {
        table_args.row_onclick = [
            { func: setup_sales_rep_form, args: [ '%row_id%', null ] }
        ];
    } else if (sub_tab === 'sales-rep-report') {
        table_args.row_onclick = [
            { func: create_rep_report, args: [ '%rep_id%', undefined ] }
        ];
    }
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
//
// this function generates the refinement fieldset for the customer table
function create_customer_table_refinement(table_func) {
    //
    // adding refinement fields
    var toggle_checkbox_value = AFW_HUB.FormUtil.toggle_checkbox_value,
        fieldset = document.createElement('FIELDSET'),
        form_elements = [
            { elm: 'legend', textNode: 'Refine Customer Table By:' },
            //
            { elm: 'label', style: { width: '12em' }, textNode: 'Customer ID:' },
            {
                elm: 'input',
                id: 'refine-by-cid',
                type: 'text',
                name: 'customer_id',
                events: [ { event: 'keyup', function: table_func } ]
            },
            { elm: 'span', textNode: '    ' },
            { elm: 'label', class: 'label-medium', textNode: 'Exact Match:' },
            {
                elm: 'input',
                id: 'refine-by-cid-match-type',
                type: 'checkbox',
                value: 'REGEXP',
                events: [
                    {
                        event: 'click',
                        function: function() {
                            toggle_checkbox_value(this, 'LIKE', 'REGEXP');
                        }
                    }
                ]
            },
            { elm: 'br' },
            //
            { elm: 'label', style: { width: '12em' }, textNode: 'Sales Rep ID:' },
            {
                elm: 'input',
                id: 'refine-by-rid',
                type: 'text',
                name: 'sales_customer_table.rep_id',
                events: [ { event: 'keyup', function: table_func } ]
            },
            { elm: 'span', textNode: '    ' },
            { elm: 'label', class: 'label-medium', textNode: 'Exact Match:' },
            {
                elm: 'input',
                id: 'refine-by-rid-match-type',
                type: 'checkbox',
                value: 'REGEXP',
                events: [
                    {
                        event: 'click',
                        function: function() {
                            toggle_checkbox_value(this, 'LIKE', 'REGEXP');
                        }
                    }
                ]
            },
            { elm: 'br' },
            //
            { elm: 'label', style: { width: '12em' }, textNode: 'Customer Name:' },
            {
                elm: 'input',
                id: 'refine-by-name',
                type: 'text',
                name: 'customer_name',
                events: [ { event: 'keyup', function: table_func } ]
            },
            { elm: 'span', textNode: '    ' },
            { elm: 'label', class: 'label-medium', textNode: 'Exact Match:' },
            {
                elm: 'input',
                id: 'refine-by-name-match-type',
                type: 'checkbox',
                value: 'REGEXP',
                events: [
                    {
                        event: 'click',
                        function: function() {
                            toggle_checkbox_value(this, 'LIKE', 'REGEXP');
                        }
                    }
                ]
            },
            { elm: 'br' },
            //
            { elm: 'label', style: { width: '12em' }, textNode: 'Customer Status:' },
            {
                elm: 'input',
                id: 'refine-by-status',
                type: 'text',
                name: 'customer_status',
                events: [ { event: 'keyup', function: table_func } ]
            },
            { elm: 'span', textNode: '    ' },
            { elm: 'label', class: 'label-medium', textNode: 'Exact Match:' },
            {
                elm: 'input',
                id: 'refine-by-status-match-type',
                type: 'checkbox',
                value: 'REGEXP',
                events: [
                    {
                        event: 'click',
                        function: function() {
                            toggle_checkbox_value(this, 'LIKE', 'REGEXP');
                        }
                    }
                ]
            },
            { elm: 'br' },
            //
            {
                elm: 'button',
                id: 'show-customer-table',
                textNode: 'Hide Customer Table',
                type: 'button',
                events: [
                    {
                        event: 'click',
                        function: function() {
                            toggle_view_element_button(
                                this.id,
                                'table-div',
                                'Hide Customer Table',
                                'Show Customer Table'
                            );
                        }
                    }
                ]
            },
            {
                elm: 'button',
                id: 'new-customer',
                textNode: 'Create New Customer',
                type: 'button',
                events: [
                    {
                        event: 'click',
                        function: setup_customer_form.bind(null, null, null, null)
                    }
                ]
            }
        ];
    fieldset.id = 'customer-selection-parameters';
    fieldset.add_children(form_elements);
    //
    return fieldset;
}
//
// this function creates the customer table
function create_customer_table(page, sort_col, sort_dir, table_args) {
    //
    // initializating argument objects
    var container = document.getElementById('customer-selection-parameters'),
        customer_table_args = {
            table_name: 'sales_customer_table',
            table_output_id: 'table-div',
            table_attr: { id: 'sales_customer_table' },
            row_id_prefix: 'customer-row-',
            row_onclick: [
                {
                    func: setup_customer_form,
                    args: [ '%customer_internal_id%', '%row_id%', null ]
                }
            ],
            data_sql_args: { where: [], order_by: [ [ sort_col, sort_dir ] ] },
            head_row_args: {
                sortable: true,
                sort_col: sort_col,
                sort_dir: sort_dir,
                sort_onclick: [
                    {
                        func: create_customer_table,
                        args: [ 1, '%column_name%', '%sort_dir%', table_args ]
                    }
                ]
            },
            page_nav_args: {
                curr_page: page,
                sort_col: sort_col,
                sort_dir: sort_dir,
                tot_pages_shown: 9,
                num_per_page: 15,
                page_nav_div_attr: { id: 'customer-table-page-nav' },
                id_prefix: 'customer',
                link_onclick: [
                    {
                        func: create_customer_table,
                        args: [ '%page%', sort_col, sort_dir, table_args ]
                    }
                ]
            }
        };
    AFW_HUB.TableUtil.get_table_inputs(customer_table_args.data_sql_args, container);
    //
    if (table_args) {
        Object.merge(customer_table_args, table_args);
    }
    AFW_HUB.TableUtil.create_standard_table(customer_table_args);
}
//
// sets up the refinement fields for the returns table
function create_returns_table_refinement(table_func) {
    //
    // creating inputs to view and modify an existing record
    var fieldset = document.createElement('FIELDSET'),
        div = document.createElementWithAttr('DIV', {
            id: 'returns-selection-params'
        }),
        input_args = { output_id: 'time-range-div', update_fun: table_func },
        opts = [
            {
                elm: 'option',
                value: 'rep_response_req|buyer_response_req',
                textNode: 'All Open'
            },
            {
                elm: 'option',
                value: 'rep_response_req',
                textNode: 'Rep Response Req.'
            },
            {
                elm: 'option',
                value: 'buyer_response_req',
                textNode: 'Buyer Response Req.'
            },
            { elm: 'option', value: 'completed', textNode: 'Completed' },
            { elm: 'option', value: 'deleted', textNode: 'Deleted' },
            { elm: 'option', value: '.*', textNode: 'All Returns' }
        ],
        fieldset_elms = [
            { elm: 'legend', textNode: 'Refine Returns Table By:' },
            { elm: 'div', id: 'returns-selection-params' },
            { elm: 'div', id: 'time-range-div' }
        ],
        ref_elms = [
            { elm: 'label', textNode: 'Return Status: ' },
            {
                elm: 'select',
                id: 'ref-entry-status',
                name: 'entry_status',
                children: opts,
                events: [ { event: 'change', function: table_func } ]
            },
            { elm: 'br' }
        ];
    //
    document.getElementById('input-div').safeAppendChild(fieldset);
    fieldset.add_children(fieldset_elms);
    div.add_children(ref_elms);
    fieldset.safeAppendChild(div);
    //
    create_time_range_inputs(input_args);
}
//
// this fucntion creates the table of returns
function create_returns_table(page, sort_col, sort_dir) {
    //
    page = page || 1;
    sort_col = sort_col || 'date';
    sort_dir = sort_dir || 'DESC';
    //
    var msg = 'Click on a return to modify it:',
        time_div = document.getElementById('time-range-div'),
        container = document.getElementById('returns-selection-params'),
        ts_obj = to_and_from_timestamps(time_div),
        table_args = {
            table_name: 'sales_returns_table',
            table_output_id: 'main-table-div',
            table_attr: { id: 'sales_items_returns_table' },
            row_id_prefix: 'item-return-row-',
            row_onclick: [
                {
                    func: setup_returns_form,
                    args: [ '%entry_id%', '%row_id%', null ]
                }
            ],
            data_sql_args: {
                where: [ [ 'date', 'BETWEEN', [ ts_obj.from_ts, ts_obj.to_ts ] ] ],
                order_by: [ [ sort_col, sort_dir ] ]
            },
            head_row_args: {
                sortable: true,
                sort_col: sort_col,
                sort_dir: sort_dir,
                sort_onclick: [
                    {
                        func: create_returns_table,
                        args: [ 1, '%column_name%', '%sort_dir%' ]
                    }
                ]
            },
            page_nav_args: {
                curr_page: page,
                sort_col: sort_col,
                sort_dir: sort_dir,
                tot_pages_shown: 9,
                num_per_page: 15,
                page_nav_div_attr: { id: 'returns-table-page-nav' },
                id_prefix: 'return',
                link_onclick: [
                    {
                        func: create_returns_table,
                        args: [ '%page%', sort_col, sort_dir ]
                    }
                ]
            }
        };
    AFW_HUB.TableUtil.get_table_inputs(table_args.data_sql_args, container);
    //
    // updating elements
    AFW_HUB.Utilities.add_class('hidden-elm', 'auxillary-div');
    AFW_HUB.Utilities.remove_class('hidden-elm', 'table-div');
    document.getElementById('main-table-header').textContent = msg;
    //
    AFW_HUB.TableUtil.create_standard_table(table_args);
}
////////////////////////////////////////////////////////////////////////////////
///////////////////////// Secondary Functions //////////////////////////////////
//
// sets the page to create or modify a rep
function setup_sales_rep_form(row_id, form) {
    var content_div = document.getElementById('content-div'),
        sub_button = document.getElementById('submit-rep-form'),
        header = 'Creating a New Sales Rep',
        validate = function(act) {
            rep_form_validation(form, act, undefined);
        },
        row_data, dropbox_args, form_args, button,
        post_data, callback;
    row_data = row_id ? document.getElementById(row_id).dataset : {};
    //
    // loading form
    if (!form) {
        callback = setup_sales_rep_form.bind(null, row_id);
        AFW_HUB.FormUtil.load_html_form('sales_rep_form', content_div, callback);
        return;
    }
    //
    // defining args
    form_args = {
        form_id: form.id,
        meta_data_only: 1,
        sql_args: {
            where: [
                ['sales_rep_table.internal_id', 'LIKE', row_data.internal_id]
            ]
        },
        data_preprocessor: function(args, response) {
            var event_args = {meta_array: response.meta_data};
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
        }
    };
    dropbox_args = {
        dropbox_data: null,
        dropbox_id: 'dbuser-internal-id',
        text_format: '%dbuser_first_name% %dbuser_last_name%',
        value_format: '%dbuser_internal_id%',
        placeholder: 'No Login',
        placeholder_disabled: false,
        placeholder_selected: true,
        placeholder_value: 'NULL',
        add_callback: AFW_HUB.FormUtil.populate_form.bind(null, form_args, undefined)
    };
    //
    sub_button.addEventListener('click', validate.bind(null, 'creation'));
    if (row_id) {
        form_args.meta_data_only = undefined;
        header = 'Modifying User: {rep_first_name} {rep_last_name}';
        header = header.format(row_data);
        document.getElementById('content-header').textContent = header;
        document.getElementById('rep-id').readOnly = true;
        //
        // adding new buttons
        button = sub_button.cloneNode();
        button.textContent = 'Modify Rep';
        button.addEventListener('click', validate.bind(null, 'modification'));
        form.replaceChild(button, sub_button);
        //
        button = button.cloneNode();
        button.id = 'delete-rep';
        button.textContent = 'Delete Rep';
        button.addEventListener('click', validate.bind(null, 'deletion'));
        form.safeAppendChild(button);
        //
        button = button.cloneNode();
        button.id = 'reinstate-rep';
        button.textContent = 'Reinstate Rep';
        button.addEventListener('click', validate.bind(null, 'reinstatement'));
        form.safeAppendChild(button);
        //
        // callback function to determine what buttons to show based on dbuser status
        form_args.add_callback =  function() {
            //
            if (document.getElementById('rep-status').value === 'active') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'reinstate-rep');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'delete-rep');
            }
        };
    }
    //
    post_data = {
        method: 'fetch_auth_data',
        data_where: JSON.stringify([ [ 'department', 'REGEXP', 'sales_rep' ] ])
    };
    callback = function(xmlhttp) {
        var response = AFW_HUB.Ajax.process_response(xmlhttp);
        response.dropbox_data = response.data;
        AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args, response);
    };
    AFW_HUB.Ajax.send_aync_request(post_data, callback);
}
//
// sets the page to create or modify customers
function setup_customer_form(customer_id, row_id, form) {
    var content_div = document.getElementById('content-div'),
        sub_button = document.getElementById('submit-customer-form'),
        header = 'Creating a New Customer',
        validate = function(act) {
            customer_form_validation(form, act, undefined);
        },
        row_data, form_args, dropbox_args, button, callback;
    row_data = row_id ? document.getElementById(row_id).dataset : {};
    //
    // loading form
    if (!form) {
        callback = setup_customer_form.bind(null, customer_id, row_id);
        AFW_HUB.FormUtil.load_html_form(
            'sales_customer_form',
            content_div,
            callback
        );
        return;
    }
    //
    // form and dropbox population args
    form_args = {
        form_id: form.id,
        trigger_events: false,
        meta_data_only: 1,
        sql_args: {
            cmd: 'SELECT',
            table: 'sales_customer_table',
            where: [
                [ 'customer_internal_id', 'LIKE', customer_id ]
            ]
        },
        data_preprocessor: function(args, response) {
            var event_args = {meta_array: response.meta_data};
            AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
        }
    };
    dropbox_args = {
        dropbox_id: 'rep-id',
        placeholder: 'Rep ID - Rep Name',
        text_format: '%rep_id% - %rep_first_name%',
        value_format: '%rep_id%',
        sql_args: {
            cmd: 'SELECT',
            table: 'sales_rep_table',
            cols: [ 'rep_id', 'rep_first_name' ]
        },
        add_callback: AFW_HUB.FormUtil.populate_form.bind(null, form_args, undefined)
    };
    //
    // adding buttons if modification
    sub_button.addEventListener('click', validate.bind(null, 'creation'));
    if (customer_id && row_id) {
        header = 'Modfiying Customer: {customer_name}'.format(row_data);
        //
        // adding new buttons
        button = sub_button.cloneNode();
        button.textContent = 'Modify Customer';
        button.addEventListener('click', validate.bind(null, 'modification'));
        form.replaceChild(button, sub_button);
        //
        button = button.cloneNode();
        button.id = 'delete-customer';
        button.textContent = 'Delete Customer';
        button.addEventListener('click', validate.bind(null, 'deletion'));
        form.safeAppendChild(button);
        //
        button = button.cloneNode();
        button.id = 'reinstate-customer';
        button.textContent = 'Reinstate Customer';
        button.addEventListener('click', validate.bind(null, 'reinstatement'));
        form.safeAppendChild(button);
        //
        // determines what buttons to show based on customer status
        form_args.meta_data_only = undefined;
        form_args.add_callback =  function() {
            //
            if (document.getElementById('customer-status').value === 'active') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'reinstate-customer');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'delete-customer');
            }
        };
    }
    //
    document.getElementById('content-header').textContent = header;
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
}
//
// adds event handlers to the returns form
function add_returns_form_events(form, response) {
    var func_obj = returns_form_funcs(),
        event_args = {meta_array: response.meta_data};
    //
    // adding event handlers to the form
    AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
    //
    var input = document.getElementById('customer-id');
    input.addEventListener('focus', func_obj.show_customer_table);
    input.addEventListener('keyup', func_obj.check_cust_id);
    //
    input = document.getElementById('picker-emp-id');
    input.addEventListener(
        'focus',
        func_obj.show_employee_table.bind(null, 'warehouse_shipping', 'picker')
    );
    input.addEventListener(
        'keyup',
        func_obj.check_emp_id.bind(null, 'picker', null)
    );
    //
    input = document.getElementById('driver-emp-id');
    input.addEventListener(
        'focus',
        func_obj.show_employee_table.bind(null, 'transportation', 'driver')
    );
    input.addEventListener(
        'keyup',
        func_obj.check_emp_id.bind(null, 'driver', null)
    );
    //
    input = document.getElementById('item-number');
    input.addEventListener('focus', func_obj.show_products_table);
    input.addEventListener('keyup', func_obj.check_item_num.bind(null, null));
}
//
// sets the page to create or update a return
function setup_returns_form(entry_id, row_id, form) {
    var form_div = document.getElementById('main-form-div'),
        sub_button = document.getElementById('submit-returns-form'),
        header = 'Creating New Item Return',
        validate = function(act) {
            returns_form_validation(form, act);
        },
        form_args, dropbox_args, button, callback;
    //
    // loading form
    if (!form) {
        //
        // fetching form
        callback = setup_returns_form.bind(null, entry_id, row_id);
        form_div.removeAll();
        AFW_HUB.FormUtil.load_html_form('sales_returns_form', form_div, callback);
        return;
    }
    //
    // setting up form population args
    form_args = {
        form_id: form.id,
        trigger_events: false,
        meta_data_only: 1,
        sql_args: {
            cmd: 'SELECT',
            table: 'sales_item_returns',
            where: [
                [ 'entry_id', 'LIKE', entry_id ]
            ]
        },
        data_preprocessor: function(args, response) {
            add_returns_form_events(form, response);
        }
    };
    dropbox_args = {
        database: 'misc_database',
        sql_args: {
            cmd: 'SELECT',
            table: 'error_codes',
            cols: [ 'reason', 'description' ]
        },
        dropbox_id: 'credit-reason',
        text_format: '%description%',
        value_format: '%description%',
        placeholder: 'Select Reason',
        placeholder_disabled: true,
        add_opts_val: [ 'other' ],
        add_opts_text: [ 'OTHER' ],
        add_callback: AFW_HUB.FormUtil.populate_form.bind(null, form_args, undefined)
    };
    //
    if (entry_id && row_id) {
        header = 'Modifying Item Return #: ' + entry_id;
        //
        // adding new buttons
        button = sub_button.cloneNode();
        button.textContent = 'Update Return';
        button.addEventListener('click', validate.bind(null, 'modification'));
        form.replaceChild(button, sub_button);
        //
        button = button.cloneNode();
        button.id = 'delete-return';
        button.textContent = 'Delete Return';
        button.addEventListener('click', validate.bind(null, 'deletion'));
        form.safeAppendChild(button);
        //
        button = button.cloneNode();
        button.id = 'restore-return';
        button.textContent = 'Restore Return';
        button.addEventListener('click', validate.bind(null, 'restoration'));
        form.safeAppendChild(button);
        //
        form_args.meta_data_only = undefined;
        form_args.add_callback = function(args) {
            // setting values based on entry status
            if (args.data_arr.entry_status.match(/rep_response_req/)) {
                document.getElementById('edit-rep-response').checked = true;
                document.getElementById('rep-response').readOnly = false;
            }
            if (args.data_arr.entry_status.match(/buyer_response_req/)) {
                document.getElementById('edit-buyer-response').checked = true;
                document.getElementById('buyer-response').readOnly = false;
            }
            //
            if (args.data_arr.entry_status !== 'deleted') {
                AFW_HUB.Utilities.add_class('hidden-elm', 'restore-return');
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', 'delete-return');
            }
        };
    } else {
        // setting some initial values
        document.getElementById('date').value = Date.current_ts().split(' ')[0];
        document.getElementById('edit-rep-response').checked = true;
        document.getElementById('rep-response').readOnly = false;
        button = document.getElementById('submit-returns-form');
        button.addEventListener('click', validate.bind(null, 'creation'));
    }
    //
    document.getElementById('main-form-header').textContent = header;
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
}
//
//
////////////////////////////////////////////////////////////////////////////////
///////////////// Returns Reporting Functions //////////////////////////////////
//
// generates filtering options for the returns report
function create_returns_report_filters(report_func) {
    var ref_div = document.getElementById('returns-selection-params'),
        dirs = [
            { elm: 'option', value: 'ASC', textNode: 'Ascending' },
            { elm: 'option', value: 'DESC', textNode: 'Descending' }
        ],
        type_elms = [
            { elm: 'label', textNode: 'Detailed Report: ' },
            {
                elm: 'input',
                type: 'radio',
                name: 'report-type',
                value: 'detailed',
                checked: true
            },
            {
                elm: 'label',
                textNode: 'Summary Report: ',
                style: { marginLeft: '0.5em' }
            },
            { elm: 'input', type: 'radio', name: 'report-type', value: 'summary' }
        ],
        ref_elms = [
            { elm: 'label', textNode: 'Sales Rep' },
            {
                elm: 'select',
                id: 'ref-sales-rep-id',
                name: 'sales_item_returns.rep_id',
                events: [ { event: 'change', function: report_func } ]
            },
            { elm: 'br' },
            { elm: 'label', textNode: 'Customer:' },
            {
                elm: 'select',
                id: 'ref-customer-id',
                name: 'sales_item_returns.customer_id',
                events: [ { event: 'change', function: report_func } ]
            },
            { elm: 'br' },
            { elm: 'label', textNode: 'Return Reason:' },
            {
                elm: 'select',
                id: 'ref-credit-reason',
                name: 'credit_reason',
                events: [ { event: 'change', function: report_func } ]
            },
            { elm: 'br' },
            { elm: 'label', textNode: 'Sort By:' },
            {
                elm: 'select',
                id: 'sort-col',
                name: 'sort-col',
                style: { marginRight: '1.5em' },
                events: [ { event: 'change', function: report_func } ]
            },
            { elm: 'label', textNode: 'Direction:', style: { display: 'inline' } },
            {
                elm: 'select',
                id: 'sort-dir',
                name: 'sort-dir',
                children: dirs,
                events: [ { event: 'change', function: report_func } ]
            }
        ],
        print_elms = [
            {
                elm: 'button',
                textNode: 'Print',
                events: [ { event: 'click', function: window.print.bind(window) } ]
            }
        ],
        dropbox_args,
        i,
        node;
    //
    // adding elements
    for (i = 0; i < type_elms.length; i++) {
        node = type_elms[i].elm;
        delete type_elms[i].elm;
        node = document.createElementWithAttr(node, type_elms[i]);
        node.textContent = type_elms[i].textNode || '';
        if (node.value === 'summary') {
            node.addEventListener(
                'click',
                add_summary_report_params.bind(null, report_func)
            );
        }
        if (node.nodeName.toLowerCase() === 'input') {
            node.addEventListener('click', report_func);
        }
        ref_div.parentNode.insertBefore(node, ref_div);
    }
    ref_div.add_children(ref_elms);
    document.getElementById('input-div').add_children(print_elms);
    //
    // populating the dropboxes
    dropbox_args = {
        sql_args: {
            cmd: 'SELECT',
            table: 'sales_item_returns',
            cols: [
                'DISTINCT(sales_item_returns.rep_id) AS `rep_id`',
                'sales_rep_table.rep_first_name'
            ],
            inner_join: [
                [
                    'sales_rep_table',
                    'sales_item_returns.rep_id',
                    'sales_rep_table.rep_id'
                ]
            ]
        },
        dropbox_id: 'ref-sales-rep-id',
        text_format: '%rep_first_name%',
        value_format: '^%rep_id%$',
        placeholder: 'All',
        placeholder_disabled: false,
        placeholder_value: '.*'
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    //
    dropbox_args = {
        sql_args: {
            cmd: 'SELECT',
            table: 'sales_item_returns',
            cols: [
                'DISTINCT(sales_item_returns.customer_id) AS `customer_id`',
                'sales_customer_table.customer_name'
            ],
            inner_join: [
                [
                    'sales_customer_table',
                    'sales_item_returns.customer_id',
                    'sales_customer_table.customer_id'
                ]
            ]
        },
        dropbox_id: 'ref-customer-id',
        text_format: '%customer_name%',
        value_format: '^%customer_id%$',
        placeholder: 'All',
        placeholder_disabled: false,
        placeholder_value: '.*'
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    //
    dropbox_args = {
        sql_args: {
            cmd: 'SELECT',
            table: 'sales_item_returns',
            cols: [ 'DISTINCT(`credit_reason`) AS `credit_reason`' ],
            inner_join: [
                [
                    'afwg_hub_misc.error_codes',
                    'sales_item_returns.credit_reason',
                    'error_codes.description'
                ]
            ]
        },
        dropbox_id: 'ref-credit-reason',
        text_format: '%credit_reason%',
        value_format: '^%credit_reason%$',
        placeholder: 'All',
        placeholder_disabled: false,
        placeholder_value: '.*'
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
    //
    dropbox_args = {
        database: 'meta_database',
        sql_args: {
            cmd: 'SELECT',
            table: 'column_meta_data',
            inner_join: [
                ['table_meta_data',
                 'column_meta_data.internal_id',
                 'table_meta_data.column_internal_id']
            ],
            cols: [ 'column_name', 'table_meta_data.display_name' ],
            where: [
                [ 'in_table', 'REGEXP', 'sales_item_returns' ],
                [ 'use_on_pages', 'REGEXP', 'returns_report' ]
            ],
            order_by: [ [ 'column_name', 'ASC' ], [ 'display_order', 'ASC' ] ]
        },
        dropbox_id: 'sort-col',
        text_format: '%display_name%',
        value_format: 'sales_item_returns.%column_name%',
        add_callback: function() {
            document.getElementById('sort-col').value = 'sales_item_returns.date';
        }
    };
    AFW_HUB.FormUtil.populate_dropbox_options(dropbox_args);
}
//
// handles some page modifications for summary reports
function add_summary_report_params(report_func) {
    var ref_div = document.getElementById('returns-selection-params'),
        select = ref_div.parentNode.querySelector('#summary-grouping'),
        label = ref_div.parentNode.querySelector('#grouping-label'),
        option,
        attr;
    //
    // creating a dropbox if it doesn't already exist
    if (select) {
        return;
    }
    label = document.createElementWithAttr('LABEL', { id: 'grouping-label' });
    label.textContent = 'Group Report By: ';
    select = document.createElementWithAttr('SELECT', { id: 'summary-grouping' });
    select.addEventListener('change', report_func);
    //
    // creating options
    attr = {
        value: 'rep_id',
        'data-join_col': 'rep_id',
        'data-name_col': 'sales_rep_name',
        'data-name_alias': 'sales_rep_table.rep_first_name',
        'data-name_table': 'sales_rep_table'
    };
    option = document.createElementWithAttr('OPTION', attr);
    option.textContent = 'Sales Rep';
    select.appendChild(option);
    //
    attr = {
        value: 'customer_id',
        'data-join_col': 'customer_id',
        'data-name_col': 'customer_name',
        'data-name_alias': 'sales_customer_table.customer_name',
        'data-name_table': 'sales_customer_table'
    };
    option = document.createElementWithAttr('OPTION', attr);
    option.textContent = 'Customer';
    select.appendChild(option);
    //
    attr = {
        value: 'dsr_route_number',
        'data-name_col': 'dsr_route_number',
        'data-name_alias': 'sales_item_returns.dsr_route_number'
    };
    option = document.createElementWithAttr('OPTION', attr);
    option.textContent = 'Route';
    select.appendChild(option);
    //
    attr = {
        value: 'driver_emp_id',
        'data-join_col': 'emp_id',
        'data-name_col': 'employee_name',
        'data-name_alias': 'CONCAT(emp_first_name, \'-\', emp_last_name) AS `employee_name`',
        'data-name_table': 'employee_info'
    };
    option = document.createElementWithAttr('OPTION', attr);
    option.textContent = 'Driver';
    select.appendChild(option);
    //
    attr = {
        value: 'picker_emp_id',
        'data-join_col': 'emp_id',
        'data-name_col': 'employee_name',
        'data-name_alias': 'CONCAT(emp_first_name, \'-\', emp_last_name) AS `employee_name`',
        'data-name_table': 'employee_info'
    };
    option = document.createElementWithAttr('OPTION', attr);
    option.textContent = 'Picker';
    select.appendChild(option);
    //
    select.value = 'rep_id';
    ref_div.parentNode.insertBefore(document.createElement('BR'), ref_div);
    ref_div.parentNode.insertBefore(label, ref_div);
    ref_div.parentNode.insertBefore(select, ref_div);
}
//
// generates the returns report using a specialized table format
function setup_returns_report() {
    //
    var sql_args,
        sql_arr = [],
        key_arr = [],
        callback,
        container = document.getElementById('main-form-div'),
        time_div = document.getElementById('time-range-div'),
        ts_obj = to_and_from_timestamps(time_div),
        params_container = document.getElementById('returns-selection-params'),
        report_type = params_container.parentNode.querySelector(
            '[name=report-type]:checked'
        ),
        group_select = document.getElementById('summary-grouping'),
        option_data;
    //
    // manually resetting page elements
    AFW_HUB.Utilities.add_class('hidden-elm', 'auxillary-div');
    AFW_HUB.Utilities.add_class('hidden-elm', 'main-table-div');
    document.getElementById('main-form-header').textContent = '';
    container.removeAll();
    //
    // disabling sorting becuase it is incompatible with a sumamry query
    params_container.querySelector('#sort-col').disabled = report_type.value ===
        'summary';
    params_container.querySelector('#sort-dir').disabled = report_type.value ===
        'summary';
    //
    // processing response
    if (report_type.value === 'summary') {
        // need to facilitate groupings by rep, customer, picker and driver
        // I'll have to use conditional statements for now to set the cols and inner join
        // later on when SQL aliases are functional I may be able to auto-gen by
        // setting report_type according, i.e. sales_item_returns_report-summary-rep_id,
        // sales_item_returns_report-summary-customer_id, etc. auto table generation would
        // need to setup inner_joins automagically
        //
        // unhiding grouping selection elements
        AFW_HUB.Utilities.remove_class('hidden-elm', 'grouping-label');
        AFW_HUB.Utilities.remove_class('hidden-elm', 'summary-grouping');
        //
        // spoofing meta_data for simplicity right now
        option_data = group_select.selectedOptions[0].dataset;
        //
        callback = create_summary_returns_report.bind(null);
        sql_args = {
            cmd: 'SELECT',
            cols: [
                'sales_item_returns.{}'.format(group_select.value),
                '{name_alias}'.format(option_data),
                'COUNT(*) AS `returns_per`'
            ],
            table: 'sales_item_returns',
            order_by: [
                [ 'sales_item_returns.{}'.format(group_select.value), 'ASC' ],
                [ 'returns_per', 'DESC' ]
            ],
            group_by: 'sales_item_returns.{}'.format(group_select.value)
        };
        //
        if (option_data.join_col !== undefined) {
            sql_args.inner_join = [
                [
                    option_data.name_table,
                    'sales_item_returns.{}'.format(group_select.value),
                    '{}.{}'.format(option_data.name_table, option_data.join_col)
                ]
            ];
        }
    } else {
        //
        // hiding grouping selection elements
        AFW_HUB.Utilities.add_class('hidden-elm', 'grouping-label');
        AFW_HUB.Utilities.add_class('hidden-elm', 'summary-grouping');
        callback = create_detailed_returns_report.bind(null);
        sql_args = {
            cmd: 'SELECT',
            cols: [
                'afwg_hub.sales_item_returns.*',
                'COALESCE(afwg_prod_data.products.descript, \'\') AS `descript`',
                'CONCAT(afwg_hub.sales_rep_table.rep_first_name, " ", afwg_hub.sales_rep_table.rep_last_name) AS `rep_name`',
                'afwg_hub.sales_customer_table.customer_name',
                'driver_data.driver_name',
                'picker_data.picker_name'
            ],
            table: 'afwg_hub.sales_item_returns',
            inner_join: [
                [
                    'sales_rep_table',
                    'sales_item_returns.rep_id',
                    'sales_rep_table.rep_id'
                ],
                [
                    'sales_customer_table',
                    'sales_item_returns.customer_id',
                    'sales_customer_table.customer_id'
                ]
            ],
            left_join: [
                [
                    'afwg_prod_data.products',
                    'afwg_hub.sales_item_returns.item_number',
                    'afwg_prod_data.products.`item number`'
                ],
                [
                    '(SELECT CONCAT(emp_first_name, " ", emp_last_name) AS `driver_name`, `emp_id` FROM employee_info) AS `driver_data`',
                    'sales_item_returns.driver_emp_id',
                    'driver_data.emp_id'
                ],
                [
                    '(SELECT CONCAT(emp_first_name, " ", emp_last_name) AS `picker_name`, `emp_id` FROM employee_info) AS `picker_data`',
                    'sales_item_returns.picker_emp_id',
                    'picker_data.emp_id'
                ]
            ]
        };
    }
    //
    // generating SQL statemets
    sql_args.where = [ [ 'date', 'BETWEEN', [ ts_obj.from_ts, ts_obj.to_ts ] ] ];
    AFW_HUB.TableUtil.get_table_inputs(sql_args, params_container);
    sql_arr.push([sql_args]);
    key_arr.push(['table_data']);
    //
    sql_args = {
        cmd: 'SELECT',
        table: 'column_meta_data',
        inner_join: [
            ['table_meta_data',
             'column_meta_data.internal_id',
             'table_meta_data.column_internal_id']
        ],
        where: [
            [ 'use_on_pages', 'REGEXP', 'returns_report' ],
            [
                'html_table',
                'LIKE',
                'sales_item_returns_report_' + report_type.value
            ]
        ],
        order_by: [ [ 'display_order', 'ASC' ] ]
    };
    // handling special case for summary report
    if (report_type.value === 'summary') {
        sql_args.where = [
            [
                'column_name',
                'REGEXP',
                '(^{}$)|(^{}$)|(^{}$)'.format(
                    group_select.value,
                    option_data.name_col,
                    'returns_per'
                )
            ],
            ['html_table', 'LIKE', 'sales_item_returns_report_' + report_type.value],
            ['use_on_pages', 'REGEXP', 'returns_report']
        ];
    }
    //
    sql_arr.push([sql_args]);
    key_arr.push(['meta_data']);
    //
    AFW_HUB.Ajax.fetch_db(sql_arr, key_arr, [null, 'meta_database'], callback);
}
//
// creates a detailed returns report
function create_detailed_returns_report(response) {
    //
    var table_format = '' +
        '<table id="return-{entry_id}" style="float: left; width: 100%; page-break-inside: avoid;">' +
        '<tr style="font-weight: bold;">' +
        '<td style="width: 10rem;">Sales Rep:</td>' +
        '<td colspan="2">{rep_name}</td>' +
        '<td colspan="3">Driver Name: {driver_name}</td>' +
        '<td colspan="3">Picker Name: {picker_name}</td>' +
        '</tr>' +
        '<tr id="headers" style="font-weight: bold; font-size:90%"></tr>' +
        '<tr id="values"></tr>' +
        '<tr><td>Entry Status:</td><td colspan="10">{entry_status}</td></tr>' +
        '<tr><td>Entry Comments:</td><td colspan="10" style="white-space: normal;">{return_comments}</td></tr>' +
        '<tr><td>Buyer Response:</td><td colspan="10" style="white-space: normal;">{buyer_response}</td></tr>' +
        '<tr><td>Rep Response:</td><td colspan="10" style="white-space: normal;">{rep_response}</td></tr>' +
        '</table>',
        meta_data = response.meta_data,
        returns_data = response.table_data,
        cell = document.createElementWithAttr('TD', {}),
        div = document.createElementWithAttr('DIV', {}),
        br = document.createElementWithAttr('BR', { style: { clear: 'both' } }),
        container = document.getElementById('main-form-div'),
        i,
        j,
        table,
        head_row,
        val_row;
    //
    container.removeAll();
    //
    // generating returns tables
    for (i = 0; i < returns_data.length; i++) {
        div = div.cloneNode();
        div.innerHTML = table_format.format(returns_data[i]);
        table = div.querySelector('table');
        head_row = div.querySelector('#headers');
        val_row = div.querySelector('#values');
        delete head_row.id;
        delete val_row.id;
        //
        for (j = 0; j < meta_data.length; j++) {
            //
            Object.merge(table.dataset, returns_data[i]);
            //
            cell = cell.cloneNode();
            cell.textContent = meta_data[j].display_name;
            head_row.appendChild(cell);
            //
            cell = cell.cloneNode();
            cell.textContent = returns_data[i][meta_data[j].column_name];
            val_row.appendChild(cell);
        }
        //
        container.addNodes([ div, br.cloneNode(), br.cloneNode() ]);
    }
}
//
// creates a summary returns report
function create_summary_returns_report(response) {
    var table_args;
    //
    // creating table argument object
    table_args = {
        table_output_id: 'main-form-div',
        table_attr: { id: 'returns-report-summary' },
        head_row_args: { sortable: false },
        page_nav_args: {},
        no_page_nav: true
    };
    //
    AFW_HUB.TableUtil.create_standard_table(table_args, response);
}
//
//
///////////////////////////////////////////////////////////////////////////////
///////////////// Commission Reporting Functions //////////////////////////////
//
// creates the data columns table for the report page
function rep_report_data_columns(toggle, reset) {
    var container = document.getElementById('sel-cols-div'),
        button = document.getElementById('show-data-sel-cols'),
        preset = document.getElementById('preset-report').value || '0',
        hide_str = 'Hide Column Selection',
        show_str = 'Show Column Selection',
        table_args,
        add_event_handlers,
        callback;
    //
    // toggling table viewability
    if (!toggle) {
        button.value = 'update-button';
    }
    toggle_view_element_button(button, container, hide_str, show_str);
    //
    // resetting table based on preset data
    if (!reset) {
        return;
    }
    //
    add_event_handlers = function(elm) {
        var event = elm.primaryEventString(), func;
        //
        if (!event) {
            return;
        }
        func = function() {
            AFW_HUB.Utilities.remove_class('hidden-elm', 'restore-defaults');
            show_update_button('create-report', 'report-table', 'Show Changes');
        };
        elm.addEventListener(event, func);
    };
    table_args = {
        table_name: 'sales_rep_commission_report',
        table_attr: { id: 'report-column-selection' },
        table_output_id: container.id,
        create_sort_row: true,
        add_event_handlers: add_event_handlers,
        add_callback: function() {}
    };
    //
    callback = function(preset_data) {
        table_args.selected_cols = preset_data.cols || [ '*' ];
        AFW_HUB.TableUtil.create_column_selection_table(table_args);
    };
    AFW_HUB.TableUtil.fetch_preset_data(preset, callback);
}
//
// this creates the rep report
function create_rep_report(rep_id, preset_sql_args, response) {
    var fieldset = document.getElementById('report-inputs'),
        update_button = document.getElementById('create-report'),
        preset = document.getElementById('preset-report').value || '0',
        input_params = AFW_HUB.FormUtil.get_all_form_values(fieldset),
        ts_obj = to_and_from_timestamps(fieldset),
        display_table = document.getElementById('report-column-selection'),
        display_params = AFW_HUB.TableUtil.process_column_selection_table(
            display_table
        ),
        end_date,
        ly_date,
        report_args,
        sql_args,
        callback;
    //
    // fetching preset data
    if (preset_sql_args === undefined) {
        callback = create_rep_report.bind(null, rep_id);
        AFW_HUB.TableUtil.fetch_preset_data(preset, callback);
        return;
    }
    //
    // finding the most recent ar entry
    if (response === undefined) {
        sql_args = {
            cmd: 'SELECT',
            table: 'sales_rep_data',
            cols: [ 'date', 'creation_timestamp' ],
            where: [ [ 'date', '<=', ts_obj.to_ts ] ],
            order_by: [ [ 'date', 'DESC' ] ],
            limit: [ 0, 1 ]
        };
        if (rep_id !== null) {
            sql_args.where.push([ 'sales_rep_data.rep_id', 'LIKE', rep_id ]);
        }
        //
        callback = create_rep_report.bind(null, rep_id, preset_sql_args);
        AFW_HUB.Ajax.fetch_db([ sql_args ], [ 0 ], null, callback);
        return;
    }
    //
    // resetting event handler on update button
    update_button.parentNode.safeAppendChild(update_button.cloneNode(true));
    update_button = document.getElementById('create-report');
    update_button.addEventListener(
        'click',
        create_rep_report.bind(null, rep_id, undefined, undefined)
    );
    //
    // setting up the report args
    end_date = new Date(response[0][0].date + ' 00:00:00');
    ly_date = new Date(
        end_date.getFullYear(),
        end_date.getMonth(),
        end_date.getDate() - 7 * 52
    );
    sql_args = Object.merge({ where: [] }, preset_sql_args);
    sql_args.sub_query = {
        '%lytd_sales%': {
            cmd: 'SELECT',
            table: 'sales_data',
            cols: [
                'sales_rep_table.rep_id',
                'SUM(sales_data.amount) AS `lytd_total`'
            ],
            inner_join: [
                [
                    'sales_customer_table',
                    'sales_data.customer_id',
                    'sales_customer_table.customer_id'
                ],
                [
                    'sales_rep_table',
                    'sales_customer_table.rep_id',
                    'sales_rep_table.rep_id'
                ]
            ],
            where: [
                [
                    'sales_data.date',
                    'BETWEEN',
                    [
                        AFW_HUB.get_constant('LY_FIRST_BUSINESS_DAY').join('-'),
                        ly_date.yyyymmdd()
                    ]
                ]
            ],
            group_by: 'sales_rep_table.rep_id'
        },
        '%ytd_sales%': {
            cmd: 'SELECT',
            table: 'sales_data',
            cols: [
                'sales_rep_table.rep_id',
                'SUM(sales_data.amount) AS `ytd_total`'
            ],
            inner_join: [
                [
                    'sales_customer_table',
                    'sales_data.customer_id',
                    'sales_customer_table.customer_id'
                ],
                [
                    'sales_rep_table',
                    'sales_customer_table.rep_id',
                    'sales_rep_table.rep_id'
                ]
            ],
            where: [
                [
                    'sales_data.date',
                    'BETWEEN',
                    [
                        AFW_HUB.get_constant('FIRST_BUSINESS_DAY').join('-'),
                        end_date.yyyymmdd()
                    ]
                ]
            ],
            group_by: 'sales_rep_table.rep_id'
        }
    };
    sql_args.where.push(
        [
            'sales_rep_data.creation_timestamp',
            'LIKE',
            response[0][0].creation_timestamp
        ],
        [
            'sales_rep_table.rep_status',
            'REGEXP',
            '^' + input_params['rep-status'] + '$'
        ]
    );
    if (rep_id !== null) {
        sql_args.where.push([ 'sales_rep_data.rep_id', 'LIKE', rep_id ]);
    }
    sql_args.order_by = [
        [
            display_params.sort_col[0] || 'sales_rep_name',
            input_params['sort-direction']
        ]
    ];
    //
    report_args = {
        table_output_id: 'content-div',
        table_name: 'sales_rep_commission_report',
        table_attr: {
            id: 'report-table',
            class: 'print-container',
            style: { border: 'solid 1px' }
        },
        ts_obj: ts_obj,
        create_data_rows: true,
        create_section_headers: false,
        create_totals_row: true,
        display_cols: display_params.display_cols,
        data_sql_args: sql_args,
        head_row_args: { sortable: false },
        data_row_attr: {},
        data_cell_attr: {},
        totals_row_attr: {},
        totals_cell_attr: {}
    };
    AFW_HUB.TableUtil.create_standard_report(report_args);
}
