/* global AFW_HUB */
////////////////////////////////////////////////////////////////////////////////
//////////////       This file holds the functions that are          ///////////
//////////////       associated only with the admin level pages      ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
//
//
// sets up the basic entry page for dbusers
function setup_dbuser_page() {
    //
    // creating/getting elements
    var table_fieldset = document.getElementById('table-fieldset');
    var br = document.createElement('BR');
    var label = document.createElement('LABEL');
    label.textContent = 'Show Inactive Users:  ';
    var checkbox = document.createElementWithAttr('INPUT', {
        id: 'show-inactive',
        type: 'checkbox',
        value: '^active$'
    });
    checkbox.addEventListener(
        'click',
        AFW_HUB.FormUtil.toggle_checkbox_value.bind(null, checkbox, '.', '^active$')
    );
    checkbox.addEventListener(
        'click',
        dbuser_table.bind(null, '1', 'dbuser_last_name', 'ASC')
    );
    var button = document.createElementWithAttr('BUTTON', { id: 'create-new-user' });
    button.textContent = 'Create New User';
    button.addEventListener('click', mod_dbuser_info.bind(null, null, null, null));
    //
    // setting up page
    document.getElementById('page-header').textContent = 'Database User Setup';
    document.getElementById('table-legend').textContent = 'Select User';
    table_fieldset.addNodes([ br, label, checkbox ]);
    table_fieldset.parentNode.insertBefore(button, table_fieldset.nextSibling);
    table_fieldset.parentNode.insertBefore(
        br.cloneNode(),
        table_fieldset.nextSibling
    );
    //
    dbuser_table(1);
}
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////// Table Generation Functions //////////////////////////
//
// creates the database user table for the mod_dbuser page
function dbuser_table(page) {
    //
    // creating argument objects
    var table_args, post_data, callback;
    //
    // creating argument object
    table_args = {
        table_output_id: 'table-fieldset',
        table_attr: { id: 'dbuser-table' },
        row_id_prefix: 'dbuser-row-',
        row_onclick: [
            {
                func: mod_dbuser_info,
                args: [ '%dbuser_internal_id%', '%row_id%', null ]
            }
        ],
        head_row_args: { sortable: false },
        page_nav_args: {
            curr_page: page,
            tot_pages_shown: 9,
            num_per_page: 10,
            page_nav_div_attr: { id: 'dbuser-table-page-nav' },
            id_prefix: 'dbuser',
            link_onclick: [ { func: dbuser_table, args: [ '%page%' ] } ]
        },
        regen_function: dbuser_table.bind(null, page)
    };
    //
    callback = function(xmlhttp) {
        var response = JSON.parse(xmlhttp.responseText);
        //
        response.table_data = response.data;
        AFW_HUB.TableUtil.create_standard_table(table_args, response);
    };
    //
    post_data = { method: 'fetch_auth_data' };
    AFW_HUB.Ajax.send_aync_request(post_data, callback);
}
//
///////////////////////////////////////////////////////////////////////////////
//////////////////////////////// Form Generation //////////////////////////////
//
// populates the dbuser form based on the id
function populate_dbuser_form(args, xmlhttp) {
    var post_data, response, callback = populate_dbuser_form.bind(null, args);
    //
    if (xmlhttp === undefined) {
        //
        post_data = {
            method: 'fetch_auth_data',
            data_where: [ [ 'dbuser_internal_id', 'LIKE', args.dbuser_id ] ]
        };
        post_data.data_where = JSON.stringify(post_data.data_where);
        AFW_HUB.Ajax.send_aync_request(post_data, callback);
        return;
    }
    //
    response = JSON.parse(xmlhttp.responseText);
    response.form_data = response.data;
    AFW_HUB.FormUtil.populate_form(args, response);
}
//
// handles creation of the setup dbuser form
function mod_dbuser_info(dbuser_id, row_id, form) {
    var form_div, args, callback, row_data = {}, msg;
    //
    // loading form
    if (!form) {
        form_div = document.getElementById('form-div');
        callback = mod_dbuser_info.bind(null, dbuser_id, row_id);
        AFW_HUB.FormUtil.load_html_form('dbuser_form', form_div, callback);
        return;
    }
    //
    // setting up args to populate form containers
    args = {
        methods: [
            {
                function: AFW_HUB.FormUtil.populate_dropbox_options,
                args: {
                    sql_args: {
                        cmd: 'SELECT',
                        table: 'departments',
                        cols: [
                            'department',
                            'department_name',
                            'default_permissions'
                        ],
                        where: [
                            [ 'department_type', 'REGEXP', '(^|%)dbusers(%|$)' ]
                        ]
                    },
                    database: 'misc_database',
                    dropbox_id: 'department',
                    text_format: '%department_name%',
                    value_format: '%department%',
                    placeholder: 'Select Department',
                    placeholder_selected: true,
                    add_opts_text: [ 'Other' ],
                    add_opts_val: [ 'other' ]
                }
            },
            {
                function: populate_dbuser_form,
                args: {
                    form: form,
                    dbuser_id: dbuser_id,
                    skip_fields_str: 'password',
                    trigger_events: true,
                    meta_data_only: 1,
                    data_preprocessor: function() {
                        add_dbuser_form_event_handlers(form);
                        gen_user_perm_inputs();
                    }
                }
            }
        ]
    };
    //
    document.getElementById('form-header').textContent = 'Creating New User';
    callback = function() {};
    if (dbuser_id && row_id) {
        args.methods[1].args.meta_data_only = undefined;
        //
        // setting form header
        row_data = document.getElementById(row_id).dataset;
        msg = 'Modfiying User: {dbuser_first_name} {dbuser_last_name}';
        document.getElementById('form-header').textContent = msg.format(row_data);
        //
        callback = function() {
            var mod_button,
                res_button,
                del_button,
                perms_string,
                dept_dropbox,
                selected_opt;
            //
            // adding buttons
            mod_button = document.createElementWithAttr('BUTTON', {
                id: 'submit-dbuser-form',
                type: 'button'
            });
            mod_button.addTextNode('Modify User');
            mod_button.addEventListener(
                'click',
                validate_dbuser_form.bind(null, form, 'modification', undefined)
            );
            del_button = document.createElementWithAttr('BUTTON', {
                id: 'del-button',
                type: 'button'
            });
            del_button.addTextNode('Delete User');
            del_button.addEventListener(
                'click',
                validate_dbuser_form.bind(null, form, 'deletion', undefined)
            );
            res_button = document.createElementWithAttr('BUTTON', {
                id: 'res-button',
                type: 'button'
            });
            res_button.addTextNode('Reinstate User');
            res_button.addEventListener(
                'click',
                validate_dbuser_form.bind(null, form, 'reinstatement', undefined)
            );
            //
            form.addNodes([ mod_button, del_button, res_button ]);
            //
            // manipulating buttons
            if (document.getElementById('dbuser-status').value !== 'active') {
                AFW_HUB.Utilities.add_class('hidden-elm', del_button);
            } else {
                AFW_HUB.Utilities.add_class('hidden-elm', res_button);
            }
            //
            select_perms_checkboxes();
            perms_string = row_data.permissions;
            dept_dropbox = document.getElementById('department');
            selected_opt = dept_dropbox.options[dept_dropbox.selectedIndex];
            if (selected_opt.dataset.default_permissions !== perms_string) {
                //
                // selecting the customize perms boxes
                document.getElementById('use-default-perms').checked = false;
                document.getElementById('use-default-perms').dispatchPrimaryEvent();
                document.getElementById('edit-permissions').checked = true;
                document.getElementById('edit-permissions').dispatchPrimaryEvent();
            }
        };
    }
    //
    AFW_HUB.FormUtil.populate_multiple_containers(args, callback);
}
//
/////////////////////// Database User Form Events /////////////////////////////
//
// adds event handlers to the initial dbuser form
function add_dbuser_form_event_handlers(form) {
    //
    var event_args = {
            add_default_events: function(elem) {
                var event = elem.primaryEventString();
                if (elem.nodeName.toLowerCase() === 'button') {
                    return;
                }
                if (!event) {
                    return;
                }
            //
                elem.addEventListener(
                event,
                AFW_HUB.Utilities.remove_class.bind(null, 'boxshadow-red', elem)
            );
            }
        },
        elm;
    AFW_HUB.FormUtil.add_standard_event_listeners(form, event_args);
    document
        .getElementById('submit-dbuser-form')
        .addEventListener(
            'click',
            validate_dbuser_form.bind(null, form, 'creation', undefined)
        );
    //
    elm = document.getElementById('department');
    elm.addEventListener('change', set_default_perms.bind(null));
    //
    elm = document.getElementById('use-default-perms');
    elm.addEventListener('click', toggle_perm_editing.bind(null));
    elm.addEventListener('click', set_default_perms.bind(null));
    //
    // adding perms customization event handlers
    elm = document.getElementById('edit-permissions');
    elm.addEventListener('click', function() {
        var perm_div = document.getElementById('user-perm-inputs'),
            checkboxes = perm_div.querySelectorAll('INPUT[type=checkbox]'),
            disable = document.getElementById('edit-permissions').checked,
            cont,
            i;
        checkboxes = Array.slice(checkboxes, 0, checkboxes.length - 1);
        //
        if (!disable) {
            cont = confirm(
                'Warning: This will undo ALL customizations ' +
                    'that differ from the checked boxes'
            );
            if (!cont) {
                document.getElementById('edit-permissions').checked = true;
                return;
            }
        }
        //
        for (i = 0; i < checkboxes.length; i++) {
            checkboxes[i].disabled = disable;
        }
    });
    elm.addEventListener('click', build_perms_string);
    //
    elm = document.getElementById('permissions');
    elm.addEventListener('keyup', select_perms_checkboxes);
}
//
// adds the user permissions inputs to the dbuser form
function gen_user_perm_inputs() {
    //
    var click_event = document.createEvent('HTMLEvents'),
        perms_div = document.getElementById('user-perm-inputs'),
        label = document.createElement('LABEL'),
        args,
        page;
    click_event.initEvent('click', true, false);
    //
    label.style.fontSize = '1.1em';
    label.style.fontWeight = 'bold';
    label.textContent = 'User Permissions: ';
    perms_div.addNodes([ label ]);
    page = 'employee_data_entry';
    args = {
        table_id: 'employee-data-entry-perms',
        all_value: page + '.*',
        tabs: [
            { value: page + '.occupancy_data_entry', text: 'Occupancy' },
            { value: page + '.drivers_data_entry', text: 'Drivers' },
            { value: page + '.freight_logistics', text: 'Freight Logistics' },
            { value: page + '.receiving_data_entry', text: 'Receiving' },
            { value: page + '.shipping_data_entry', text: 'Shipping' }
        ],
        label: 'Employee Data Entry: ',
        output_element: perms_div
    };
    make_perms_table(args);
    //
    args = {
        table_id: 'employee-data',
        all_value: 'view_employee_data.*,employee_data_maintenance.*,employee_report.*',
        tabs: [
            { value: 'view_employee_data.*', text: 'View' },
            { value: 'employee_data_maintenance.*', text: 'Maintenance' },
            { value: 'employee_report.*', text: 'Reporting' }
        ],
        label: 'Employee Data: ',
        output_element: perms_div
    };
    make_perms_table(args);
    //
    page = 'marketing';
    args = {
        table_id: 'marketing',
        all_value: page + '.*',
        tabs: [
            { value: page + '.vendor_broker_setup', text: 'Vendor Setup' },
            { value: page + '.contact_information', text: 'Contact Information' },
            { value: page + '.marketing_commitment', text: 'Marketing Commitment' },
            { value: page + '.services_performed', text: 'Services Performed' },
            'new_row',
            { value: page + '.food_show', text: 'Food Show' },
            { value: page + '.growth', text: 'Growth' },
            { value: page + '.payments', text: 'Payments' },
            { value: page + '.overview', text: 'Dashboard' },
            { value: page + '.marketing-archive', text: 'Archive' }
        ],
        label: 'Marketing: ',
        output_element: perms_div
    };
    make_perms_table(args);
    //
    page = 'meat_shop';
    args = {
        table_id: 'meat-shop-perms',
        all_value: page + '.*',
        tabs: [
            { value: page + '.item_maintenance', text: 'Item Maintenance' },
            { value: page + '.update_stock', text: 'Update Stock' },
            {
                value: page + '.modify_stock_change_record',
                text: 'Records Maintenance'
            },
            { value: page + '.inventory_report', text: 'Inventory Report' },
            { value: page + '.stock_change_report', text: 'Records Report' }
        ],
        label: 'Meat Shop: ',
        output_element: perms_div
    };
    make_perms_table(args);
    //
    page = 'sales';
    args = {
        table_id: 'sales-perms',
        all_value: page + '.*',
        tabs: [
            {
                value: page + '.sales_rep_maintenance',
                text: 'Sales Rep Maintenance'
            },
            { value: page + '.customer_maintenance', text: 'Customer Maintenance' },
            { value: page + '.sales_rep_report', text: 'Sales Rep Report' },
            { value: page + '.customer_report', text: 'Customer Report' },
            { value: page + '.returns_report', text: 'Returns Report' }
        ],
        label: 'Sales: ',
        output_element: perms_div
    };
    make_perms_table(args);
    //
    args = {
        table_id: 'admin-perms',
        all_value: 'setup_employee.*,setup_dbuser.*,table_maintenance.*',
        tabs: [
            { value: 'setup_employee.*', text: 'Employee Setup' },
            { value: 'setup_dbuser.*', text: 'Database User Setup' },
            { value: 'table_maintenance.*', text: 'Table Maintenance' }
        ],
        label: 'Administration: ',
        output_element: perms_div
    };
    make_perms_table(args);
}
//
// handles cration of a perms table
function make_perms_table(args) {
    //
    var table = document.createElementWithAttr('TABLE', {
        id: args.table_id,
        class: 'plain',
        style: { margin: 'initial' }
    });
    var head_tr = document.createElementWithAttr('TR', {
        class: 'no-hover no-active'
    });
    var chkb_tr = document.createElementWithAttr('TR', {
        class: 'no-hover no-active'
    });
    var td = document.createElementWithAttr('TD', {
        style: { border: '1px solid rgb(0,0,0)' }
    });
    var checkbox = document.createElementWithAttr('INPUT', { type: 'checkbox' });
    var br = document.createElement('BR');
    var label = document.createElementWithAttr('LABEL', {
        style: { 'padding-right': '0.3em' }
    });
    //
    var label_td = td.cloneNode();
    var i;
    label_td.rowSpan = 2;
    label_td.textContent = args.label;
    label.style.display = 'inline';
    label_td.style.fontWeight = 'bold';
    label_td.style.padding = '0em 1em';
    label_td.appendChild(br.cloneNode());
    head_tr.appendChild(label_td);
    //
    td = document.createElement('TD');
    td.style.textAlign = 'center';
    //
    label = label.cloneNode();
    label.textContent = 'All: ';
    checkbox = checkbox.cloneNode();
    checkbox.id = args.table_id + '-all';
    checkbox.value = args.all_value;
    label_td.addNodes([ label, checkbox ]);
    label_td.addTextNode('   ');
    label = label.cloneNode();
    label.textContent = 'None: ';
    checkbox = checkbox.cloneNode();
    checkbox.id = args.table_id + '-none';
    checkbox.value = '';
    label_td.addNodes([ label, checkbox ]);
    //
    checkbox.style.margin = 'auto';
    checkbox.style.display = 'inline-block';
    var checkboxes = [];
    for (i = 0; i < args.tabs.length; i++) {
        //
        if (args.tabs[i] === 'new_row') {
            table.addNodes([ head_tr, chkb_tr ]);
            head_tr = head_tr.cloneNode();
            chkb_tr = chkb_tr.cloneNode();
            label_td.rowSpan += 2;
            continue;
        }
        //
        td = td.cloneNode();
        td.textContent = args.tabs[i].text;
        head_tr.appendChild(td);
        td = td.cloneNode();
        checkbox = checkbox.cloneNode();
        checkbox.id = args.tabs[i].value;
        checkbox.value = args.tabs[i].value;
        checkbox.addEventListener('click', build_perms_string);
        checkboxes.push(checkbox);
        td.appendChild(checkbox);
        chkb_tr.appendChild(td);
    }
    table.addNodes([ head_tr, chkb_tr ]);
    args.output_element.addNodes([ table, br ]);
    //
    // adding event listeners to the all and none checkboxes
    checkbox = document.getElementById(args.table_id + '-all');
    checkbox.addEventListener('click', function() {
        for (i = 0; i < checkboxes.length; i++) {
            checkboxes[i].disabled = !checkboxes[i].disabled;
        }
        build_perms_string();
    });
    checkbox = document.getElementById(args.table_id + '-none');
    checkbox.addEventListener('click', function() {
        for (i = 0; i < checkboxes.length; i++) {
            checkboxes[i].disabled = !checkboxes[i].disabled;
            checkboxes[i].checked = false;
        }
        document.getElementById(args.table_id + '-all').checked = false;
        build_perms_string();
    });
}
//
// toggles editability of the permissions textarea
function toggle_perm_editing() {
    //
    AFW_HUB.Utilities.show_hide('user-perm-inputs');
    //
    var custom_checkbox = document.getElementById('edit-permissions');
    if (custom_checkbox.checked) {
        var event = document.createEvent('HTMLEvents');
        event.initEvent('click', true, false);
        custom_checkbox.dispatchEvent(event);
        custom_checkbox.checked = false;
    }
}
//
// sets the default perms for a user
function set_default_perms() {
    //
    var use_default_perms = document.getElementById('use-default-perms').checked;
    var perms_textarea = document.getElementById('permissions');
    var dept_dropbox = document.getElementById('department');
    var selected_opt = dept_dropbox.options[dept_dropbox.selectedIndex];
    var perms_string = selected_opt.dataset.default_permissions;
    //
    if (!use_default_perms) {
        return;
    }
    if (!perms_string) {
        perms_string = '';
    }
    //
    perms_textarea.value = perms_string;
    select_perms_checkboxes();
}
//
// builds perms string from selected checkboxes
function build_perms_string() {
    //
    var perm_div = document.getElementById('user-perm-inputs');
    var checkboxes = perm_div.querySelectorAll('INPUT:checked:not(:disabled)');
    var perms = [];
    //
    if (document.getElementById('edit-permissions').checked) {
        return;
    }
    //
    for (var i = 0; i < checkboxes.length; i++) {
        if (checkboxes[i].value === '') {
            continue;
        }
        perms.push(checkboxes[i].value);
    }
    var perms_string = perms.join(',');
    document.getElementById('permissions').value = perms_string;
}
//
// selects checkboxes from a perms string
function select_perms_checkboxes() {
    var perms_string = document.getElementById('permissions').value;
    var perms_arr = perms_string.split(',');
    var perm_div = document.getElementById('user-perm-inputs');
    var checkboxes = perm_div.querySelectorAll('INPUT[type=checkbox]');
    //
    // building pattern
    for (var i = 0; i < perms_arr.length; i++) {
        perms_arr[i] = '(^' + perms_arr[i] + '[^,]?$)';
    }
    var perms_pat = perms_arr.join('|');
    perms_pat = new RegExp(perms_pat);
    //
    // selecting checkboxes matching the perms pattern
    var checked = false;
    for (i = 0; i < checkboxes.length; i++) {
        if (checkboxes[i].value === '') {
            continue;
        }
        checked = perms_pat.test(checkboxes[i].value);
        //
        checkboxes[i].checked = checked;
    }
}
//
///////////////////////////////////////////////////////////////////////////////
//////////////////////////////// Form Submission //////////////////////////////
//
// validates the dbuser form
function validate_dbuser_form(form, act, xmlhttp) {
    var username_elm = document.getElementById('username'),
        dbuser_id_elm = document.getElementById('dbuser-internal-id'),
        errors,
        post_data,
        response,
        callback,
        args;
    //
    // fetching data
    if (xmlhttp === undefined) {
        //
        post_data = {
            method: 'fetch_auth_data',
            data_where: JSON.stringify([
                [ 'username', 'LIKE', username_elm.value ]
            ])
        };
        callback = validate_dbuser_form.bind(null, form, act);
        AFW_HUB.Ajax.send_aync_request(post_data, callback);
        return;
    }
    //
    // processing xmlhttp for response data
    response = AFW_HUB.Ajax.process_response(xmlhttp);
    //
    // checking if username is unique
    args = [ username_elm, dbuser_id_elm, null, null, null, [ response.data ] ];
    AFW_HUB.FormUtil.check_unique.apply(null, args);
    //
    if (act !== 'creation') {
        document.getElementById('password').dataset.skip_auto_validate = '1';
    }
    errors = AFW_HUB.FormUtil.auto_validate(form);
    if (errors) {
        return;
    }
    //
    submit_dbuser_form(form, act);
}
//
// submits the dbuser form
function submit_dbuser_form(form, act) {
    //
    var form_values = AFW_HUB.FormUtil.get_all_form_values(form),
        cont,
        post_data,
        callback;
    //
    // finalizing form values
    if (form_values.password === '') {
        delete form_values.password;
    }
    //
    if (act === 'creation') {
        form_values.dbuser_status = 'active';
        delete form_values.dbuser_internal_id;
    } else if (act === 'deletion') {
        form_values.dbuser_status = 'inactive';
    } else if (act === 'reinstatement') {
        form_values.dbuser_status = 'active';
    }
    cont = confirm('Confirm ' + act + ' of database user.');
    if (!cont) {
        return;
    }
    //
    // sending async request to update auth table
    post_data = {
        method: 'update_auth_table',
        form_values: JSON.stringify(form_values)
    };
    callback = reset_dbuser_form.bind(null, act);
    AFW_HUB.Ajax.send_aync_request(post_data, callback);
}
//
// resets the page after form submission
function reset_dbuser_form(act) {
    //
    alert('Successful ' + act + ' of Database User.');
    //
    var nav = document.getElementById('dbuser-table-page-nav');
    var curr_page = nav.dataset.currPage;
    dbuser_table(curr_page);
    //
    document.getElementById('dbuser-table').dispatchPrimaryEvent();
    document.getElementById('form-header').textContent = '';
    document.getElementById('form-div').removeAll();
}
