/*eslint-disable camelcase */
/*eslint-disable no-alert */
/*eslint-disable global-strict */
////////////////////////////////////////////////////////////////////////////////
///////////   This file holds general purpose utility functons that  ///////////
///////////   aid in automatic generation of tables and reports      ///////////
////////////////////////////////////////////////////////////////////////////////
'use strict';
var AFW_HUB = AFW_HUB || {};
//
//
AFW_HUB.TableUtil = function() {
    //
    ///////////////// General report/table associated functions ///////////////
    //
    // handles general fetching of table data
    function fetch_table_data(html_table, post_data, callback, xmlhttp) {
        var url_data = AFW_HUB.Navigation.parse_url(),
            page = url_data.sub_tab || url_data.tab,
            response,
            key;
        post_data = post_data || {};
        //
        if (xmlhttp === undefined) {
            //
            // JSON encoding any intial args passed in
            for (key in post_data) {
                if (post_data[key] === undefined) {
                    delete post_data[key];
                } else if (typeof post_data[key] === 'object') {
                    post_data[key] = JSON.stringify(post_data[key]);
                }
            }
            //
            // setting additional API requirements
            post_data.method = 'fetch_html_table_data';
            post_data.html_table = html_table;
            post_data.page = post_data.page || page;
            //
            callback = fetch_table_data.bind(null, html_table, post_data, callback);
            AFW_HUB.Ajax.send_aync_request(post_data, callback);
            return;
        }
        response = AFW_HUB.Ajax.process_response(xmlhttp);
        callback(response);
    }
    //
    // processes preset data for an html table
    function fetch_preset_data(preset_index, callback, response) {
        var sql_args;
        //
        // querying database
        if (response === undefined) {
            //
            // setting preset sql args
            sql_args = {
                cmd: 'SELECT',
                cols: [ 'sql_args_json' ],
                table: 'table_preset_sql_args',
                where: [ [ 'internal_id', 'LIKE', preset_index ] ]
            };
            callback = fetch_preset_data.bind(null, preset_index, callback);
            AFW_HUB.Ajax.fetch_db(
                [ sql_args ],
                [ 'preset_data' ],
                'meta_database',
                callback
            );
            return;
        }
        //
        // processing response
        sql_args = response.preset_data[0].sql_args_json;
        sql_args = JSON.parse(sql_args);
        callback(sql_args);
    }
    //
    // this function takes inputs from a fieldset or form and adds them to the
    // where parameter of the sql args using a REGEXP
    function get_table_inputs(sql_args, input_container) {
        var all_elms = input_container.allSubNodes(),
            params = AFW_HUB.FormUtil.get_all_form_values(input_container),
            match_types = {},
            col,
            i;
        //
        sql_args.where = sql_args.where || [];
        sql_args.order_by = sql_args.order_by || [];
        //
        // setting up match type object, defaults to REGEXP
        for (i = 0; i < all_elms.length; i++) {
            if (!all_elms[i].name) {
                continue;
            }
            if (!params.hasOwnProperty(all_elms[i].name)) {
                continue;
            }
            //
            match_types[all_elms[i].name] = all_elms[i].dataset.match_type ||
                'REGEXP';
        }
        //
        // checking for sorting terms
        if (params['sort-col'] && params['sort-dir']) {
            sql_args.order_by.push([ params['sort-col'], params['sort-dir'] ]);
        }
        delete params['sort-col'];
        delete params['sort-dir'];
        //
        // adding where blocks to sql args
        for (col in params) {
            if (params[col] === '') {
                continue;
            }
            //
            if (match_types[col] === 'REGEXP') {
                try {
                    RegExp(params[col]);
                    sql_args.where.push([ col, 'REGEXP', params[col] ]);
                } catch (err) {
                    console.log('Invalid RegExp pattern: ' + params[col]);
                }
            } else {
                sql_args.where.push([ col, match_types[col], params[col] ]);
            }
        }
    }
    //
    /////////////////////// Table generation functions ////////////////////////
    //
    // creates page link line
    function create_page_links(args) {
        var default_args = AFW_HUB.get_constant('PAGE_NAV_DEFAULTS'),
            build_runtime_function = AFW_HUB.Utilities.build_runtime_function,
            num_pages = Math.ceil(args.data_length / args.num_per_page),
            num_left = Math.floor((args.tot_pages_shown - 1) / 2),
            page_arr = [ 1 ],
            page_nav,
            arg,
            func_obj,
            runtime_func,
            i,
            j;
        //
        // processing argument object
        args = Object.merge(default_args, args);
        for (arg in args) {
            if (typeof args[arg] !== 'string') {
                continue;
            }
            if (args[arg].match(/^-?\d+$/)) {
                args[arg] = Number(args[arg]);
            }
        }
        //
        // creating page nav div
        page_nav = document.createElementWithAttr('DIV', args.page_nav_div_attr);
        page_nav.addTextNode('Pages: ');
        //
        // determining page number range to display
        page_arr[0] = args.curr_page - num_left;
        if (num_pages - page_arr[0] <= args.tot_pages_shown) {
            page_arr[0] = num_pages - args.tot_pages_shown + 1;
        }
        if (num_pages <= args.tot_pages_shown) {
            page_arr[0] = 1;
        }
        if (page_arr[0] <= 0) {
            page_arr[0] = 1;
        }
        //
        page_arr[0] = { textNode: '[' + page_arr[0] + ']', page: page_arr[0] };
        for (i = 1; i < args.tot_pages_shown; i++) {
            if (page_arr[i - 1].page >= num_pages) {
                break;
            }
            page_arr[i] = page_arr[i - 1].page + 1;
            page_arr[i] = { textNode: '[' + page_arr[i] + ']', page: page_arr[i] };
        }
        //
        // setting left arrow
        if (args.curr_page !== 1) {
            page_arr.unshift({ textNode: '❮', page: args.curr_page - 1 });
        }
        //
        // setting right arrow
        if (args.curr_page < num_pages) {
            page_arr.push({ textNode: '❯', page: args.curr_page + 1 });
        }
        //
        // builing page link element objects
        for (i = 0; i < page_arr.length; i++) {
            page_arr[i] = {
                elm: 'A',
                id: args.id_prefix + '-page-nav-' + page_arr[i].page,
                class: args.link_attr.class || '',
                page: page_arr[i].page,
                textNode: page_arr[i].textNode,
                events: []
            };
            page_arr[i] = Object.merge(Object.clone(args.link_attr), page_arr[i]);
            if (page_arr[i].page === args.curr_page) {
                page_arr[i].class += ' page-nav-link_curr';
            }
            //
            // adding onmouseover events
            for (j = 0; j < args.link_onmouse.length; j++) {
                func_obj = args.link_onmouse[j];
                runtime_func = build_runtime_function(func_obj, page_arr[i]);
                page_arr[i].events.push({
                    event: 'mouseover',
                    function: runtime_func
                });
            }
            //
            // adding onclick events
            for (j = 0; j < args.link_onclick.length; j++) {
                func_obj = args.link_onclick[j];
                runtime_func = build_runtime_function(func_obj, page_arr[i]);
                page_arr[i].events.push({ event: 'click', function: runtime_func });
            }
        }
        //
        page_nav.add_children(page_arr);
        return page_nav;
    }
    //
    // recursively handles adding cells to lower teirs adjusting properties as needed
    function add_header_group_cell(tiers, group, col_meta_data, max_rowspan) {
        var disp_name = tiers.shift(), cell_obj;
        if (disp_name === undefined) {
            return;
        }
        //
        // incrementing colspan or adding a new cell to the group
        if (group.columns[disp_name]) {
            group.columns[disp_name].cell_attr.colSpan += 1;
            cell_obj = group.columns[disp_name];
        } else {
            cell_obj = Object.clone(col_meta_data);
            cell_obj.cell_attr.textNode = disp_name;
            cell_obj.cell_attr.colSpan = 1;
            cell_obj.cell_attr.rowSpan = max_rowspan;
            // I only want arrows and tooltips on the lowest row
            if (tiers.length !== 0) {
                cell_obj.cell_attr.rowSpan = 1;
                cell_obj.arrow_attr = null;
                cell_obj.tooltip_attr = null;
            }
            cell_obj.group = { columns: {}, order: [] };
            group.columns[disp_name] = cell_obj;
            group.order.push(cell_obj);
        }
        add_header_group_cell(tiers, cell_obj.group, col_meta_data, max_rowspan - 1);
    }
    //
    // determines which level a cell pertains to
    function generate_header_groups(meta_data, num_rows) {
        var group = { columns: {}, order: [] }, tiers, i;
        //
        // creating nested objects for row generation
        for (i = 0; i < meta_data.length; i++) {
            tiers = meta_data[i].display_name.split('-');
            add_header_group_cell(tiers, group, meta_data[i], num_rows);
        }
        //
        return group;
    }
    //
    // adds a set of cells to a row
    function add_header_row_cells(rows, group, row_attr, level) {
        var col_obj, i;
        //
        // creating row if needed
        if (rows[level] === undefined) {
            rows[level] = Object.clone(row_attr);
        }
        //
        // adding cells to row
        for (i = 0; i < group.order.length; i++) {
            //
            // adding objects for current level
            col_obj = group.order[i];
            if (col_obj.arrow_attr) {
                col_obj.cell_attr.children.push(col_obj.arrow_attr);
            }
            if (col_obj.tooltip_attr) {
                col_obj.cell_attr.children.push(col_obj.tooltip_attr);
            }
            rows[level].children.push(col_obj.cell_attr);
            //
            // checking for cells in a lower level
            if (col_obj.group.order.length > 0) {
                add_header_row_cells(rows, col_obj.group, row_attr, level + 1);
            }
        }
    }
    //
    // processes the groups array into a suitable form for add_children
    function process_header_groups(group, row_attr) {
        var rows = [];
        row_attr.elm = 'TR';
        row_attr.children = [];
        //
        // adding cell objects to the row
        add_header_row_cells(rows, group, row_attr, 0);
        return rows;
    }
    //
    // creates the header rows for a table
    function make_head_rows(table, meta_data, args) {
        var default_args = AFW_HUB.get_constant('HEAD_ROW_DEFAULTS'),
            thead = document.createElement('THEAD'),
            build_runtime_function = AFW_HUB.Utilities.build_runtime_function,
            func_obj,
            runtime_func,
            num_rows = 1,
            groups,
            rows,
            i,
            j,
            n;
        args = Object.merge(default_args, args);
        meta_data = Object.clone(meta_data);
        //
        // modifying args if table is not sortable
        if (!args.sortable) {
            args.sort_onclick = [];
            args.asc_arrow = '';
            args.desc_arrow = '';
        }
        //
        // adding data to column objects for element generation
        for (i = 0; i < meta_data.length; i++) {
            //
            // initializing attribute objects and cell parameters
            meta_data[i].sort_dir = 'ASC';
            meta_data[i].cell_attr = {
                elm: 'TH',
                colSpan: 1,
                children: [],
                events: []
            };
            meta_data[i].arrow_attr = {
                elm: 'SPAN',
                style: { marginLeft: '0.5em' },
                textNode: args.asc_arrow,
                events: []
            };
            meta_data[i].tooltip_attr = Object.clone(args.tooltip);
            meta_data[i].tooltip_attr.events = [];
            meta_data[i].tooltip_attr.textNode = meta_data[i].tooltip_attr.textNode.format(
                meta_data[i]
            );
            //
            if (
                args.sort_col === meta_data[i].column_name && args.sort_dir === 'ASC'
            ) {
                meta_data[i].arrow_attr.textNode = args.desc_arrow;
                meta_data[i].sort_dir = 'DESC';
            }
            //
            // processing events
            for (j = 0; j < args.cell_onclick.length; j++) {
                func_obj = args.cell_onclick[j];
                runtime_func = build_runtime_function(func_obj, meta_data[i]);
                meta_data[i].cell_attr.events.push({
                    event: 'click',
                    function: runtime_func
                });
            }
            //
            for (j = 0; j < args.sort_onclick.length; j++) {
                func_obj = args.sort_onclick[j];
                runtime_func = build_runtime_function(func_obj, meta_data[i]);
                meta_data[i].arrow_attr.events.push({
                    event: 'click',
                    function: runtime_func
                });
            }
            //
            for (j = 0; j < args.tooltip.onclick.length; j++) {
                func_obj = args.tooltip.onclick[j];
                runtime_func = build_runtime_function(func_obj, meta_data[i]);
                meta_data[i].tooltip_attr.events.push({
                    event: 'click',
                    function: runtime_func
                });
            }
            //
            // assesing column display names for groups separated by a '-'
            n = 0;
            if (meta_data[i].display_name.match(/-/g)) {
                n = 1 + meta_data[i].display_name.match(/-/g).length;
            }
            if (n > num_rows) {
                num_rows = n;
            }
        }
        //
        // determining header row layout
        groups = generate_header_groups(meta_data, num_rows);
        rows = process_header_groups(groups, args.row_attr);
        //
        // adding header rows to THEAD and then appending THEAD to table
        thead.add_children(rows);
        table.appendChild(thead);
    }
    //
    // handles the extra processing options of numeric data for table display
    function display_numeric_data(value, data_type, element, args) {
        value = Number(value);
        element.style.textAlign = 'right';
        //
        // handling input pre-processing and rounding
        if (data_type.match(/(^|%)percent(%|$)/)) {
            value = 100 * Number(value);
            value = AFW_HUB.FormUtil.round_data_type(value, data_type);
            value += ' %';
        } else if (data_type.match(/(^|%)bool(%|$)/)) {
            value = Number(value);
            if (value > 0) {
                value = 'YES';
            } else {
                value = 'NO';
            }
        } else {
            value = AFW_HUB.FormUtil.round_data_type(value, data_type);
            value += '';
            if (args.add_commas) {
                value = value.replace(/\B(?=(\d{3})+(?!\d))/g, ',');
            }
        }
        //
        return value;
    }
    //
    // displays table data based on the column data type
    function display_table_data(value, data_type, element, args) {
        var attr,
            aux_elm,
            numeric = /int(%|$)|float(%|$)|percent(%|$)|monetary(%|$)|(^|%)round/i,
            MAX_STR_LENGTH = AFW_HUB.get_constant('MAX_STR_LENGTH');
        value = value === null ? '' : value;
        data_type = data_type || 'info';
        data_type = data_type.toLowerCase();
        //
        // pre-processing numeric data
        if (numeric.test(data_type) && isFinite(Number(value))) {
            value = display_numeric_data(value, data_type, element, args);
        }
        //
        // Applying format string to value
        value = args.format_str.replace(/%value%/, value);
        //
        // handling special cases where additional elements are created
        if (data_type.match(/(^|%)monetary(%|$)/)) {
            attr = {
                style: { display: 'table-cell', width: '100%', textAlign: 'left' }
            };
            aux_elm = document.createElementWithAttr('SPAN', attr);
            aux_elm.textContent = '$ ';
            element.appendChild(aux_elm);
            //
            attr = { style: { display: 'table-cell' } };
            aux_elm = document.createElementWithAttr('SPAN', attr);
            element.appendChild(aux_elm);
            element = aux_elm;
        } else if (
            data_type.match(/(?:^|%)text(?:%|$)/) && value.length > MAX_STR_LENGTH
        ) {
            element.style.textAlign = 'left';
            element.style.color = 'rgb(0,0,225)';
            element.addEventListener(
                'click',
                function(str1, str2) {
                    AFW_HUB.Utilities.toggle_textContent(this, str1, str2);
                }.bind(element, value.slice(0, MAX_STR_LENGTH) + '...', value)
            );
            //
            value = value.slice(0, MAX_STR_LENGTH) + '...';
        }
        //
        // ouputting textNode with display value
        element.addTextNode(value);
        return value;
    }
    //
    // handles construction of a single table row
    function build_table_row(tbody, args, meta_data, data) {
        var tr, td, i, prop, col, disp_args = args.data_display_args;
        //
        // creating row
        args.row_attr.elm = 'tr';
        tr = tbody.add_children([ args.row_attr ])[0];
        //
        // setting dataset values of the row
        for (prop in data) {
            try {
                tr.dataset[prop] = data[prop];
            } catch (err) {
                console.log('Property: ' + prop + ' could not be stored.');
            }
        }
        //
        for (i = 0; i < meta_data.length; i++) {
            col = meta_data[i];
            td = document.createElementWithAttr('TD', args.cell_attr);
            td.id = data.row_id + '-' + col.column_name;
            //
            display_table_data(data[col.column_name], col.data_type, td, disp_args);
            tr.appendChild(td);
        }
        //
        return tr;
    }
    //
    // generates the DOM nodes for the table
    function make_standard_table(args) {
        var meta_data = args.meta_data,
            data = args.data,
            table = document.createElementWithAttr('TABLE', args.table_attr),
            tbody = document.createElement('TBODY'),
            row_attr = args.row_attr,
            build_runtime_function = AFW_HUB.Utilities.build_runtime_function,
            i,
            j,
            func_obj,
            runtime_func;
        //
        // creating column head rows
        make_head_rows(table, args.meta_data, args.head_row_args);
        table.appendChild(tbody);
        //
        // building table rows
        for (i = args.start_index; i < args.end_index; i++) {
            data[i].row_id = args.row_id_prefix + i;
            row_attr.id = data[i].row_id;
            row_attr.events = [];
            //
            // adding onmouseenter events
            for (j = 0; j < args.row_onmouseenter.length; j++) {
                func_obj = args.row_onmouseenter[j];
                runtime_func = build_runtime_function(func_obj, data[i]);
                row_attr.events.push({
                    event: 'mouseenter',
                    function: runtime_func
                });
            }
            //
            // adding onmouseleave events
            for (j = 0; j < args.row_onmouseleave.length; j++) {
                func_obj = args.row_onmouseleave[j];
                runtime_func = build_runtime_function(func_obj, data[i]);
                row_attr.events.push({
                    event: 'mouseleave',
                    function: runtime_func
                });
            }
            //
            // adding onclick events
            for (j = 0; j < args.row_onclick.length; j++) {
                func_obj = args.row_onclick[j];
                runtime_func = build_runtime_function(func_obj, data[i]);
                row_attr.events.push({ event: 'click', function: runtime_func });
            }
            //
            build_table_row(tbody, args, meta_data, data[i]);
        }
        //
        return table;
    }
    //
    // function to generate a standard table using meta data
    function create_standard_table(table_args, response) {
        var page_nav,
            table,
            post_data,
            callback,
            url_data = AFW_HUB.Navigation.parse_url(),
            page = url_data.sub_tab || url_data.tab,
            output_elm = document.getElementById(table_args.table_output_id),
            regen_function = create_standard_table.bind(null, table_args, undefined),
            default_args = AFW_HUB.get_constant('STANDARD_TABLE_DEFAULTS');
        //
        // merging defaults
        table_args = Object.merge(default_args, table_args);
        default_args = AFW_HUB.get_constant('PAGE_NAV_DEFAULTS');
        table_args.page_nav_args = Object.merge(
            default_args,
            table_args.page_nav_args
        );
        default_args = AFW_HUB.get_constant('HEAD_ROW_DEFAULTS');
        table_args.head_row_args = Object.merge(
            default_args,
            table_args.head_row_args
        );
        //
        // fetching data
        if (response === undefined) {
            post_data = {
                database: table_args.database,
                display_cols: table_args.display_cols,
                data_sql_args: table_args.data_sql_args
            };
            //
            callback = create_standard_table.bind(null, table_args);
            fetch_table_data(table_args.table_name, post_data, callback);
            return;
        }
        //
        // adding data to table_args and callling pre-processor
        table_args.meta_data = response.meta_data;
        table_args.data = response.table_data;
        table_args.data_preprocessor(table_args, response);
        //
        // setting up page nav and data display indicies
        table_args.start_index = 0;
        table_args.end_index = table_args.data.length;
        table_args.page_nav_args.data_length = table_args.data.length;
        if (!table_args.no_page_nav) {
            page = table_args.page_nav_args.curr_page;
            table_args.start_index = (page - 1) *
                table_args.page_nav_args.num_per_page;
            table_args.end_index = page * table_args.page_nav_args.num_per_page;
            if (table_args.end_index > table_args.data.length) {
                table_args.end_index = table_args.data.length;
            }
            //
            page_nav = create_page_links(table_args.page_nav_args);
            output_elm.safeAppendChild(page_nav);
        }
        //
        // building table and adding "regenerate" event
        table = make_standard_table(table_args);
        if (!table_args.regen_function) {
            table_args.regen_function = regen_function;
        }
        table.addEventListener('regenerate', table_args.regen_function);
        //
        output_elm.safeAppendChild(table);
        table_args.add_callback(response, table);
    }
    //
    // creates the section row for the report
    function build_report_section_row(row_data, meta_data, args) {
        var tr = document.createElementWithAttr('TR', args.section_row_attr),
            cell_attr = Object.clone(args.section_cell_attr),
            td,
            attr;
        for (attr in cell_attr) {
            if (typeof cell_attr[attr] !== 'string') {
                continue;
            }
            cell_attr[attr] = cell_attr[attr].format(row_data);
        }
        cell_attr.colSpan = meta_data.length;
        td = document.createElementWithAttr('TD', cell_attr);
        td.textContent = cell_attr.textContent;
        tr.appendChild(td);
        //
        return tr;
    }
    //
    // creates a total row for the report
    function build_report_totals_row(total, totals_data, meta_data, args) {
        var tr = document.createElementWithAttr('TR', args.totals_row_attr),
            head_td = document.createElementWithAttr('TD', args.totals_cell_attr),
            td = document.createElementWithAttr('TD', args.totals_cell_attr),
            disp_args = args.data_display_args,
            col,
            total_type,
            value,
            i;
        tr.id = Math.floor(Math.random() * 1000000000);
        //
        // setting up first cell
        head_td.colSpan = 0;
        head_td.textContent = total.toTitleCase() + ' Total:';
        for (i = 1; i < meta_data.length; i++) {
            col = meta_data[i].column_name;
            total_type = args.total_types[col] || meta_data[i].total_type;
            if (total_type.toLowerCase() !== 'none') {
                break;
            }
            head_td.colSpan += 1;
        }
        tr.appendChild(head_td);
        //
        // outputting total cells
        for (i = head_td.colSpan; i < meta_data.length; i++) {
            td = td.cloneNode();
            col = meta_data[i].column_name;
            td.id = tr.id + '-' + col;
            total_type = args.total_types[col] || meta_data[i].total_type;
            if (total_type.toLowerCase() === 'sum') {
                value = totals_data[total + '_total'][col];
            } else if (total_type.toLowerCase() === 'avg') {
                value = totals_data[total + '_total'][col];
                value = value / totals_data[total + '_count'][col];
            } else if (total_type.toLowerCase() === 'flag') {
                value = totals_data[total + '_total'][col] + '';
                value = value.trim() !== '' ? 'X' : '';
            } else {
                value = '';
            }
            col = meta_data[i];
            display_table_data(value, col.data_type, td, disp_args);
            tr.appendChild(td);
        }
        //
        // adding totals to column dataset
        for (col in totals_data[total + '_total']) {
            tr.dataset[col] = totals_data[total + '_total'][col];
        }
        //
        // re-initializing totals
        for (col in totals_data[total + '_total']) {
            totals_data[total + '_count'][col] = 0;
            totals_data[total + '_total'][col] = 0;
        }
        //
        args.totals_rows.push(tr);
        return tr;
    }
    //
    // handles generation of the standard report
    function make_standard_report(args) {
        var meta_data = args.meta_data,
            data = args.data,
            table = document.createElementWithAttr('TABLE', args.table_attr),
            tbody = document.createElement('TBODY'),
            spacer_td = document.createElement('TD'),
            totals = {
                section_count: {},
                section_total: {},
                overall_count: {},
                overall_total: {}
            },
            sect_col = args.section_column,
            tr,
            td,
            sect_value,
            col,
            val,
            i;
        spacer_td.style.border = 'none';
        spacer_td.style.width = '1em';
        args.section_rows = [];
        args.totals_rows = [];
        //
        // creating column header rows
        if (args.create_section_headers) {
            meta_data.unshift({ db_table_col: '', display_name: '' });
        }
        make_head_rows(table, meta_data, args.head_row_args);
        if (args.create_section_headers) {
            meta_data.shift();
            tr = table.firstChild.firstChild;
            tr.firstChild.style.border = 'none';
            tr.firstChild.style.width = '1em';
        }
        //
        // returning early if there isn't any data
        table.appendChild(tbody);
        if (data.length === 0) {
            tr = document.createElement('TR');
            td = document.createElementWithAttr('TD', {
                colSpan: meta_data.length + 1
            });
            td.textContent = '***** No Data for Selected Report Parameters *****';
            tr.appendChild(td);
            tbody.appendChild(tr);
            return table;
        }
        //
        // initializing totals
        for (col in data[0]) {
            totals.section_count[col] = 0;
            totals.section_total[col] = 0;
            totals.overall_count[col] = 0;
            totals.overall_total[col] = 0;
        }
        //
        // building the report body
        for (i = 0; i < data.length; i++) {
            data[i].row_id = 'report';
            if (sect_value !== data[i][sect_col] && args.create_section_headers) {
                if (args.create_totals_row && i !== 0) {
                    tr = build_report_totals_row('section', totals, meta_data, args);
                    tr.insertBefore(spacer_td.cloneNode(), tr.firstChild);
                    tbody.appendChild(tr);
                }
                //
                sect_value = data[i][sect_col];
                tr = build_report_section_row(data[i], meta_data, args);
                args.section_rows.push(tr);
                tbody.appendChild(tr);
            }
            //
            // building a row and adding the spacer cell
            if (args.create_data_rows) {
                args.row_attr = args.data_row_attr;
                tr = build_table_row(tbody, args, meta_data, data[i]);
                if (args.create_section_headers) {
                    tr.insertBefore(spacer_td.cloneNode(), tr.firstChild);
                }
            }
            //
            // incrementing totals
            for (col in data[i]) {
                val = data[i][col];
                val = isFinite(Number(val)) ? Number(val) : val;
                //
                totals.section_count[col] += 1;
                totals.section_total[col] += val;
                totals.overall_count[col] += 1;
                totals.overall_total[col] += val;
            }
        }
        //
        // outputting last section total and overall total
        if (args.create_totals_row && args.create_section_headers) {
            tr = build_report_totals_row('section', totals, meta_data, args);
            tr.insertBefore(spacer_td.cloneNode(), tr.firstChild);
            tbody.appendChild(tr);
        }
        if (args.create_totals_row) {
            tr = build_report_totals_row('overall', totals, meta_data, args);
            if (args.create_section_headers) {
                tr.insertBefore(spacer_td.cloneNode(), tr.firstChild);
            }
            tbody.appendChild(tr);
        }
        //
        return table;
    }
    //
    // generates a standard report with section heading and totals
    function create_standard_report(report_args, response) {
        var table,
            post_data,
            callback,
            output_elm = document.getElementById(report_args.table_output_id),
            default_args = AFW_HUB.get_constant('STANDARD_REPORT_DEFAULTS');
        //
        // merging defaults
        report_args = Object.merge(default_args, report_args);
        default_args = AFW_HUB.get_constant('HEAD_ROW_DEFAULTS');
        report_args.head_row_args = Object.merge(
            default_args,
            report_args.head_row_args
        );
        //
        // fetching data
        if (response === undefined) {
            post_data = {
                database: report_args.database,
                display_cols: report_args.display_cols,
                data_sql_args: report_args.data_sql_args
            };
            //
            callback = create_standard_report.bind(null, report_args);
            fetch_table_data(report_args.table_name, post_data, callback);
            return;
        }
        //
        // adding data to table_args and callling pre-processor
        report_args.meta_data = response.meta_data;
        report_args.data = response.table_data;
        report_args.data_preprocessor(report_args, response);
        //
        table = make_standard_report(report_args);
        output_elm.safeAppendChild(table);
        report_args.add_callback(report_args, table);
    }
    //
    // creates a display column selection table for tables and reports
    function make_column_selection_table(table_args) {
        var table = document.createElementWithAttr('TABLE', table_args.table_attr),
            tr = document.createElement('TR'),
            td = document.createElementWithAttr('TD', {
                style: { textAlign: 'center' }
            }),
            input = document.createElement('INPUT'),
            column_pattern,
            meta_data,
            i;
        //
        // creating header row with spacer
        meta_data = [ { column_name: ' ', display_name: ' ' } ];
        meta_data = meta_data.concat(table_args.meta_data);
        make_head_rows(table, meta_data, table_args.head_row_args);
        //
        // setting up column name pattern
        if (table_args.selected_cols[0] === '*') {
            column_pattern = /.*/;
        } else {
            column_pattern = table_args.selected_cols.join('$)|(^');
            column_pattern = '(^{}$)'.format(column_pattern);
            column_pattern = new RegExp(column_pattern);
        }
        //
        // creating selection row
        tr = tr.cloneNode();
        td = td.cloneNode();
        td.textContent = 'Show Column:';
        tr.appendChild(td);
        for (i = 1; i < meta_data.length; i++) {
            td = td.cloneNode();
            input = input.cloneNode();
            input.type = 'checkbox';
            input.value = meta_data[i].db_table_col;
            input.dataset.action = 'display_cols';
            input.dataset.column_name = meta_data[i].db_table_col;
            input.checked = column_pattern.test(meta_data[i].db_table_col);
            table_args.add_event_handlers(input);
            td.appendChild(input);
            tr.appendChild(td);
        }
        table.appendChild(tr);
        //
        // creating sorting row
        if (table_args.create_sort_row) {
            tr = tr.cloneNode();
            td = td.cloneNode();
            td.textContent = 'Sort By:';
            tr.appendChild(td);
            for (i = 1; i < meta_data.length; i++) {
                td = td.cloneNode();
                tr.appendChild(td);
                if (meta_data[i].column_type === 'js-generated') {
                    continue;
                }
                input = input.cloneNode();
                td.appendChild(input);
                input.type = 'radio';
                input.name = 'sort-col';
                input.value = meta_data[i].db_table_col;
                input.checked = false;
                input.dataset.action = 'sort_col';
                input.dataset.column_name = meta_data[i].db_table_col;
                table_args.add_event_handlers(input);
            }
            table.appendChild(tr);
        }
        //
        return table;
    }
    //
    // Queries database to generate display table and output it to the DOM
    function create_column_selection_table(table_args, response) {
        var table,
            post_data,
            callback,
            output_elm = document.getElementById(table_args.table_output_id);
        table_args.head_row_args = { sortable: false };
        //
        // fetching data
        if (response === undefined) {
            post_data = {
                meta_data_only: 1,
                database: table_args.database,
                display_cols: table_args.display_cols,
                data_sql_args: table_args.data_sql_args
            };
            //
            callback = create_column_selection_table.bind(null, table_args);
            fetch_table_data(table_args.table_name, post_data, callback);
            return;
        }
        //
        // building and outputting table
        table_args.meta_data = response.meta_data;
        table = make_column_selection_table(table_args);
        output_elm.safeAppendChild(table);
        //
        table_args.add_callback(table, table_args);
    }
    //
    // processes a column selection table pulling values of the inputs into an object
    function process_column_selection_table(table) {
        var inputs = table.querySelectorAll('INPUT'),
            display_params = { column_data: {} },
            i,
            col_name,
            action;
        //
        // looping through inputs populating arrays based on action dataset attribute
        for (i = 0; i < inputs.length; i++) {
            col_name = inputs[i].dataset.column_name;
            action = inputs[i].dataset.action;
            //
            // ensuring arrays always exist, even if empty
            if (!display_params[action]) {
                display_params[action] = [];
            }
            if (!display_params.column_data[col_name]) {
                display_params.column_data[col_name] = {};
            }
            //
            // ignoring unselected values
            if (!inputs[i].checked && inputs[i].getAttribute('checked') !== 'true') {
                continue;
            }
            //
            // appending to action specific array
            display_params[action].push(inputs[i].value);
            //
            // adding attributes to a general 'data' object for column
            display_params.column_data[col_name][action] = inputs[i].value;
        }
        //
        return display_params;
    }
    //
    // returning interface
    return {
        fetch_table_data: fetch_table_data,
        fetch_preset_data: fetch_preset_data,
        get_table_inputs: get_table_inputs,
        display_table_data: display_table_data,
        create_standard_table: create_standard_table,
        create_standard_report: create_standard_report,
        create_column_selection_table: create_column_selection_table,
        process_column_selection_table: process_column_selection_table
    };
}();
//
// setting exports for nodejs
var module = module || {};
module.exports = AFW_HUB.TableUtil;
